@@ -34,7 +34,7 @@
 
 #include <cstdio>
 
-#include "filesystem.h"
+#include "OpenImageIO/filesystem.h"
 
 #include "CineonStream.h"
 
