@@ -53,7 +53,7 @@ public:
     DPXOutput ();
     virtual ~DPXOutput ();
     virtual const char * format_name (void) const { return "dpx"; }
-    virtual bool supports (const std::string &feature) const {
+    virtual int supports (string_view feature) const {
         if (feature == "multiimage"
             || feature == "alpha"
             || feature == "nchannels"
