@@ -50,6 +50,20 @@ OIIO_PLUGIN_EXPORTS_END
 
 
 
+int
+FitsOutput::supports (string_view feature) const
+{
+    return (feature == "multiimage"
+         || feature == "alpha"
+         || feature == "nchannels"
+         || feature == "random_access"
+         || feature == "arbitrary_metadata"
+         || feature == "exif"   // Because of arbitrary_metadata
+         || feature == "iptc"); // Because of arbitrary_metadata
+}
+
+
+
 bool
 FitsOutput::open (const std::string &name, const ImageSpec &spec,
                   OpenMode mode)
@@ -148,19 +162,6 @@ FitsOutput::write_tile (int x, int y, int z, TypeDesc format,
 
 
 
-bool
-FitsOutput::supports (const std::string &feature) const
-{
-    // for now we only supports IMAGE extensions
-    if (feature == "multiimage")
-        return true;
-    if (feature == "random_access")
-        return true;
-    return false;
-}
-
-
-
 bool
 FitsOutput::close (void)
 {
