@@ -191,7 +191,8 @@ GIFInput::read_gif_extension (int ext_code, GifByteType *ext,
 
         int delay = (ext[3] << 8) | ext[2];
         if (delay) {
-            newspec.attribute ("gif:DelayMs", delay * 10);
+            newspec.attribute ("FramesPerSecond", float(100.0f/delay));
+            newspec.attribute ("oiio:Movie", 1);
         }
         
     } else if (ext_code == COMMENT_EXT_FUNC_CODE) {
@@ -218,6 +219,7 @@ GIFInput::read_subimage_metadata (ImageSpec &newspec)
     newspec.nchannels = 4;
     newspec.default_channel_names ();
     newspec.alpha_channel = 4;
+    newspec.attribute ("oiio:ColorSpace", "sRGB");
 
     m_previous_disposal_method = m_disposal_method;
     m_disposal_method = DISPOSAL_UNSPECIFIED;
@@ -394,6 +396,10 @@ GIFInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
 
     newspec.width = m_gif_file->SWidth;
     newspec.height = m_gif_file->SHeight;
+    newspec.depth = 1;
+    newspec.full_height = newspec.height;
+    newspec.full_width = newspec.width;
+    newspec.full_depth = newspec.depth;
 
     m_spec = newspec;
     m_subimage = subimage;
