@@ -33,9 +33,9 @@
 #include <cstdio>
 #include <iostream>
 
-#include "imageio.h"
-#include "filesystem.h"
-#include "fmath.h"
+#include "OpenImageIO/imageio.h"
+#include "OpenImageIO/filesystem.h"
+#include "OpenImageIO/fmath.h"
 #include "rgbe.h"
 
 
