@@ -49,14 +49,13 @@ OIIO_PLUGIN_EXPORTS_END
 
 
 
-bool
-IffOutput::supports (const std::string &feature) const
+int
+IffOutput::supports (string_view feature) const
 {
-    if (feature == "tiles")
-        return true;
-
-    // Everything else, we either don't support or don't know about
-    return false;
+    return (feature == "tiles"
+         || feature == "alpha"
+         || feature == "nchannels"
+        );
 }
 
 
@@ -121,7 +120,7 @@ IffOutput::open (const std::string &name, const ImageSpec &spec,
     // currently only RGB RLE compression is supported, we default to RLE
     // as Maya does not handle non-compressed IFF's very well.
     m_iff_header.compression =
-    (m_spec.get_string_attribute ("compression", "none") == std::string("none")) ? NONE : RLE;
+        (m_spec.get_string_attribute("compression") == "none") ? NONE : RLE;
 
     // we write the header of the file
     m_iff_header.x = m_spec.x;
@@ -270,8 +269,7 @@ IffOutput::close (void)
                 length += 8;
               
                 // tile compression.
-                bool tile_compress = 
-                (m_spec.get_string_attribute ("compression", "none") == std::string("none")) ? NONE : RLE;
+                bool tile_compress = (m_iff_header.compression == RLE);
                 
                 // set bytes.
                 std::vector<uint8_t> scratch;
@@ -485,8 +483,8 @@ IffOutput::close (void)
                                         swap_endian (&pixel);
                                     }
                                     // set pixel
-                                    *out_p++ = pixel;
-                                    (*out_p)++; // avoid gcc4.x warning
+                                    *out_p++ = pixel & 0xff;
+                                    *out_p++ = pixel >> 8;
                                 }
                             }
                         }
