@@ -43,8 +43,7 @@
 
 #include <OpenEXR/ImathVec.h>       /* because we need V3f */
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
 
 // Forward declaration
 namespace pvt {
@@ -98,6 +97,8 @@ public:
         WrapClamp,          ///< Clamp to [0..1]
         WrapPeriodic,       ///< Periodic mod 1
         WrapMirror,         ///< Mirror the image
+        WrapPeriodicPow2,   ///< Periodic, but only for powers of 2!!!
+        WrapPeriodicSharedBorder,  ///< Periodic with shared border (env)
         WrapLast            ///< Mark the end -- don't use this!
     };
 
@@ -124,17 +125,16 @@ public:
     /// Create a TextureOpt with all fields initialized to reasonable
     /// defaults.
     TextureOpt () 
-        : nchannels(1), firstchannel(0), subimage(0),
+        : firstchannel(0), subimage(0),
         swrap(WrapDefault), twrap(WrapDefault),
         mipmode(MipModeDefault), interpmode(InterpSmartBicubic),
         anisotropic(32), conservative_filter(true),
         sblur(0.0f), tblur(0.0f), swidth(1.0f), twidth(1.0f),
         fill(0.0f), missingcolor(NULL),
-        dresultds(NULL), dresultdt(NULL),
-        time(0.0f), bias(0.0f), samples(1),
-        rwrap(WrapDefault), rblur(0.0f), rwidth(1.0f), dresultdr(NULL),
-        actualchannels(0),
-        swrap_func(NULL), twrap_func(NULL), rwrap_func(NULL),
+        // dresultds(NULL), dresultdt(NULL),
+        time(0.0f), // bias(0.0f), samples(1),
+        rwrap(WrapDefault), rblur(0.0f), rwidth(1.0f), // dresultdr(NULL),
+        // actualchannels(0),
         envlayout(0)
     { }
 
@@ -142,7 +142,6 @@ public:
     ///
     TextureOpt (const TextureOptions &opt, int index);
 
-    int nchannels;            ///< Number of channels to look up
     int firstchannel;         ///< First channel of the lookup
     int subimage;             ///< Subimage or face ID
     ustring subimagename;     ///< Subimage name
@@ -156,8 +155,6 @@ public:
     float swidth, twidth;     ///< Multiplier for derivatives
     float fill;               ///< Fill value for missing channels
     const float *missingcolor;///< Color for missing texture
-    float *dresultds;         ///< Deriv of the result along s (if not NULL)
-    float *dresultdt;         ///< Deriv of the result along t (if not NULL)
     float time;               ///< Time (for time-dependent texture lookups)
     float bias;               ///< Bias for shadows
     int   samples;            ///< Number of samples for shadows
@@ -166,7 +163,6 @@ public:
     Wrap rwrap;               ///< Wrap mode in the r direction
     float rblur;              ///< Blur amount in the r direction
     float rwidth;             ///< Multiplier for derivatives in r direction
-    float *dresultdr;         ///< Deriv of the result along r (if not NULL)
 
     /// Utility: Return the Wrap enum corresponding to a wrap name:
     /// "default", "black", "clamp", "periodic", "mirror".
@@ -183,9 +179,6 @@ public:
 private:
     // Options set INTERNALLY by libtexture after the options are passed
     // by the user.  Users should not attempt to alter these!
-    int actualchannels;    // True number of channels read
-    typedef bool (*wrap_impl) (int &coord, int origin, int width);
-    wrap_impl swrap_func, twrap_func, rwrap_func;
     int envlayout;    // Layout for environment wrap
     friend class pvt::TextureSystemImpl;
 };
@@ -208,6 +201,8 @@ public:
         WrapClamp,          ///< Clamp to [0..1]
         WrapPeriodic,       ///< Periodic mod 1
         WrapMirror,         ///< Mirror the image
+        WrapPeriodicPow2,   ///< Periodic, but only for powers of 2!!!
+        WrapPeriodicSharedBorder,  ///< Periodic with shared border (env)
         WrapLast            ///< Mark the end -- don't use this!
     };
 
@@ -240,7 +235,6 @@ public:
 
     // Options that must be the same for all points we're texturing at once
     int firstchannel;         ///< First channel of the lookup
-    int nchannels;            ///< Number of channels to look up
     int subimage;             ///< Subimage or face ID
     ustring subimagename;     ///< Subimage name
     Wrap swrap;               ///< Wrap mode in the s direction
@@ -258,14 +252,11 @@ public:
     VaryingRef<float> fill;           ///< Fill value for missing channels
     VaryingRef<float> missingcolor;   ///< Color for missing texture
     VaryingRef<int>   samples;        ///< Number of samples
-    float *dresultds;                 ///< Deriv of the result along s (if not NULL)
-    float *dresultdt;                 ///< Deriv of the result along t (if not NULL)
 
     // For 3D volume texture lookups only:
     Wrap rwrap;                ///< Wrap mode in the r direction
     VaryingRef<float> rblur;   ///< Blur amount in the r direction
     VaryingRef<float> rwidth;  ///< Multiplier for derivatives in r direction
-    float *dresultdr;          ///< Deriv of the result along r (if not NULL)
 
     /// Utility: Return the Wrap enum corresponding to a wrap name:
     /// "default", "black", "clamp", "periodic", "mirror".
@@ -290,9 +281,6 @@ public:
 private:
     // Options set INTERNALLY by libtexture after the options are passed
     // by the user.  Users should not attempt to alter these!
-    int actualchannels;    // True number of channels read
-    typedef bool (*wrap_impl) (int &coord, int origin, int width);
-    wrap_impl swrap_func, twrap_func, rwrap_func;
     friend class pvt::TextureSystemImpl;
     friend class TextureOpt;
 };
@@ -310,8 +298,8 @@ public:
     /// freed by passing it to TextureSystem::destroy()!
     ///
     /// If shared==true, it's intended to be shared with other like-minded
-    /// owners in the same process who also ask for a shared cache.  If
-    /// false, a private image cache will be created.
+    /// owners in the same process who also ask for a shared texture system.
+    /// If false, a private texture system and cache will be created.
     static TextureSystem *create (bool shared=true);
 
     /// Destroy a TextureSystem that was created using
@@ -346,6 +334,7 @@ public:
     ///     int failure_retries : how many times to retry a read failure
     ///     int deduplicate : if nonzero, detect duplicate textures (default=1)
     ///     int gray_to_rgb : make 1-channel images fill RGB lookups
+    ///     int max_tile_channels : max channels to store all chans in a tile
     ///     string latlong_up : default "up" direction for latlong ("y")
     ///
     virtual bool attribute (string_view name, TypeDesc type, const void *val) = 0;
@@ -356,22 +345,39 @@ public:
     virtual bool attribute (string_view name, string_view val) = 0;
 
     /// Get the named attribute, store it in value.
-    virtual bool getattribute (string_view name, TypeDesc type, void *val) = 0;
+    virtual bool getattribute (string_view name,
+                               TypeDesc type, void *val) const = 0;
     // Shortcuts for common types
-    virtual bool getattribute (string_view name, int &val) = 0;
-    virtual bool getattribute (string_view name, float &val) = 0;
-    virtual bool getattribute (string_view name, double &val) = 0;
-    virtual bool getattribute (string_view name, char **val) = 0;
-    virtual bool getattribute (string_view name, std::string &val) = 0;
+    virtual bool getattribute (string_view name, int &val) const = 0;
+    virtual bool getattribute (string_view name, float &val) const = 0;
+    virtual bool getattribute (string_view name, double &val) const = 0;
+    virtual bool getattribute (string_view name, char **val) const = 0;
+    virtual bool getattribute (string_view name, std::string &val) const = 0;
 
     /// Define an opaque data type that allows us to have a pointer
     /// to certain per-thread information that the TextureSystem maintains.
+    /// Any given one of these should NEVER be shared between running
+    /// threads.
     class Perthread;
 
-    /// Retrieve an opaque handle for per-thread info, to be used for
-    /// get_texture_handle and the texture routines that take handles
-    /// directly.
-    virtual Perthread * get_perthread_info () = 0;
+    /// Retrieve a Perthread, unique to the calling thread. This is a
+    /// thread-specific pointer that will always return the Perthread for a
+    /// thread, which will also be automatically destroyed when the thread
+    /// terminates.
+    ///
+    /// Applications that want to manage their own Perthread pointers (with
+    /// create_thread_info and destroy_thread_info) should still call this,
+    /// but passing in their managed pointer. If the passed-in threadinfo is
+    /// not NULL, it won't create a new one or retrieve a TSP, but it will
+    /// do other necessary housekeeping on the Perthread information.
+    virtual Perthread * get_perthread_info (Perthread *thread_info=NULL) = 0;
+
+    /// Create a new Perthread. It is the caller's responsibility to
+    /// eventually destroy it using destroy_thread_info().
+    virtual Perthread * create_thread_info () = 0;
+
+    /// Destroy a Perthread that was allocated by create_thread_info().
+    virtual void destroy_thread_info (Perthread *threadinfo) = 0;
 
     /// Define an opaque data type that allows us to have a handle to a
     /// texture (already having its name resolved) but without exposing
@@ -379,12 +385,17 @@ public:
     class TextureHandle;
 
     /// Retrieve an opaque handle for fast texture lookups.  The opaque
-    /// point thread_info is thread-specific information returned by
+    /// pointer thread_info is thread-specific information returned by
     /// get_perthread_info().  Return NULL if something has gone
     /// horribly wrong.
     virtual TextureHandle * get_texture_handle (ustring filename,
                                             Perthread *thread_info=NULL) = 0;
 
+    /// Return true if the texture handle (previously returned by
+    /// get_texture_handle()) is a valid texture that can be subsequently
+    /// read or sampled.
+    virtual bool good (TextureHandle *texture_handle) = 0;
+
     /// Filtered 2D texture lookup for a single point.
     ///
     /// s,t are the texture coordinates; dsdx, dtdx, dsdy, and dtdy are
@@ -394,24 +405,27 @@ public:
     /// pixels in screen space, adjacent samples in parameter space on a
     /// surface, etc.
     ///
+    /// The result is placed in result[0..nchannels-1].  If dresuls and
+    /// dresultdt are non-NULL, then they [0..nchannels-1] will get the
+    /// texture gradients, i.e., the rate of change per unit s and t,
+    /// respectively, of the texture.
+    ///
     /// Return true if the file is found and could be opened by an
     /// available ImageIO plugin, otherwise return false.
     virtual bool texture (ustring filename, TextureOpt &options,
                           float s, float t, float dsdx, float dtdx,
-                          float dsdy, float dtdy, float *result) = 0;
+                          float dsdy, float dtdy,
+                          int nchannels, float *result,
+                          float *dresultds=NULL, float *dresultdt=NULL) = 0;
 
-    /// Slightly faster version of 2D texture() lookup if the app already
-    /// has a texture handle and per-thread info.
+    /// Version that takes nchannels and derivatives explicitly, if the app
+    /// already has a texture handle and per-thread info.
     virtual bool texture (TextureHandle *texture_handle,
                           Perthread *thread_info, TextureOpt &options,
                           float s, float t, float dsdx, float dtdx,
-                          float dsdy, float dtdy, float *result) = 0;
-
-    /// Deprecated version that uses old TextureOptions for a single-point
-    /// lookup.
-    virtual bool texture (ustring filename, TextureOptions &options,
-                          float s, float t, float dsdx, float dtdx,
-                          float dsdy, float dtdy, float *result) = 0;
+                          float dsdy, float dtdy,
+                          int nchannels, float *result,
+                          float *dresultds=NULL, float *dresultdt=NULL) = 0;
 
     /// Retrieve filtered (possibly anisotropic) texture lookups for
     /// several points at once.
@@ -429,7 +443,16 @@ public:
                           VaryingRef<float> s, VaryingRef<float> t,
                           VaryingRef<float> dsdx, VaryingRef<float> dtdx,
                           VaryingRef<float> dsdy, VaryingRef<float> dtdy,
-                          float *result) = 0;
+                          int nchannels, float *result,
+                          float *dresultds=NULL, float *dresultdt=NULL) = 0;
+    virtual bool texture (TextureHandle *texture_handle,
+                          Perthread *thread_info, TextureOptions &options,
+                          Runflag *runflags, int beginactive, int endactive,
+                          VaryingRef<float> s, VaryingRef<float> t,
+                          VaryingRef<float> dsdx, VaryingRef<float> dtdx,
+                          VaryingRef<float> dsdy, VaryingRef<float> dtdy,
+                          int nchannels, float *result,
+                          float *dresultds=NULL, float *dresultdt=NULL) = 0;
 
     /// Retrieve a 3D texture lookup at a single point.
     ///
@@ -438,7 +461,9 @@ public:
     virtual bool texture3d (ustring filename, TextureOpt &options,
                             const Imath::V3f &P, const Imath::V3f &dPdx,
                             const Imath::V3f &dPdy, const Imath::V3f &dPdz,
-                            float *result) = 0;
+                            int nchannels, float *result,
+                            float *dresultds=NULL, float *dresultdt=NULL,
+                            float *dresultdr=NULL) = 0;
 
     /// Slightly faster version of texture3d() lookup if the app already
     /// has a texture handle and per-thread info.
@@ -446,17 +471,9 @@ public:
                             Perthread *thread_info, TextureOpt &options,
                             const Imath::V3f &P, const Imath::V3f &dPdx,
                             const Imath::V3f &dPdy, const Imath::V3f &dPdz,
-                            float *result) = 0;
-
-    /// Deprecated
-    ///
-    virtual bool texture3d (ustring filename, TextureOptions &options,
-                            const Imath::V3f &P, const Imath::V3f &dPdx,
-                            const Imath::V3f &dPdy, const Imath::V3f &dPdz,
-                            float *result) {
-        TextureOpt opt (options, 0);
-        return texture3d (filename, opt, P, dPdx, dPdy, dPdz, result);
-    }
+                            int nchannels, float *result,
+                            float *dresultds=NULL, float *dresultdt=NULL,
+                            float *dresultdr=NULL) = 0;
 
     /// Retrieve a 3D texture lookup at many points at once.
     ///
@@ -468,7 +485,19 @@ public:
                             VaryingRef<Imath::V3f> dPdx,
                             VaryingRef<Imath::V3f> dPdy,
                             VaryingRef<Imath::V3f> dPdz,
-                            float *result) = 0;
+                            int nchannels, float *result,
+                            float *dresultds=NULL, float *dresultdt=NULL,
+                            float *dresultdr=NULL) = 0;
+    virtual bool texture3d (TextureHandle *texture_handle,
+                            Perthread *thread_info, TextureOptions &options,
+                            Runflag *runflags, int beginactive, int endactive,
+                            VaryingRef<Imath::V3f> P,
+                            VaryingRef<Imath::V3f> dPdx,
+                            VaryingRef<Imath::V3f> dPdy,
+                            VaryingRef<Imath::V3f> dPdz,
+                            int nchannels, float *result,
+                            float *dresultds=NULL, float *dresultdt=NULL,
+                            float *dresultdr=NULL) = 0;
 
     /// Retrieve a shadow lookup for a single position P.
     ///
@@ -476,14 +505,16 @@ public:
     /// available ImageIO plugin, otherwise return false.
     virtual bool shadow (ustring filename, TextureOpt &options,
                          const Imath::V3f &P, const Imath::V3f &dPdx,
-                         const Imath::V3f &dPdy, float *result) = 0;
+                         const Imath::V3f &dPdy, float *result,
+                         float *dresultds=NULL, float *dresultdt=NULL) = 0;
 
     /// Slightly faster version of shadow() lookup if the app already
     /// has a texture handle and per-thread info.
     virtual bool shadow (TextureHandle *texture_handle, Perthread *thread_info,
                          TextureOpt &options,
                          const Imath::V3f &P, const Imath::V3f &dPdx,
-                         const Imath::V3f &dPdy, float *result) = 0;
+                         const Imath::V3f &dPdy, float *result,
+                         float *dresultds=NULL, float *dresultdt=NULL) = 0;
 
     /// Retrieve a shadow lookup for position P at many points at once.
     ///
@@ -494,7 +525,16 @@ public:
                          VaryingRef<Imath::V3f> P,
                          VaryingRef<Imath::V3f> dPdx,
                          VaryingRef<Imath::V3f> dPdy,
-                         float *result) = 0;
+                         float *result,
+                         float *dresultds=NULL, float *dresultdt=NULL) = 0;
+    virtual bool shadow (TextureHandle *texture_handle, Perthread *thread_info,
+                         TextureOptions &options,
+                         Runflag *runflags, int beginactive, int endactive,
+                         VaryingRef<Imath::V3f> P,
+                         VaryingRef<Imath::V3f> dPdx,
+                         VaryingRef<Imath::V3f> dPdy,
+                         float *result,
+                         float *dresultds=NULL, float *dresultdt=NULL) = 0;
 
     /// Retrieve an environment map lookup for direction R.
     ///
@@ -502,14 +542,16 @@ public:
     /// available ImageIO plugin, otherwise return false.
     virtual bool environment (ustring filename, TextureOpt &options,
                               const Imath::V3f &R, const Imath::V3f &dRdx,
-                              const Imath::V3f &dRdy, float *result) = 0;
+                              const Imath::V3f &dRdy, int nchannels, float *result,
+                              float *dresultds=NULL, float *dresultdt=NULL) = 0;
 
     /// Slightly faster version of environment() lookup if the app already
     /// has a texture handle and per-thread info.
     virtual bool environment (TextureHandle *texture_handle,
                               Perthread *thread_info, TextureOpt &options,
                               const Imath::V3f &R, const Imath::V3f &dRdx,
-                              const Imath::V3f &dRdy, float *result) = 0;
+                              const Imath::V3f &dRdy, int nchannels, float *result,
+                              float *dresultds=NULL, float *dresultdt=NULL) = 0;
 
     /// Retrieve an environment map lookup for direction R, for many
     /// points at once.
@@ -521,7 +563,16 @@ public:
                               VaryingRef<Imath::V3f> R,
                               VaryingRef<Imath::V3f> dRdx,
                               VaryingRef<Imath::V3f> dRdy,
-                              float *result) = 0;
+                              int nchannels, float *result,
+                              float *dresultds=NULL, float *dresultdt=NULL) = 0;
+    virtual bool environment (TextureHandle *texture_handle,
+                              Perthread *thread_info, TextureOptions &options,
+                              Runflag *runflags, int beginactive, int endactive,
+                              VaryingRef<Imath::V3f> R,
+                              VaryingRef<Imath::V3f> dRdx,
+                              VaryingRef<Imath::V3f> dRdy,
+                              int nchannels, float *result,
+                              float *dresultds=NULL, float *dresultdt=NULL) = 0;
 
     /// Given possibly-relative 'filename', resolve it using the search
     /// path rules and return the full resolved filename.
@@ -533,6 +584,12 @@ public:
     /// doesn't match the type requested. or some other failure.
     virtual bool get_texture_info (ustring filename, int subimage,
                           ustring dataname, TypeDesc datatype, void *data) = 0;
+    virtual bool get_texture_info (TextureHandle *texture_handle,
+                          Perthread *thread_info, int subimage,
+                          ustring dataname, TypeDesc datatype, void *data) = 0;
+    /// DEPRECATED (1.6.2)
+    virtual bool get_texture_info (TextureHandle *texture_handle, int subimage,
+                          ustring dataname, TypeDesc datatype, void *data) = 0;
 
     /// Get the ImageSpec associated with the named texture
     /// (specifically, the first MIP-map level).  If the file is found
@@ -542,6 +599,9 @@ public:
     /// available ImageIO plugin.
     virtual bool get_imagespec (ustring filename, int subimage,
                                 ImageSpec &spec) = 0;
+    virtual bool get_imagespec (TextureHandle *texture_handle,
+                                Perthread *thread_info, int subimage,
+                                ImageSpec &spec) = 0;
 
     /// Return a pointer to an ImageSpec associated with the named
     /// texture (specifically, the first MIP-map level) if the file is
@@ -556,6 +616,9 @@ public:
     /// calls invalidate() on the file, or invalidate_all(), or destroys
     /// the TextureSystem.
     virtual const ImageSpec *imagespec (ustring filename, int subimage=0) = 0;
+    virtual const ImageSpec *imagespec (TextureHandle *texture_handle,
+                                        Perthread *thread_info = NULL,
+                                        int subimage=0) = 0;
 
     /// Retrieve the rectangle of raw unfiltered texels spanning
     /// [xbegin..xend) X [ybegin..yend) X [zbegin..zend), with
@@ -574,18 +637,14 @@ public:
     virtual bool get_texels (ustring filename, TextureOpt &options,
                              int miplevel, int xbegin, int xend,
                              int ybegin, int yend, int zbegin, int zend,
+                             int chbegin, int chend,
                              TypeDesc format, void *result) = 0;
-
-    /// Deprecated
-    ///
-    virtual bool get_texels (ustring filename, TextureOptions &options,
+    virtual bool get_texels (TextureHandle *texture_handle,
+                             Perthread *thread_info, TextureOpt &options,
                              int miplevel, int xbegin, int xend,
                              int ybegin, int yend, int zbegin, int zend,
-                             TypeDesc format, void *result) {
-        TextureOpt opt (options, 0);
-        return get_texels (filename, opt, miplevel, xbegin, xend,
-                           ybegin, yend, zbegin, zend, format, result);
-    }
+                             int chbegin, int chend,
+                             TypeDesc format, void *result) = 0;
 
     /// If any of the API routines returned false indicating an error,
     /// this routine will return the error string (and clear any error
@@ -624,7 +683,6 @@ private:
 };
 
 
-}
-OIIO_NAMESPACE_EXIT
+OIIO_NAMESPACE_END
 
 #endif // OPENIMAGEIO_TEXTURE_H
