@@ -39,47 +39,43 @@
 #ifndef OPENIMAGEIO_THREAD_H
 #define OPENIMAGEIO_THREAD_H
 
+#include <vector>
+
 #include "oiioversion.h"
-#include "sysutil.h"
+#include "platform.h"
 
 
-// defining NOMINMAX to prevent problems with std::min/std::max
-// and std::numeric_limits<type>::min()/std::numeric_limits<type>::max()
-// when boost include windows.h
-#ifdef _MSC_VER
-# define WIN32_LEAN_AND_MEAN
-# define VC_EXTRALEAN
-# ifndef NOMINMAX
-#   define NOMINMAX
+
+#if OIIO_CPLUSPLUS_VERSION >= 11
+# include <thread>
+# include <mutex>
+# include <atomic>
+# define not_yet_OIIO_USE_STDATOMIC 1
+#else   /* prior to C++11... */
+  // Use Boost mutexes & guards when C++11 is not available
+# include <boost/version.hpp>
+# if defined(__GNUC__) && (BOOST_VERSION == 104500)
+   // gcc reports errors inside some of the boost headers with boost 1.45
+   // See: https://svn.boost.org/trac/boost/ticket/4818
+#  pragma GCC diagnostic ignored "-Wunused-variable"
+# endif
+# include <boost/thread.hpp>
+# if defined(__GNUC__) && (BOOST_VERSION == 104500)
+   // can't restore via push/pop in all versions of gcc (warning push/pop implemented for 4.6+ only)
+#  pragma GCC diagnostic error "-Wunused-variable"
 # endif
 #endif
- 
-#include <boost/version.hpp>
-#if defined(__GNUC__) && (BOOST_VERSION == 104500)
-// gcc reports errors inside some of the boost headers with boost 1.45
-// See: https://svn.boost.org/trac/boost/ticket/4818
-#pragma GCC diagnostic ignored "-Wunused-variable"
-#endif
 
-#include <boost/thread.hpp>
-#include <boost/thread/tss.hpp>
-#include <boost/version.hpp>
-
-#if defined(__GNUC__) && (BOOST_VERSION == 104500)
-// can't restore via push/pop in all versions of gcc (warning push/pop implemented for 4.6+ only)
-#pragma GCC diagnostic error "-Wunused-variable"
-#endif
 
 #if defined(_MSC_VER)
-#  include <windows.h>
-#  include <winbase.h>
+   // N.B. including platform.h also included <windows.h>
 #  pragma intrinsic (_InterlockedExchangeAdd)
 #  pragma intrinsic (_InterlockedCompareExchange)
 #  pragma intrinsic (_InterlockedCompareExchange64)
 #  if defined(_WIN64)
 #    pragma intrinsic(_InterlockedExchangeAdd64)
 #  endif
-// InterlockedExchangeAdd64 is not available for XP
+// InterlockedExchangeAdd64 & InterlockedExchange64 are not available for XP
 #  if defined(_WIN32_WINNT) && _WIN32_WINNT <= 0x0501
 inline long long
 InterlockedExchangeAdd64 (volatile long long *Addend, long long Value)
@@ -90,11 +86,21 @@ InterlockedExchangeAdd64 (volatile long long *Addend, long long Value)
     } while (_InterlockedCompareExchange64(Addend, Old + Value, Old) != Old);
     return Old;
 }
+
+inline long long
+InterlockedExchange64 (volatile long long *Target, long long Value)
+{
+    long long Old;
+    do {
+        Old = *Target;
+    } while (_InterlockedCompareExchange64(Target, Value, Old) != Old);
+    return Old;
+}
 #  endif
 #endif
 
 #if defined(__GNUC__) && (defined(_GLIBCXX_ATOMIC_BUILTINS) || (__GNUC__ * 100 + __GNUC_MINOR__ >= 401))
-#define USE_GCC_ATOMICS
+#  define USE_GCC_ATOMICS
 #  if !defined(__clang__) && (__GNUC__ * 100 + __GNUC_MINOR__ >= 408)
 #    define OIIO_USE_GCC_NEW_ATOMICS
 #  endif
@@ -112,8 +118,19 @@ InterlockedExchangeAdd64 (volatile long long *Addend, long long Value)
 #endif
 
 
-OIIO_NAMESPACE_ENTER
-{
+
+// Some helpful links:
+//
+// Descriptions of the "new" gcc atomic intrinsics:
+//    https://gcc.gnu.org/onlinedocs/gcc/_005f_005fatomic-Builtins.html
+// Old gcc atomic intrinsics:
+//    https://gcc.gnu.org/onlinedocs/gcc-4.4.2/gcc/Atomic-Builtins.html
+// C++11 and beyond std::atomic:
+//    http://en.cppreference.com/w/cpp/atomic
+
+
+
+OIIO_NAMESPACE_BEGIN
 
 /// Null mutex that can be substituted for a real one to test how much
 /// overhead is associated with a particular mutex.
@@ -137,66 +154,24 @@ public:
 };
 
 
-// Null thread-specific ptr that just wraps a single ordinary pointer
-//
-template<typename T>
-class null_thread_specific_ptr {
-public:
-    typedef void (*destructor_t)(T *);
-    null_thread_specific_ptr (destructor_t dest=NULL)
-        : m_ptr(NULL), m_dest(dest) { }
-    ~null_thread_specific_ptr () { reset (NULL); }
-    T * get () { return m_ptr; }
-    void reset (T *newptr=NULL) {
-        if (m_ptr) {
-            if (m_dest)
-                (*m_dest) (m_ptr);
-            else
-                delete m_ptr;
-        }
-        m_ptr = newptr;
-    }
-private:
-    T *m_ptr;
-    destructor_t m_dest;
-};
-
 
 #ifdef NOTHREADS
 
 // Definitions that we use for debugging to turn off all mutexes, locks,
 // and atomics in order to test the performance hit of our thread safety.
 
-// Null thread-specific ptr that just wraps a single ordinary pointer
-//
-template<typename T>
-class thread_specific_ptr {
-public:
-    typedef void (*destructor_t)(T *);
-    thread_specific_ptr (destructor_t dest=NULL)
-        : m_ptr(NULL), m_dest(dest) { }
-    ~thread_specific_ptr () { reset (NULL); }
-    T * get () { return m_ptr; }
-    void reset (T *newptr=NULL) {
-        if (m_ptr) {
-            if (m_dest)
-                (*m_dest) (m_ptr);
-            else
-                delete m_ptr;
-        }
-        m_ptr = newptr;
-    }
-private:
-    T *m_ptr;
-    destructor_t m_dest;
-};
-
-
 typedef null_mutex mutex;
 typedef null_mutex recursive_mutex;
 typedef null_lock<mutex> lock_guard;
 typedef null_lock<recursive_mutex> recursive_lock_guard;
 
+#elif OIIO_CPLUSPLUS_VERSION >= 11
+
+typedef std::mutex mutex;
+typedef std::recursive_mutex recursive_mutex;
+typedef std::lock_guard< mutex > lock_guard;
+typedef std::lock_guard< recursive_mutex > recursive_lock_guard;
+typedef std::thread thread;
 
 #else
 
@@ -204,23 +179,48 @@ typedef null_lock<recursive_mutex> recursive_lock_guard;
 
 typedef boost::mutex mutex;
 typedef boost::recursive_mutex recursive_mutex;
-typedef boost::lock_guard< boost::mutex > lock_guard;
-typedef boost::lock_guard< boost::recursive_mutex > recursive_lock_guard;
-using boost::thread_specific_ptr;
+typedef boost::lock_guard< mutex > lock_guard;
+typedef boost::lock_guard< recursive_mutex > recursive_lock_guard;
+typedef boost::thread thread;
 
 #endif
 
 
 
+#if OIIO_USE_STDATOMIC
+using std::memory_order;
+#else
+enum memory_order {
+#if defined(OIIO_USE_GCC_NEW_ATOMICS)
+    memory_order_relaxed = __ATOMIC_RELAXED,
+    memory_order_consume = __ATOMIC_CONSUME,
+    memory_order_acquire = __ATOMIC_ACQUIRE,
+    memory_order_release = __ATOMIC_RELEASE,
+    memory_order_acq_rel = __ATOMIC_ACQ_REL,
+    memory_order_seq_cst = __ATOMIC_SEQ_CST
+#else
+    memory_order_relaxed,
+    memory_order_consume,
+    memory_order_acquire,
+    memory_order_release,
+    memory_order_acq_rel,
+    memory_order_seq_cst
+#endif
+};
+#endif
+
+
+
 /// Atomic version of:  r = *at, *at += x, return r
 /// For each of several architectures.
 inline int
-atomic_exchange_and_add (volatile int *at, int x)
+atomic_exchange_and_add (volatile int *at, int x,
+                         memory_order order = memory_order_seq_cst)
 {
 #ifdef NOTHREADS
     int r = *at;  *at += x;  return r;
 #elif defined(OIIO_USE_GCC_NEW_ATOMICS)
-    return __atomic_fetch_add (at, x, __ATOMIC_SEQ_CST);
+    return __atomic_fetch_add (at, x, order);
 #elif defined(USE_GCC_ATOMICS)
     return __sync_fetch_and_add ((int *)at, x);
 #elif defined(_MSC_VER)
@@ -234,12 +234,13 @@ atomic_exchange_and_add (volatile int *at, int x)
 
 
 inline long long
-atomic_exchange_and_add (volatile long long *at, long long x)
+atomic_exchange_and_add (volatile long long *at, long long x,
+                         memory_order order = memory_order_seq_cst)
 {
 #ifdef NOTHREADS
     long long r = *at;  *at += x;  return r;
 #elif defined(OIIO_USE_GCC_NEW_ATOMICS)
-    return __atomic_fetch_add (at, x, __ATOMIC_SEQ_CST);
+    return __atomic_fetch_add (at, x, order);
 #elif defined(USE_GCC_ATOMICS)
     return __sync_fetch_and_add (at, x);
 #elif defined(_MSC_VER)
@@ -263,7 +264,10 @@ atomic_exchange_and_add (volatile long long *at, long long x)
 ///        return false;
 ///    }
 inline bool
-atomic_compare_and_exchange (volatile int *at, int compareval, int newval)
+atomic_compare_and_exchange (volatile int *at, int compareval, int newval,
+                             bool weak = false,
+                             memory_order success = memory_order_seq_cst,
+                             memory_order failure = memory_order_seq_cst)
 {
 #ifdef NOTHREADS
     if (*at == compareval) {
@@ -272,8 +276,8 @@ atomic_compare_and_exchange (volatile int *at, int compareval, int newval)
         return false;
     }
 #elif defined(OIIO_USE_GCC_NEW_ATOMICS)
-    return __atomic_compare_exchange_n (at, &compareval, newval, false,
-                                        __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST);
+    return __atomic_compare_exchange_n (at, &compareval, newval, weak,
+                                        success, failure);
 #elif defined(USE_GCC_ATOMICS)
     return __sync_bool_compare_and_swap (at, compareval, newval);
 #elif defined(_MSC_VER)
@@ -286,7 +290,10 @@ atomic_compare_and_exchange (volatile int *at, int compareval, int newval)
 
 
 inline bool
-atomic_compare_and_exchange (volatile long long *at, long long compareval, long long newval)
+atomic_compare_and_exchange (volatile long long *at, long long compareval, long long newval,
+                             bool weak = false,
+                             memory_order success = memory_order_seq_cst,
+                             memory_order failure = memory_order_seq_cst)
 {
 #ifdef NOTHREADS
     if (*at == compareval) {
@@ -295,8 +302,8 @@ atomic_compare_and_exchange (volatile long long *at, long long compareval, long
         return false;
     }
 #elif defined(OIIO_USE_GCC_NEW_ATOMICS)
-    return __atomic_compare_exchange_n (at, &compareval, newval, false,
-                                        __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST);
+    return __atomic_compare_exchange_n (at, &compareval, newval, weak,
+                                        success, failure);
 #elif defined(USE_GCC_ATOMICS)
     return __sync_bool_compare_and_swap (at, compareval, newval);
 #elif defined(_MSC_VER)
@@ -311,12 +318,13 @@ atomic_compare_and_exchange (volatile long long *at, long long compareval, long
 /// Atomic version of:  r = *at, *at = x, return r
 /// For each of several architectures.
 inline int
-atomic_exchange (volatile int *at, int x)
+atomic_exchange (volatile int *at, int x,
+                 memory_order order = memory_order_seq_cst)
 {
 #ifdef NOTHREADS
     int r = *at;  *at = x;  return r;
 #elif defined(OIIO_USE_GCC_NEW_ATOMICS)
-    return __atomic_exchange_n (at, x, __ATOMIC_SEQ_CST);
+    return __atomic_exchange_n (at, x, order);
 #elif defined(USE_GCC_ATOMICS)
     // No __sync version of atomic exchange! Do it the hard way:
     while (1) {
@@ -336,12 +344,13 @@ atomic_exchange (volatile int *at, int x)
 
 
 inline long long
-atomic_exchange (volatile long long *at, long long x)
+atomic_exchange (volatile long long *at, long long x,
+                 memory_order order = memory_order_seq_cst)
 {
 #ifdef NOTHREADS
     long long r = *at;  *at = x;  return r;
 #elif defined(OIIO_USE_GCC_NEW_ATOMICS)
-    return __atomic_exchange_n (at, x, __ATOMIC_SEQ_CST);
+    return __atomic_exchange_n (at, x, order);
 #elif defined(USE_GCC_ATOMICS)
     // No __sync version of atomic exchange! Do it the hard way:
     while (1) {
@@ -364,6 +373,27 @@ atomic_exchange (volatile long long *at, long long x)
 
 
 
+/// Memory fence / synchronization barrier
+OIIO_FORCEINLINE void
+atomic_thread_fence (memory_order order = memory_order_seq_cst)
+{
+#ifdef NOTHREADS
+    // nothing
+#elif OIIO_USE_STDATOMIC
+    std::__atomic_thread_fence (order);
+#elif defined(OIIO_USE_GCC_NEW_ATOMICS)
+    __atomic_thread_fence (order);
+#elif defined(USE_GCC_ATOMICS)
+    __sync_synchronize ();
+#elif defined(__GNUC__) && (defined(__x86_64__) || defined(__i386__))
+    __asm__ __volatile__ ("":::"memory");
+#elif defined(_MSC_VER)
+    MemoryBarrier ();
+#else
+#   error No atomics on this platform.
+#endif
+}
+
 
 
 /// Yield the processor for the rest of the timeslice.
@@ -444,51 +474,68 @@ public:
 
     /// Retrieve value
     ///
-    T operator() () const { return atomic_exchange_and_add (&m_val, 0); }
+    T load (memory_order order = memory_order_seq_cst) const {
+        return atomic_exchange_and_add (&m_val, 0, order);
+    }
 
     /// Retrieve value
     ///
-    operator T() const { return atomic_exchange_and_add (&m_val, 0); }
+    T operator() () const { return load(); }
+
+    /// Retrieve value
+    ///
+    operator T() const { return load(); }
 
     /// Fast retrieval of value, no interchange, don't care about memory
-    /// fences.
+    /// fences. Use with extreme caution!
     T fast_value () const { return m_val; }
 
+    /// Assign new value, atomically.
+    void store (T x, memory_order order = memory_order_seq_cst) {
+        atomic_exchange (&m_val, x, order);
+    }
+
+    /// Atomic exchange
+    T exchange (T x, memory_order order = memory_order_seq_cst) {
+        return atomic_exchange (&m_val, x, order);
+    }
+
+    /// Atomic fetch-and-add: add x and return the old value.
+    T fetch_add (T x, memory_order order = memory_order_seq_cst) {
+        return atomic_exchange_and_add (&m_val, x, order);
+    }
+    /// Atomic fetch-and-subtract: subtract x and return the old value.
+    T fetch_sub (T x, memory_order order = memory_order_seq_cst) {
+        return atomic_exchange_and_add (&m_val, -x, order);
+    }
+
     /// Assign new value.
     ///
-    T operator= (T x) {
-        //incorrect? return (m_val = x);
-        while (1) {
-            T result = m_val;
-            if (atomic_compare_and_exchange (&m_val, result, x))
-                break;
-        }
-        return x;
-    }
+    T operator= (T x) { store(x); return x; }
 
     /// Pre-increment:  ++foo
     ///
-    T operator++ () { return atomic_exchange_and_add (&m_val, 1) + 1; }
+    T operator++ () { return fetch_add(1) + 1; }
 
     /// Post-increment:  foo++
     ///
-    T operator++ (int) {  return atomic_exchange_and_add (&m_val, 1); }
+    T operator++ (int) {  return fetch_add(1); }
 
     /// Pre-decrement:  --foo
     ///
-    T operator-- () {  return atomic_exchange_and_add (&m_val, -1) - 1; }
+    T operator-- () {  return fetch_sub(1) - 1; }
 
     /// Post-decrement:  foo--
     ///
-    T operator-- (int) {  return atomic_exchange_and_add (&m_val, -1); }
+    T operator-- (int) {  return fetch_sub(1); }
 
     /// Add to the value, return the new result
     ///
-    T operator+= (T x) { return atomic_exchange_and_add (&m_val, x) + x; }
+    T operator+= (T x) { return fetch_add(x) + x; }
 
     /// Subtract from the value, return the new result
     ///
-    T operator-= (T x) { return atomic_exchange_and_add (&m_val, -x) - x; }
+    T operator-= (T x) { return fetch_sub(x) - x; }
 
     bool bool_compare_and_swap (T compareval, T newval) {
         return atomic_compare_and_exchange (&m_val, compareval, newval);
@@ -786,7 +833,46 @@ typedef spin_rw_mutex::read_lock_guard spin_rw_read_lock;
 typedef spin_rw_mutex::write_lock_guard spin_rw_write_lock;
 
 
-}
-OIIO_NAMESPACE_EXIT
+
+/// Simple thread group class. This is just as good as boost::thread_group,
+/// for the limited functionality that we use.
+class thread_group {
+public:
+    thread_group () {}
+    ~thread_group () {
+        for (size_t i = 0, e = m_threads.size(); i < e; ++i)
+            delete m_threads[i];
+    }
+    void add_thread (thread *t) {
+        if (t) {
+            lock_guard lock (m_mutex);
+            m_threads.push_back (t);
+        }
+    }
+    template<typename FUNC>
+    thread *create_thread (FUNC func) {
+        lock_guard lock (m_mutex);
+        thread *t = new thread (func);
+        m_threads.push_back (t);
+        return t;
+    }
+    void join_all () {
+        lock_guard lock (m_mutex);
+        for (size_t i = 0, e = m_threads.size(); i < e; ++i) {
+            if (m_threads[i]->joinable())
+                m_threads[i]->join();
+        }
+    }
+    size_t size () {
+        lock_guard lock (m_mutex);
+        return m_threads.size();
+    }
+private:
+    mutex m_mutex;
+    std::vector<thread *> m_threads;
+};
+
+
+OIIO_NAMESPACE_END
 
 #endif // OPENIMAGEIO_THREAD_H
