@@ -29,15 +29,10 @@
   (This is the Modified BSD License)
 */
 
-
 #include <cassert>
 #include <cstdio>
 #include <vector>
 
-extern "C" {
-#include "jpeglib.h"
-}
-
 #include "OpenImageIO/imageio.h"
 #include "OpenImageIO/filesystem.h"
 #include "OpenImageIO/fmath.h"
@@ -57,7 +52,10 @@ class JpgOutput : public ImageOutput {
     JpgOutput () { init(); }
     virtual ~JpgOutput () { close(); }
     virtual const char * format_name (void) const { return "jpeg"; }
-    virtual bool supports (const std::string &property) const { return false; }
+    virtual int supports (string_view feature) const {
+        return (feature == "exif"
+             || feature == "iptc");
+    }
     virtual bool open (const std::string &name, const ImageSpec &spec,
                        OpenMode mode=Create);
     virtual bool write_scanline (int y, int z, TypeDesc format,
@@ -140,7 +138,7 @@ JpgOutput::open (const std::string &name, const ImageSpec &newspec,
 
     if (m_spec.nchannels != 1 && m_spec.nchannels != 3 &&
             m_spec.nchannels != 4) {
-        error ("%s does not support %d-channel images\n",
+        error ("%s does not support %d-channel images",
                format_name(), m_spec.nchannels);
         return false;
     }
@@ -166,9 +164,29 @@ JpgOutput::open (const std::string &name, const ImageSpec &newspec,
         m_cinfo.input_components = 1;
         m_cinfo.in_color_space = JCS_GRAYSCALE;
     }
-    m_cinfo.density_unit = 2; // RESUNIT_INCH;
-    m_cinfo.X_density = 72;
-    m_cinfo.Y_density = 72;
+
+    string_view resunit = m_spec.get_string_attribute ("ResolutionUnit");
+    if (Strutil::iequals (resunit, "none"))
+        m_cinfo.density_unit = 0;
+    else if (Strutil::iequals (resunit, "in"))
+        m_cinfo.density_unit = 1;
+    else if (Strutil::iequals (resunit, "cm"))
+        m_cinfo.density_unit = 2;
+    else
+        m_cinfo.density_unit = 0;
+    m_cinfo.X_density = int (m_spec.get_float_attribute ("XResolution"));
+    m_cinfo.Y_density = int (m_spec.get_float_attribute ("YResolution"));
+    float aspect = m_spec.get_float_attribute ("PixelAspectRatio", 1.0f);
+    if (m_cinfo.X_density <= 1 && m_cinfo.Y_density <= 1 && aspect != 1.0f) {
+        // No useful [XY]Resolution, but there is an aspect ratio requested.
+        // Arbitrarily pick 72 dots per undefined unit, and jigger it to
+        // honor it as best as we can.
+        m_cinfo.X_density = 72;
+        m_cinfo.Y_density = int (m_cinfo.X_density * aspect);
+        m_spec.attribute ("XResolution", 72.0f);
+        m_spec.attribute ("YResolution", 72.0f*aspect);
+    }
+
     m_cinfo.write_JFIF_header = TRUE;
 
     if (m_copy_coeffs) {
@@ -180,6 +198,9 @@ JpgOutput::open (const std::string &name, const ImageSpec &newspec,
     } else {
         // normal write of scanlines
         jpeg_set_defaults (&m_cinfo);                 // default compression
+        // Careful -- jpeg_set_defaults overwrites density
+        m_cinfo.X_density = int (m_spec.get_float_attribute ("XResolution"));
+        m_cinfo.Y_density = int (m_spec.get_float_attribute ("YResolution", m_cinfo.X_density));
         DBG std::cout << "out open: set_defaults\n";
         int quality = newspec.get_int_attribute ("CompressionQuality", 98);
         jpeg_set_quality (&m_cinfo, quality, TRUE);   // baseline values
@@ -257,6 +278,35 @@ JpgOutput::open (const std::string &name, const ImageSpec &newspec,
     }
 
     m_spec.set_format (TypeDesc::UINT8);  // JPG is only 8 bit
+
+    // Write ICC profile, if we have anything
+    const ImageIOParameter* icc_profile_parameter = m_spec.find_attribute(ICC_PROFILE_ATTR);
+    if (icc_profile_parameter != NULL) {
+        unsigned char *icc_profile = (unsigned char*)icc_profile_parameter->data();
+        unsigned int icc_profile_length = icc_profile_parameter->type().size();
+        if (icc_profile && icc_profile_length){
+            /* Calculate the number of markers we'll need, rounding up of course */
+            int num_markers = icc_profile_length / MAX_DATA_BYTES_IN_MARKER;
+            if ((unsigned int)(num_markers * MAX_DATA_BYTES_IN_MARKER) != icc_profile_length)
+                num_markers++;
+            int curr_marker = 1;  /* per spec, count strarts at 1*/
+            std::vector<unsigned char> profile (MAX_DATA_BYTES_IN_MARKER + ICC_HEADER_SIZE);
+            while (icc_profile_length > 0) {
+                // length of profile to put in this marker
+                unsigned int length = std::min (icc_profile_length, (unsigned int)MAX_DATA_BYTES_IN_MARKER);
+                icc_profile_length -= length;
+                // Write the JPEG marker header (APP2 code and marker length)
+                strcpy ((char *)&profile[0], "ICC_PROFILE");
+                profile[11] = 0;
+                profile[12] = curr_marker;
+                profile[13] = (unsigned char) num_markers;
+                memcpy(&profile[0] + ICC_HEADER_SIZE, icc_profile+length*(curr_marker-1), length);
+                jpeg_write_marker(&m_cinfo, JPEG_APP0 + 2, &profile[0], ICC_HEADER_SIZE+length);
+                curr_marker++;
+            }
+        }
+    }
+
     m_dither = m_spec.get_int_attribute ("oiio:dither", 0);
 
     // If user asked for tiles -- which JPEG doesn't support, emulate it by
