@@ -34,7 +34,6 @@
 
 #include <OpenEXR/half.h>
 
-#include <boost/tokenizer.hpp>
 #include <boost/foreach.hpp>
 
 #include "OpenImageIO/dassert.h"
@@ -43,21 +42,24 @@
 #include "OpenImageIO/fmath.h"
 #include "OpenImageIO/imageio.h"
 #include "imageio_pvt.h"
-#include "OpenImageIO/pugixml.hpp"
 
+#if USE_EXTERNAL_PUGIXML
+# include "pugixml.hpp"
+#else
+# include "OpenImageIO/pugixml.hpp"
+#endif
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
 
 // Generate the default quantization parameters, templated on the data
 // type.
 template <class T>
 inline void
-get_default_quantize_ (int &quant_min, int &quant_max)
+get_default_quantize_ (long long &quant_min, long long &quant_max)
 {
     if (std::numeric_limits <T>::is_integer) {
-        quant_min    = (int) std::numeric_limits <T>::min();
-        quant_max    = (int) std::numeric_limits <T>::max();
+        quant_min    = (long long) std::numeric_limits <T>::min();
+        quant_max    = (long long) std::numeric_limits <T>::max();
     } else {
         quant_min    = 0;
         quant_max    = 0;
@@ -69,7 +71,8 @@ get_default_quantize_ (int &quant_min, int &quant_max)
 // Given the format, set the default quantization range.
 // Rely on the template version to make life easy.
 void
-pvt::get_default_quantize (TypeDesc format, int &quant_min, int &quant_max)
+pvt::get_default_quantize (TypeDesc format,
+                           long long &quant_min, long long &quant_max)
 {
     switch (format.basetype) {
     case TypeDesc::UNKNOWN:
@@ -401,10 +404,70 @@ ImageSpec::find_attribute (string_view name, TypeDesc searchtype,
 
 
 
+const ImageIOParameter *
+ImageSpec::find_attribute (string_view name, ImageIOParameter &tmpparam,
+                           TypeDesc searchtype, bool casesensitive) const
+{
+    ImageIOParameterList::const_iterator iter =
+        extra_attribs.find (name, searchtype, casesensitive);
+    if (iter != extra_attribs.end())
+        return &(*iter);
+    // Check named items in the ImageSpec structs, not in extra_attrubs
+#define MATCH(n,t) (((!casesensitive && Strutil::iequals(name,n)) || \
+                     ( casesensitive && name == n)) && \
+                    (searchtype == TypeDesc::UNKNOWN || searchtype == t))
+#define GETINT(n) if (MATCH(#n,TypeDesc::TypeInt)) { \
+                      tmpparam.init (#n, TypeDesc::TypeInt, 1, &this->n); \
+                      return &tmpparam; \
+                  }
+    GETINT(nchannels);
+    GETINT(width);
+    GETINT(height);
+    GETINT(depth);
+    GETINT(x);
+    GETINT(y);
+    GETINT(z);
+    GETINT(full_width);
+    GETINT(full_height);
+    GETINT(full_depth);
+    GETINT(full_x);
+    GETINT(full_y);
+    GETINT(full_z);
+    GETINT(tile_width);
+    GETINT(tile_height);
+    GETINT(tile_depth);
+    GETINT(alpha_channel);
+    GETINT(z_channel);
+    // some special cases
+    if (MATCH("geom", TypeDesc::TypeString)) {
+        ustring s = (depth <= 1 && full_depth <= 1)
+                    ? ustring::format ("%dx%d%+d%+d", width, height, x, y)
+                    : ustring::format ("%dx%dx%d%+d%+d%+d", width, height, depth, x, y, z);
+        tmpparam.init ("geom", TypeDesc::TypeString, 1, &s);
+        return &tmpparam;
+    }
+    if (MATCH("full_geom", TypeDesc::TypeString)) {
+        ustring s = (depth <= 1 && full_depth <= 1)
+                    ? ustring::format ("%dx%d%+d%+d",
+                                       full_width, full_height, full_x, full_y)
+                    : ustring::format ("%dx%dx%d%+d%+d%+d",
+                                       full_width, full_height, full_depth,
+                                       full_x, full_y, full_z);
+        tmpparam.init ("full_geom", TypeDesc::TypeString, 1, &s);
+        return &tmpparam;
+    }
+#undef GETINT
+#undef MATCH
+    return NULL;
+}
+
+
+
 int
 ImageSpec::get_int_attribute (string_view name, int val) const
 {
-    const ImageIOParameter *p = find_attribute (name);
+    ImageIOParameter tmpparam;
+    const ImageIOParameter *p = find_attribute (name, tmpparam);
     if (p) {
         if (p->type() == TypeDesc::INT)
             val = *(const int *)p->data();
@@ -431,7 +494,8 @@ ImageSpec::get_int_attribute (string_view name, int val) const
 float
 ImageSpec::get_float_attribute (string_view name, float val) const
 {
-    const ImageIOParameter *p = find_attribute (name);
+    ImageIOParameter tmpparam;
+    const ImageIOParameter *p = find_attribute (name, tmpparam);
     if (p) {
         if (p->type() == TypeDesc::FLOAT)
             val = *(const float *)p->data();
@@ -464,7 +528,8 @@ ImageSpec::get_float_attribute (string_view name, float val) const
 string_view
 ImageSpec::get_string_attribute (string_view name, string_view val) const
 {
-    const ImageIOParameter *p = find_attribute (name, TypeDesc::STRING);
+    ImageIOParameter tmpparam;
+    const ImageIOParameter *p = find_attribute (name, tmpparam, TypeDesc::STRING);
     if (p)
         return *(ustring *)p->data();
     else return val;
@@ -474,6 +539,16 @@ ImageSpec::get_string_attribute (string_view name, string_view val) const
 
 namespace {  // make an anon namespace
 
+template < typename T >
+void formatType(const ImageIOParameter& p, const int n, const TypeDesc& element, const char* formatString, std::string& out) {
+  const T *f = (const T *)p.data();
+  for (int i = 0;  i < n;  ++i) {
+      if (i)
+          out += ", ";
+      for (int c = 0;  c < (int)element.aggregate;  ++c, ++f)
+          out += Strutil::format (formatString, (c ? " " : ""), f[0]);
+  }
+}
 
 static std::string
 format_raw_metadata (const ImageIOParameter &p, int maxsize=16)
@@ -482,59 +557,33 @@ format_raw_metadata (const ImageIOParameter &p, int maxsize=16)
     TypeDesc element = p.type().elementtype();
     int nfull = p.type().numelements() * p.nvalues();
     int n = std::min (nfull, maxsize);
-    if (element == TypeDesc::STRING) {
+    if (element.basetype == TypeDesc::STRING) {
         for (int i = 0;  i < n;  ++i) {
             const char *s = ((const char **)p.data())[i];
             out += Strutil::format ("%s\"%s\"", (i ? ", " : ""), s ? s : "");
         }
     } else if (element.basetype == TypeDesc::FLOAT) {
-        const float *f = (const float *)p.data();
-        for (int i = 0;  i < n;  ++i) {
-            if (i)
-                out += ", ";
-            for (int c = 0;  c < (int)element.aggregate;  ++c, ++f)
-                out += Strutil::format ("%s%g", (c ? " " : ""), f[0]);
-        }
+        formatType< float >(p, n, element, "%s%g", out);
     } else if (element.basetype == TypeDesc::DOUBLE) {
-        const double *f = (const double *)p.data();
-        for (int i = 0;  i < n;  ++i) {
-            if (i)
-                out += ", ";
-            for (int c = 0;  c < (int)element.aggregate;  ++c, ++f)
-                out += Strutil::format ("%s%g", (c ? " " : ""), f[0]);
-        }
+        formatType< double >(p, n, element, "%s%g", out);
     } else if (element.basetype == TypeDesc::HALF) {
-        const half *f = (const half *)p.data();
-        for (int i = 0;  i < n;  ++i) {
-            if (i)
-                out += ", ";
-            for (int c = 0;  c < (int)element.aggregate;  ++c, ++f)
-                out += Strutil::format ("%s%g", (c ? " " : ""), (float)f[0]);
-        }
-    } else if (element == TypeDesc::INT) {
-        for (int i = 0;  i < n;  ++i)
-            out += Strutil::format ("%s%d", (i ? ", " : ""), ((const int *)p.data())[i]);
-    } else if (element == TypeDesc::UINT) {
-        for (int i = 0;  i < n;  ++i)
-            out += Strutil::format ("%s%d", (i ? ", " : ""), ((const unsigned int *)p.data())[i]);
-    } else if (element == TypeDesc::UINT16) {
-        for (int i = 0;  i < n;  ++i)
-            out += Strutil::format ("%s%u", (i ? ", " : ""), ((const unsigned short *)p.data())[i]);
-    } else if (element == TypeDesc::INT16) {
-        for (int i = 0;  i < n;  ++i)
-            out += Strutil::format ("%s%d", (i ? ", " : ""), ((const short *)p.data())[i]);
-    } else if (element == TypeDesc::UINT64) {
-        for (int i = 0;  i < n;  ++i)
-            out += Strutil::format ("%s%llu", (i ? ", " : ""), ((const unsigned long long *)p.data())[i]);
-    } else if (element == TypeDesc::INT64) {
-        for (int i = 0;  i < n;  ++i)
-            out += Strutil::format ("%s%lld", (i ? ", " : ""), ((const long long *)p.data())[i]);
-    } else if (element == TypeDesc::UINT8) {
-        for (int i = 0;  i < n;  ++i)
-            out += Strutil::format ("%s%d", (i ? ", " : ""), int(((const unsigned char *)p.data())[i]));
-    } else if (element == TypeDesc::INT8) {
-        for (int i = 0;  i < n;  ++i)
-            out += Strutil::format ("%s%d", (i ? ", " : ""), int(((const char *)p.data())[i]));
+        formatType< half >(p, n, element, "%s%g", out);
+    } else if (element.basetype == TypeDesc::INT) {
+        formatType< int >(p, n, element, "%s%d", out);
+    } else if (element.basetype == TypeDesc::UINT) {
+        formatType< unsigned int >(p, n, element, "%s%d", out);
+    } else if (element.basetype == TypeDesc::UINT16) {
+        formatType< unsigned short >(p, n, element, "%s%u", out);
+    } else if (element.basetype == TypeDesc::INT16) {
+        formatType< short >(p, n, element, "%s%d", out);
+    } else if (element.basetype == TypeDesc::UINT64) {
+        formatType< unsigned long long >(p, n, element, "%s%llu", out);
+    } else if (element.basetype == TypeDesc::INT64) {
+        formatType< long long >(p, n, element, "%s%lld", out);
+    } else if (element.basetype == TypeDesc::UINT8) {
+        formatType< unsigned char >(p, n, element, "%s%d", out);
+    } else if (element.basetype == TypeDesc::INT8) {
+        formatType< char >(p, n, element, "%s%d", out);
     } else {
         out += Strutil::format ("<unknown data type> (base %d, agg %d vec %d)",
                 p.type().basetype, p.type().aggregate,
@@ -545,8 +594,6 @@ format_raw_metadata (const ImageIOParameter &p, int maxsize=16)
     return out;
 }
 
-
-
 struct LabelTable {
     int value;
     const char *label;
@@ -721,7 +768,7 @@ static LabelTable GPSAltitudeRef_table[] = {
 };
 
 static LabelTable GPSStatus_table[] = {
-    { 'A', "measurement in progress" }, { 'V', "measurement interoperability" },
+    { 'A', "measurement active" }, { 'V', "measurement void" },
     { -1, NULL }
 };
 
@@ -735,12 +782,12 @@ static LabelTable GPSSpeedRef_table[] = {
 };
 
 static LabelTable GPSDestDistanceRef_table[] = {
-    { 'K', "km" }, { 'M', "miles" }, { 'N', "knots" }, 
+    { 'K', "km" }, { 'M', "miles" }, { 'N', "nautical miles" },
     { -1, NULL }
 };
 
 static LabelTable magnetic_table[] = {
-    { 'T', "true direction" }, { 'M', "magnetic direction" }, { -1, NULL }
+    { 'T', "true north" }, { 'M', "magnetic north" }, { -1, NULL }
 };
 
 typedef std::string (*ExplainerFunc) (const ImageIOParameter &p, 
@@ -796,7 +843,7 @@ static ExplanationTableEntry explanation[] = {
 
 
 std::string
-ImageSpec::metadata_val (const ImageIOParameter &p, bool human) const
+ImageSpec::metadata_val (const ImageIOParameter &p, bool human)
 {
     std::string out = format_raw_metadata (p, human ? 16 : 1024);
 
@@ -947,6 +994,5 @@ ImageSpec::from_xml (const char *xml)
 }
 
 
-}
-OIIO_NAMESPACE_EXIT
+OIIO_NAMESPACE_END
 
