@@ -51,8 +51,7 @@
 #include "OpenImageIO/thread.h"
 #include "OpenImageIO/simd.h"
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
 
 
 static atomic_ll IB_local_mem_current;
@@ -110,10 +109,13 @@ set_roi_full (ImageSpec &spec, const ROI &newroi)
 ROI
 roi_union (const ROI &A, const ROI &B)
 {
-    return ROI (std::min (A.xbegin, B.xbegin), std::max (A.xend, B.xend),
-                std::min (A.ybegin, B.ybegin), std::max (A.yend, B.yend),
-                std::min (A.zbegin, B.zbegin), std::max (A.zend, B.zend),
-                std::min (A.chbegin, B.chbegin), std::max (A.chend, B.chend));
+    ROI R (A.defined() ? A : B);
+    if (A.defined() && B.defined())
+        R = ROI (std::min (R.xbegin,  B.xbegin),  std::max (R.xend,  B.xend),
+                 std::min (R.ybegin,  B.ybegin),  std::max (R.yend,  B.yend),
+                 std::min (R.zbegin,  B.zbegin),  std::max (R.zend,  B.zend),
+                 std::min (R.chbegin, B.chbegin), std::max (R.chend, B.chend));
+    return R;
 }
 
 
@@ -121,10 +123,13 @@ roi_union (const ROI &A, const ROI &B)
 ROI
 roi_intersection (const ROI &A, const ROI &B)
 {
-    return ROI (std::max (A.xbegin, B.xbegin), std::min (A.xend, B.xend),
-                std::max (A.ybegin, B.ybegin), std::min (A.yend, B.yend),
-                std::max (A.zbegin, B.zbegin), std::min (A.zend, B.zend),
-                std::max (A.chbegin, B.chbegin), std::min (A.chend, B.chend));
+    ROI R (A.defined() ? A : B);
+    if (A.defined() && B.defined())
+        R = ROI (std::max (R.xbegin,  B.xbegin),  std::min (R.xend,  B.xend),
+                 std::max (R.ybegin,  B.ybegin),  std::min (R.yend,  B.yend),
+                 std::max (R.zbegin,  B.zbegin),  std::min (R.zend,  B.zend),
+                 std::max (R.chbegin, B.chbegin), std::min (R.chend, B.chend));
+    return R;
 }
 
 
@@ -241,6 +246,9 @@ public:
         return m_spec;
     }
 
+    void threads (int n) { m_threads = n; }
+    int threads () const { return m_threads; }
+
 private:
     ImageBuf::IBStorage m_storage; ///< Pixel storage class
     ustring m_name;              ///< Filename of the image
@@ -249,6 +257,7 @@ private:
     int m_current_subimage;      ///< Current subimage we're viewing
     int m_current_miplevel;      ///< Current miplevel we're viewing
     int m_nmiplevels;            ///< # of MIP levels in the current subimage
+    int m_threads;               ///< thread policy for this image
     ImageSpec m_spec;            ///< Describes the image (size, etc)
     ImageSpec m_nativespec;      ///< Describes the true native image
     boost::scoped_array<char> m_pixels; ///< Pixel data, if local and we own it
@@ -287,6 +296,8 @@ ImageBufImpl::ImageBufImpl (string_view filename,
     : m_storage(ImageBuf::UNINITIALIZED),
       m_name(filename), m_nsubimages(0),
       m_current_subimage(subimage), m_current_miplevel(miplevel),
+      m_nmiplevels(0),
+      m_threads(0),
       m_localpixels(NULL),
       m_spec_valid(false), m_pixels_valid(false),
       m_badfile(false), m_pixelaspect(1),
@@ -332,6 +343,7 @@ ImageBufImpl::ImageBufImpl (const ImageBufImpl &src)
       m_current_subimage(src.m_current_subimage),
       m_current_miplevel(src.m_current_miplevel),
       m_nmiplevels(src.m_nmiplevels),
+      m_threads(src.m_threads),
       m_spec(src.m_spec), m_nativespec(src.m_nativespec),
       m_pixels(src.m_localpixels ? new char [src.m_spec.image_bytes()] : NULL),
       m_localpixels(m_pixels.get()),
@@ -629,11 +641,7 @@ ImageBufImpl::realloc ()
     if (m_allocated_size)
         m_pixels_valid = true;
     if (m_spec.deep) {
-        int npixels = (int)m_spec.image_pixels();
-        std::vector<TypeDesc> chanformats (m_spec.channelformats);
-        chanformats.resize (m_spec.nchannels, m_spec.format);
-        m_deepdata.init (npixels, m_spec.nchannels,
-                         &(*chanformats.begin()), &(*chanformats.end()));
+        m_deepdata.init (m_spec);
         m_storage = ImageBuf::LOCALBUFFER;
     }
 #if 0
@@ -661,31 +669,6 @@ ImageBufImpl::alloc (const ImageSpec &spec)
 
 
 
-void
-ImageBuf::copy_from (const ImageBuf &src)
-{
-    if (this == &src)
-        return;
-    const ImageBufImpl *srcimpl (src.impl());
-    srcimpl->validate_pixels ();
-    const ImageSpec &srcspec (srcimpl->spec());
-    ImageBufImpl *impl (this->impl());
-    const ImageSpec &spec (impl->spec());
-    ASSERT (spec.width == srcspec.width &&
-            spec.height == srcspec.height &&
-            spec.depth == srcspec.depth &&
-            spec.nchannels == srcspec.nchannels);
-    impl->realloc ();
-    if (spec.deep)
-        impl->m_deepdata = srcimpl->m_deepdata;
-    else
-        src.get_pixels (src.xbegin(), src.xend(), src.ybegin(), src.yend(),
-                        src.zbegin(), src.zend(), spec.format,
-                        impl->m_localpixels);
-}
-
-
-
 bool
 ImageBufImpl::init_spec (string_view filename, int subimage, int miplevel)
 {
@@ -794,6 +777,7 @@ ImageBufImpl::read (int subimage, int miplevel, bool force, TypeDesc convert,
             error ("%s", OIIO::geterror());
             return false;
         }
+        input->threads (threads());  // Pass on our thread policy
         ImageSpec dummyspec;
         if (! input->seek_subimage (subimage, miplevel, dummyspec)) {
             error ("%s", input->geterror());
@@ -828,13 +812,13 @@ ImageBufImpl::read (int subimage, int miplevel, bool force, TypeDesc convert,
         m_pixels_valid = true;
         m_storage = ImageBuf::IMAGECACHE;
 #ifndef NDEBUG
-        std::cerr << "read was not necessary -- using cache\n";
+        // std::cerr << "read was not necessary -- using cache\n";
 #endif
         return true;
     } else {
 #ifndef NDEBUG
-        std::cerr << "going to have to read " << m_name << ": "
-                  << m_spec.format.c_str() << " vs " << convert.c_str() << "\n";
+        // std::cerr << "going to have to read " << m_name << ": "
+        //           << m_spec.format.c_str() << " vs " << convert.c_str() << "\n";
 #endif
         // FIXME/N.B. - is it really best to go through the ImageCache
         // for forced IB reads?  Are there circumstances in which we
@@ -868,6 +852,7 @@ ImageBufImpl::read (int subimage, int miplevel, bool force, TypeDesc convert,
         ImageInput *in = ImageInput::open (m_name.string(), m_configspec.get());
         bool ok = true;
         if (in) {
+            in->threads (threads());  // Pass on our thread policy
             if (subimage || miplevel) {
                 ImageSpec newspec;
                 ok &= in->seek_subimage (subimage, miplevel, newspec);
@@ -986,6 +971,7 @@ ImageBuf::write (string_view _filename, string_view _fileformat,
         error ("%s", geterror());
         return false;
     }
+    out->threads (threads());  // Pass on our thread policy
 
     // Write scanline files by default, but if the file type allows tiles,
     // user can override via ImageBuf::set_write_tiles(), or by using the
@@ -1022,6 +1008,18 @@ ImageBuf::write (string_view _filename, string_view _fileformat,
 
 
 
+bool
+ImageBuf::make_writeable (bool keep_cache_type)
+{
+    if (storage() == IMAGECACHE) {
+        return read (subimage(), miplevel(), true /*force*/,
+                     keep_cache_type ? impl()->m_cachedpixeltype : TypeDesc());
+    }
+    return true;
+}
+
+
+
 void
 ImageBufImpl::copy_metadata (const ImageBufImpl &src)
 {
@@ -1233,6 +1231,22 @@ ImageBuf::initialized () const
 
 
 
+void
+ImageBuf::threads (int n) const
+{
+    impl()->threads();
+}
+
+
+
+int
+ImageBuf::threads () const
+{
+    return impl()->threads();
+}
+
+
+
 
 namespace {
 
@@ -1272,6 +1286,9 @@ ImageBuf::copy_pixels (const ImageBuf &src)
     if (this == &src)
         return true;
 
+    if (deep() || src.deep())
+        return false;   // This operation is not supported for deep images
+
     // compute overlap
     ROI myroi = get_roi(spec());
     ROI roi = roi_intersection (myroi, get_roi(src.spec()));
@@ -1289,7 +1306,7 @@ ImageBuf::copy_pixels (const ImageBuf &src)
 
 
 bool
-ImageBuf::copy (const ImageBuf &src)
+ImageBuf::copy (const ImageBuf &src, TypeDesc format)
 {
     src.impl()->validate_pixels ();
     if (this == &src)     // self-assignment
@@ -1298,12 +1315,32 @@ ImageBuf::copy (const ImageBuf &src)
         clear();
         return true;
     }
-    reset (src.name(), src.spec());
+    if (src.deep()) {
+        reset (src.name(), src.spec());
+        impl()->m_deepdata = src.impl()->m_deepdata;
+        return true;
+    }
+    if (format.basetype == TypeDesc::UNKNOWN || src.deep())
+        reset (src.name(), src.spec());
+    else {
+        ImageSpec newspec (src.spec());
+        newspec.set_format (format);
+        newspec.channelformats.clear ();
+        reset (src.name(), newspec);
+    }
     return this->copy_pixels (src);
 }
 
 
 
+bool
+ImageBuf::copy (const ImageBuf &src)
+{
+    return copy (src, TypeDesc::UNKNOWN);
+}
+
+
+
 template<typename T>
 static inline float getchannel_ (const ImageBuf &buf, int x, int y, int z,
                                  int c, ImageBuf::WrapMode wrap)
@@ -1538,28 +1575,55 @@ ImageBuf::setpixel (int i, const float *pixel, int maxchannels)
 
 template<typename D, typename S>
 static bool
-get_pixel_channels_ (const ImageBuf &buf, int xbegin, int xend,
-                     int ybegin, int yend, int zbegin, int zend, 
-                     int chbegin, int chend, void *r_,
-                     stride_t xstride, stride_t ystride, stride_t zstride)
+get_pixels_ (const ImageBuf &buf, ROI whole_roi, ROI roi, void *r_,
+             stride_t xstride, stride_t ystride, stride_t zstride,
+             int nthreads=0)
 {
+    if (nthreads != 1 && roi.npixels() >= 64*1024) {
+        // Possible multiple thread case -- recurse via parallel_image
+        ImageBufAlgo::parallel_image (
+            OIIO::bind(get_pixels_<D,S>, OIIO::ref(buf),
+                        whole_roi,  _1 /*roi*/, r_,
+                        xstride, ystride, zstride, 1 /*nthreads*/),
+            roi, nthreads);
+        return true;
+    }
+
     D *r = (D *)r_;
-    int w = (xend-xbegin), h = (yend-ybegin);
-    int nchans = chend - chbegin;
-    ImageSpec::auto_stride (xstride, ystride, zstride, sizeof(D), nchans, w, h);
-    for (ImageBuf::ConstIterator<S,D> p (buf, xbegin, xend, ybegin, yend, zbegin, zend);
-         !p.done(); ++p) {
-        imagesize_t offset = (p.z()-zbegin)*zstride + (p.y()-ybegin)*ystride
-                           + (p.x()-xbegin)*xstride;
+    int nchans = roi.nchannels();
+    for (ImageBuf::ConstIterator<S,D> p (buf, roi); !p.done(); ++p) {
+        imagesize_t offset = (p.z()-whole_roi.zbegin)*zstride
+                           + (p.y()-whole_roi.ybegin)*ystride
+                           + (p.x()-whole_roi.xbegin)*xstride;
         D *rc = (D *)((char *)r + offset);
         for (int c = 0;  c < nchans;  ++c)
-            rc[c] = p[c+chbegin];
+            rc[c] = p[c+roi.chbegin];
     }
     return true;
 }
 
 
 
+bool
+ImageBuf::get_pixels (ROI roi, TypeDesc format, void *result,
+                      stride_t xstride, stride_t ystride,
+                      stride_t zstride) const
+{
+    if (! roi.defined())
+        roi = this->roi();
+    roi.chend = std::min (roi.chend, nchannels());
+    ImageSpec::auto_stride (xstride, ystride, zstride, format.size(),
+                            roi.nchannels(), roi.width(), roi.height());
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "get_pixels", get_pixels_,
+                          format, spec().format, *this, roi, roi,
+                          result, xstride, ystride, zstride, threads());
+    return ok;
+}
+
+
+
+// DEPRECATED
 template<typename D>
 bool
 ImageBuf::get_pixel_channels (int xbegin, int xend, int ybegin, int yend,
@@ -1568,16 +1632,19 @@ ImageBuf::get_pixel_channels (int xbegin, int xend, int ybegin, int yend,
                               stride_t xstride, stride_t ystride,
                               stride_t zstride) const
 {
+    ROI roi (xbegin, xend, ybegin, yend, zbegin, zend, chbegin, chend);
+    ImageSpec::auto_stride (xstride, ystride, zstride, sizeof(D),
+                            roi.nchannels(), roi.width(), roi.height());
     bool ok;
-    OIIO_DISPATCH_TYPES2_HELP (ok, "get_pixel_channels", get_pixel_channels_,
-                               D, spec().format, *this,
-                               xbegin, xend, ybegin, yend, zbegin, zend,
-                               chbegin, chend, r, xstride, ystride, zstride);
+    OIIO_DISPATCH_TYPES2_HELP (ok, "get_pixel_channels", get_pixels_,
+                               D, spec().format, *this, roi, roi,
+                               r, xstride, ystride, zstride);
     return ok;
 }
 
 
 
+// DEPRECATED
 bool
 ImageBuf::get_pixel_channels (int xbegin, int xend, int ybegin, int yend,
                               int zbegin, int zend, int chbegin, int chend,
@@ -1585,25 +1652,61 @@ ImageBuf::get_pixel_channels (int xbegin, int xend, int ybegin, int yend,
                               stride_t xstride, stride_t ystride,
                               stride_t zstride) const
 {
-    bool ok;
-    OIIO_DISPATCH_TYPES2 (ok, "get_pixel_channels", get_pixel_channels_,
-                          format, spec().format, *this,
-                          xbegin, xend, ybegin, yend, zbegin, zend,
-                          chbegin, chend, result, xstride, ystride, zstride);
-    return ok;
+    ROI roi (xbegin, xend, ybegin, yend, zbegin, zend, chbegin, chend);
+    return get_pixels (roi, format, result, xstride, ystride, zstride);
 }
 
 
 
+// DEPRECATED
 bool
 ImageBuf::get_pixels (int xbegin, int xend, int ybegin, int yend,
                       int zbegin, int zend, TypeDesc format, void *result,
                       stride_t xstride, stride_t ystride,
                       stride_t zstride) const
 {
-    return get_pixel_channels (xbegin, xend, ybegin, yend, zbegin, zend,
-                               0, nchannels(), format, result,
-                               xstride, ystride, zstride);
+    ROI roi (xbegin, xend, ybegin, yend, zbegin, zend, 0, nchannels());
+    return get_pixels (roi, format, result, xstride, ystride, zstride);
+}
+
+
+
+template<typename D, typename S>
+static bool
+set_pixels_ (ImageBuf &buf, ROI roi, const void *data_,
+             stride_t xstride, stride_t ystride, stride_t zstride)
+{
+    const D *data = (const D *)data_;
+    int w = roi.width(), h = roi.height(), nchans = roi.nchannels();
+    ImageSpec::auto_stride (xstride, ystride, zstride, sizeof(S), nchans, w, h);
+    for (ImageBuf::Iterator<D,S> p (buf, roi);  !p.done();  ++p) {
+        if (! p.exists())
+            continue;
+        imagesize_t offset = (p.z()-roi.zbegin)*zstride
+                           + (p.y()-roi.ybegin)*ystride
+                           + (p.x()-roi.xbegin)*xstride;
+        const S *src = (const S *)((const char *)data + offset);
+        for (int c = 0;  c < nchans;  ++c)
+            p[c+roi.chbegin] = src[c];
+    }
+    return true;
+}
+
+
+
+bool
+ImageBuf::set_pixels (ROI roi, TypeDesc format, const void *data,
+                      stride_t xstride, stride_t ystride, stride_t zstride)
+{
+    bool ok;
+    if (! roi.defined())
+        roi = this->roi();
+    roi.chend = std::min (roi.chend, nchannels());
+    OIIO_DISPATCH_TYPES2 (ok, "set_pixels", set_pixels_,
+                          spec().format, format, *this, roi,
+                          data, xstride, ystride, zstride);
+    return ok;
+
 }
 
 
@@ -1621,7 +1724,7 @@ ImageBuf::deep_samples (int x, int y, int z) const
     if (x >= m_spec.width || y >= m_spec.height || z >= m_spec.depth)
         return 0;
     int p = (z * m_spec.height + y) * m_spec.width  + x;
-    return deepdata()->nsamples[p];
+    return deepdata()->samples(p);
 }
 
 
@@ -1640,7 +1743,7 @@ ImageBuf::deep_pixel_ptr (int x, int y, int z, int c) const
         c < 0 || c >= m_spec.nchannels)
         return NULL;
     int p = (z * m_spec.height + y) * m_spec.width  + x;
-    return deepdata()->nsamples[p] ? deepdata()->pointers[p*m_spec.nchannels] : NULL;
+    return deepdata()->samples(p) ? deepdata()->channel_ptr (p, c) : NULL;
 }
 
 
@@ -1673,6 +1776,19 @@ ImageBuf::deep_value_uint (int x, int y, int z, int c, int s) const
 
 
 
+void
+ImageBuf::set_deep_samples (int x, int y, int z, int samps)
+{
+    if (! deep())
+        return ;
+    const ImageSpec &m_spec (spec());
+    x -= m_spec.x;  y -= m_spec.y;  z -= m_spec.z;
+    int p = (z * m_spec.height + y) * m_spec.width + x;
+    impl()->m_deepdata.set_samples (p, samps);
+}
+
+
+
 void
 ImageBuf::set_deep_value (int x, int y, int z, int c, int s, float value)
 {
@@ -1696,7 +1812,7 @@ ImageBuf::set_deep_value_uint (int x, int y, int z, int c, int s, uint32_t value
     const ImageSpec &m_spec (spec());
     x -= m_spec.x;  y -= m_spec.y;  z -= m_spec.z;
     int p = (z * m_spec.height + y) * m_spec.width + x;
-    return impl()->m_deepdata.set_deep_value_uint (p, c, s, value);
+    return impl()->m_deepdata.set_deep_value (p, c, s, value);
 }
 
 
@@ -2104,5 +2220,4 @@ ImageBuf::retile (int x, int y, int z, ImageCache::Tile* &tile,
 
 
 
-}
-OIIO_NAMESPACE_EXIT
+OIIO_NAMESPACE_END
