@@ -32,14 +32,6 @@
 /// Implementation of ImageBufAlgo algorithms that analize or compare
 /// images.
 
-/* This header has to be included before boost/regex.hpp header
-   If it is included after, there is an error
-   "undefined reference to CSHA1::Update (unsigned char const*, unsigned long)"
-*/
-#include "OpenImageIO/SHA1.h"
-
-#include <boost/bind.hpp>
-
 #include <OpenEXR/half.h>
 
 #include <cmath>
@@ -50,7 +42,7 @@
 #include "OpenImageIO/imagebufalgo.h"
 #include "OpenImageIO/imagebufalgo_util.h"
 #include "OpenImageIO/dassert.h"
-#include "OpenImageIO/thread.h"
+#include "OpenImageIO/SHA1.h"
 
 #ifdef USE_OPENSSL
 #ifdef __APPLE__
@@ -63,8 +55,7 @@
 
 
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
 
 
 inline void
@@ -132,7 +123,7 @@ finalize (ImageBufAlgo::PixelStats &p)
             double Count = static_cast<double>(p.finitecount[c]);
             double davg = p.sum[c] / Count;
             p.avg[c] = static_cast<float>(davg);
-            p.stddev[c] = static_cast<float>(sqrt(p.sum2[c]/Count - davg*davg));
+            p.stddev[c] = static_cast<float>(safe_sqrt(p.sum2[c]/Count - davg*davg));
         }
     }
 }
@@ -203,8 +194,8 @@ computePixelStats_ (const ImageBuf &src, ImageBufAlgo::PixelStats &stats,
 
     // Compute final results
     finalize (stats);
-    
-    return true;
+
+    return ! src.has_error();
 };
 
 
@@ -223,9 +214,10 @@ ImageBufAlgo::computePixelStats (PixelStats &stats, const ImageBuf &src,
         return false;
     }
 
-    OIIO_DISPATCH_TYPES ("computePixelStats", computePixelStats_,
+    bool ok;
+    OIIO_DISPATCH_TYPES (ok, "computePixelStats", computePixelStats_,
                          src.spec().format, src, stats, roi, nthreads);
-    return false;
+    return ok;
 }
 
 
@@ -237,22 +229,38 @@ compare_value (ImageBuf::ConstIterator<BUFT,float> &a, int chan,
                float &maxval, double &batcherror, double &batch_sqrerror,
                bool &failed, bool &warned, float failthresh, float warnthresh)
 {
+    if (!isfinite(aval) || !isfinite(bval)) {
+        if (isnan(aval) == isnan(bval) && isinf(aval) == isinf(bval))
+            return; // NaN may match NaN, Inf may match Inf
+        if (isfinite(result.maxerror)) {
+            // non-finite errors trump finite ones
+            result.maxerror = std::numeric_limits<float>::infinity();
+            result.maxx = a.x();
+            result.maxy = a.y();
+            result.maxz = a.z();
+            result.maxc = chan;
+            return;
+        }
+    }
     maxval = std::max (maxval, std::max (aval, bval));
     double f = fabs (aval - bval);
     batcherror += f;
     batch_sqrerror += f*f;
-    if (f > result.maxerror) {
+    // We use the awkward '!(a<=threshold)' construct so that we have
+    // failures when f is a NaN (since all comparisons involving NaN will
+    // return false).
+    if (!(f <= result.maxerror)) {
         result.maxerror = f;
         result.maxx = a.x();
         result.maxy = a.y();
         result.maxz = a.z();
         result.maxc = chan;
     }
-    if (! warned && f > warnthresh) {
+    if (! warned && !(f <= warnthresh)) {
         ++result.nwarn;
         warned = true;
     }
-    if (! failed && f > failthresh) {
+    if (! failed && !(f <= failthresh)) {
         ++result.nfail;
         failed = true;
     }
@@ -337,14 +345,15 @@ ImageBufAlgo::compare (const ImageBuf &A, const ImageBuf &B,
     if (B.deep() != A.deep())
         return false;
 
-    OIIO_DISPATCH_TYPES2 ("compare", compare_,
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "compare", compare_,
                           A.spec().format, B.spec().format,
                           A, B, failthresh, warnthresh, result,
                           roi, nthreads);
     // FIXME - The nthreads argument is for symmetry with the rest of
     // ImageBufAlgo and for future expansion. But for right now, we
     // don't actually split by threads.  Maybe later.
-    return false;
+    return ok;
 }
 
 
@@ -394,8 +403,10 @@ ImageBufAlgo::isConstantColor (const ImageBuf &src, float *color,
     if (roi.nchannels() == 0)
         return true;
     
-    OIIO_DISPATCH_TYPES ("isConstantColor", isConstantColor_,
+    bool ok;
+    OIIO_DISPATCH_TYPES (ok, "isConstantColor", isConstantColor_,
                          src.spec().format, src, color, roi, nthreads);
+    return ok;
     // FIXME -  The nthreads argument is for symmetry with the rest of
     // ImageBufAlgo and for future expansion. But for right now, we
     // don't actually split by threads.  Maybe later.
@@ -428,8 +439,10 @@ ImageBufAlgo::isConstantChannel (const ImageBuf &src, int channel, float val,
     if (channel < 0 || channel >= src.nchannels())
         return false;  // that channel doesn't exist in the image
 
-    OIIO_DISPATCH_TYPES ("isConstantChannel", isConstantChannel_,
+    bool ok;
+    OIIO_DISPATCH_TYPES (ok, "isConstantChannel", isConstantChannel_,
                          src.spec().format, src, channel, val, roi, nthreads);
+    return ok;
     // FIXME -  The nthreads argument is for symmetry with the rest of
     // ImageBufAlgo and for future expansion. But for right now, we
     // don't actually split by threads.  Maybe later.
@@ -466,8 +479,10 @@ ImageBufAlgo::isMonochrome (const ImageBuf &src, ROI roi, int nthreads)
     if (roi.nchannels() < 2)
         return true;  // 1 or fewer channels are always "monochrome"
 
-    OIIO_DISPATCH_TYPES ("isMonochrome", isMonochrome_, src.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES (ok, "isMonochrome", isMonochrome_, src.spec().format,
                          src, roi, nthreads);
+    return ok;
     // FIXME -  The nthreads argument is for symmetry with the rest of
     // ImageBufAlgo and for future expansion. But for right now, we
     // don't actually split by threads.  Maybe later.
@@ -484,7 +499,7 @@ color_count_ (const ImageBuf &src, atomic_ll *count,
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Lots of pixels and request for multi threads? Parallelize.
         ImageBufAlgo::parallel_image (
-            boost::bind(color_count_<T>, boost::ref(src),
+            OIIO::bind(color_count_<T>, OIIO::ref(src),
                         count, ncolors, color, eps,
                         _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
@@ -537,9 +552,11 @@ ImageBufAlgo::color_count (const ImageBuf &src, imagesize_t *count,
 
     for (int col = 0;  col < ncolors;  ++col)
         count[col] = 0;
-    OIIO_DISPATCH_TYPES ("color_count", color_count_, src.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES (ok, "color_count", color_count_, src.spec().format,
                          src, (atomic_ll *)count, ncolors, color, eps,
                          roi, nthreads);
+    return ok;
 }
 
 
@@ -554,7 +571,7 @@ color_range_check_ (const ImageBuf &src, atomic_ll *lowcount,
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Lots of pixels and request for multi threads? Parallelize.
         ImageBufAlgo::parallel_image (
-            boost::bind(color_range_check_<T>, boost::ref(src),
+            OIIO::bind(color_range_check_<T>, OIIO::ref(src),
                         lowcount, highcount, inrangecount, low, high,
                         _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
@@ -607,11 +624,39 @@ ImageBufAlgo::color_range_check (const ImageBuf &src, imagesize_t *lowcount,
         *highcount = 0;
     if (inrangecount)
         *inrangecount = 0;
-    OIIO_DISPATCH_TYPES ("color_range_check", color_range_check_,
+    bool ok;
+    OIIO_DISPATCH_TYPES (ok, "color_range_check", color_range_check_,
                          src.spec().format,
                          src, (atomic_ll *)lowcount, 
                          (atomic_ll *)highcount, (atomic_ll *)inrangecount,
                          low, high, roi, nthreads);
+    return ok;
+}
+
+
+
+// Helper: is the roi devoid of any deep samples?
+static ROI
+deep_nonempty_region (const ImageBuf &src, ROI roi)
+{
+    DASSERT (src.deep());
+    ROI r;   // Initially undefined
+    for (int z = roi.zbegin; z < roi.zend; ++z)
+        for (int y = roi.ybegin; y < roi.yend; ++y)
+            for (int x = roi.xbegin; x < roi.xend; ++x)
+                if (src.deep_samples (x, y, z) != 0) {
+                    if (! r.defined()) {
+                        r = ROI (x, x+1, y, y+1, z, z+1, 0, src.nchannels());
+                    } else {
+                        r.xbegin = std::min (x,   r.xbegin);
+                        r.xend   = std::max (x+1, r.xend);
+                        r.ybegin = std::min (y,   r.ybegin);
+                        r.yend   = std::max (y+1, r.yend);
+                        r.zbegin = std::min (z,   r.zbegin);
+                        r.zend   = std::max (z+1, r.zend);
+                    }
+                }
+    return r;
 }
 
 
@@ -619,10 +664,14 @@ ImageBufAlgo::color_range_check (const ImageBuf &src, imagesize_t *lowcount,
 ROI
 ImageBufAlgo::nonzero_region (const ImageBuf &src, ROI roi, int nthreads)
 {
+    roi = roi_intersection (roi, src.roi());
+
+    if (src.deep()) {
+        return deep_nonempty_region (src, roi);
+    }
+
     std::vector<float> zero (src.nchannels(), 0.0f);
     std::vector<float> color (src.nchannels(), 0.0f);
-    if (! roi.defined())
-        roi = get_roi (src.spec());
     // Trim bottom
     for ( ; roi.ybegin < roi.yend; --roi.yend) {
         ROI test = roi;  test.ybegin = roi.yend-1;
@@ -671,7 +720,7 @@ namespace {
 
 std::string
 simplePixelHashSHA1 (const ImageBuf &src,
-                     const std::string & extrainfo, ROI roi)
+                     string_view extrainfo, ROI roi)
 {
     if (! roi.defined())
         roi = get_roi (src.spec());
@@ -708,7 +757,7 @@ simplePixelHashSHA1 (const ImageBuf &src,
     
     // If extra info is specified, also include it in the sha computation
     if (!extrainfo.empty())
-        SHA1_Update (&sha, extrainfo.c_str(), extrainfo.size());
+        SHA1_Update (&sha, extrainfo.data(), extrainfo.size());
 
     unsigned char md[SHA_DIGEST_LENGTH];
     char hash_digest[2*SHA_DIGEST_LENGTH+1];
@@ -739,7 +788,7 @@ simplePixelHashSHA1 (const ImageBuf &src,
     
     // If extra info is specified, also include it in the sha computation
     if (!extrainfo.empty()) {
-        sha.Update ((const unsigned char*) extrainfo.c_str(), extrainfo.size());
+        sha.Update ((const unsigned char*) extrainfo.data(), extrainfo.size());
     }
     
     sha.Final ();
@@ -773,7 +822,7 @@ sha1_hasher (const ImageBuf *src, ROI roi, int blocksize,
 
 std::string
 ImageBufAlgo::computePixelHashSHA1 (const ImageBuf &src,
-                                    const std::string & extrainfo,
+                                    string_view extrainfo,
                                     ROI roi, int blocksize, int nthreads)
 {
     if (! roi.defined())
@@ -793,7 +842,7 @@ ImageBufAlgo::computePixelHashSHA1 (const ImageBuf &src,
         sha1_hasher (&src, roi, blocksize, &results[0], 0);
     } else {
         // parallel case
-        boost::thread_group threads;
+        OIIO::thread_group threads;
         int blocks_per_thread = (nblocks+nthreads-1) / nthreads;
         ROI broi = roi;
         for (int b = 0, t = 0;  b < nblocks;  b += blocks_per_thread, ++t) {
@@ -802,7 +851,7 @@ ImageBufAlgo::computePixelHashSHA1 (const ImageBuf &src,
                 break;
             broi.ybegin = y;
             broi.yend = std::min (y+blocksize*blocks_per_thread, roi.yend);
-            threads.add_thread (new boost::thread (sha1_hasher, &src, broi,
+            threads.add_thread (new OIIO::thread (sha1_hasher, &src, broi,
                                                    blocksize, &results[0], b));
         }
         threads.join_all ();
@@ -983,5 +1032,4 @@ ImageBufAlgo::histogram_draw (ImageBuf &R,
 
 
 
-}
-OIIO_NAMESPACE_EXIT
+OIIO_NAMESPACE_END
