@@ -29,7 +29,6 @@
 */
 
 
-#include <boost/bind.hpp>
 #include <OpenEXR/half.h>
 
 #include <cmath>
@@ -40,26 +39,29 @@
 #include "OpenImageIO/imagebufalgo.h"
 #include "OpenImageIO/imagebufalgo_util.h"
 #include "OpenImageIO/dassert.h"
-#include "OpenImageIO/sysutil.h"
 #include "OpenImageIO/thread.h"
 
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
 
 
 // Helper for flatten: identify channels in the spec that are important to
 // deciphering deep images. Return true if appropriate alphas were found.
 static bool
 find_deep_channels (const ImageSpec &spec, int &alpha_channel,
-                    int &RA_channel, int &GA_channel, int &BA_channel,
+                    int &AR_channel, int &AG_channel, int &AB_channel,
                     int &R_channel, int &G_channel, int &B_channel,
                     int &Z_channel, int &Zback_channel)
 {
-    static const char *names[] = { "A", "RA", "GA", "BA",
-                                   "R", "G", "B", "Z", "Zback", NULL };
-    int *chans[] = { &alpha_channel, &RA_channel, &GA_channel, &BA_channel,
-                     &R_channel, &G_channel, &B_channel, 
+    static const char *names[] = { "A",
+                                   "RA", "GA", "BA", // old OpenEXR recommendation
+                                   "AR", "AG", "AB", // new OpenEXR recommendation
+                                   "R", "G", "B",
+                                   "Z", "Zback", NULL };
+    int *chans[] = { &alpha_channel,
+                     &AR_channel, &AG_channel, &AB_channel,
+                     &AR_channel, &AG_channel, &AB_channel,
+                     &R_channel, &G_channel, &B_channel,
                      &Z_channel, &Zback_channel };
     for (int i = 0;  names[i];  ++i)
         *chans[i] = -1;
@@ -74,7 +76,7 @@ find_deep_channels (const ImageSpec &spec, int &alpha_channel,
     if (Zback_channel < 0)
         Zback_channel = Z_channel;
     return (alpha_channel >= 0 ||
-            (RA_channel >= 0 && GA_channel >= 0 && BA_channel >= 0));
+            (AR_channel >= 0 && AG_channel >= 0 && AB_channel >= 0));
 }
 
 
@@ -89,7 +91,7 @@ flatten_ (ImageBuf &dst, const ImageBuf &src,
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Possible multiple thread case -- recurse via parallel_image
         ImageBufAlgo::parallel_image (
-            boost::bind(flatten_<DSTTYPE>, boost::ref(dst), boost::cref(src),
+            OIIO::bind(flatten_<DSTTYPE>, OIIO::ref(dst), OIIO::cref(src),
                         _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
         return true;
@@ -98,22 +100,22 @@ flatten_ (ImageBuf &dst, const ImageBuf &src,
     const ImageSpec &srcspec (src.spec());
     int nc = srcspec.nchannels;
 
-    int alpha_channel, RA_channel, GA_channel, BA_channel;
+    int alpha_channel, AR_channel, AG_channel, AB_channel;
     int R_channel, G_channel, B_channel;
     int Z_channel, Zback_channel;
     if (! find_deep_channels (srcspec, alpha_channel,
-                              RA_channel, GA_channel, BA_channel,
+                              AR_channel, AG_channel, AB_channel,
                               R_channel, G_channel, B_channel,
                               Z_channel, Zback_channel)) {
         dst.error ("No alpha channel could be identified");
         return false;
     }
     ASSERT (alpha_channel >= 0 ||
-            (RA_channel >= 0 && GA_channel >= 0 && BA_channel >= 0));
+            (AR_channel >= 0 && AG_channel >= 0 && AB_channel >= 0));
     float *val = ALLOCA (float, nc);
-    float &RAval (RA_channel >= 0 ? val[RA_channel] : val[alpha_channel]);
-    float &GAval (GA_channel >= 0 ? val[GA_channel] : val[alpha_channel]);
-    float &BAval (BA_channel >= 0 ? val[BA_channel] : val[alpha_channel]);
+    float &ARval (AR_channel >= 0 ? val[AR_channel] : val[alpha_channel]);
+    float &AGval (AG_channel >= 0 ? val[AG_channel] : val[alpha_channel]);
+    float &ABval (AB_channel >= 0 ? val[AB_channel] : val[alpha_channel]);
 
     for (ImageBuf::Iterator<DSTTYPE> r (dst, roi);  !r.done();  ++r) {
         int x = r.x(), y = r.y(), z = r.z();
@@ -125,8 +127,8 @@ flatten_ (ImageBuf &dst, const ImageBuf &src,
         if (Zback_channel >= 0 && samps == 0)
             val[Zback_channel] = 1.0e30;
         for (int s = 0;  s < samps;  ++s) {
-            float RA = RAval, GA = GAval, BA = BAval;  // make copies
-            float alpha = (RA + GA + BA) / 3.0f;
+            float AR = ARval, AG = AGval, AB = ABval;  // make copies
+            float alpha = (AR + AG + AB) / 3.0f;
             if (alpha >= 1.0f)
                 break;
             for (int c = 0;  c < nc;  ++c) {
@@ -135,11 +137,11 @@ flatten_ (ImageBuf &dst, const ImageBuf &src,
                     val[c] *= alpha;  // because Z are not premultiplied
                 float a;
                 if (c == R_channel)
-                    a = RA;
+                    a = AR;
                 else if (c == G_channel)
-                    a = GA;
+                    a = AG;
                 else if (c == B_channel)
-                    a = BA;
+                    a = AB;
                 else
                     a = alpha;
                 val[c] += (1.0f - a) * v;
@@ -158,12 +160,19 @@ bool
 ImageBufAlgo::flatten (ImageBuf &dst, const ImageBuf &src,
                        ROI roi, int nthreads)
 {
+    if (! src.deep()) {
+        // For some reason, we were asked to flatten an already-flat image.
+        // So just copy it.
+        return dst.copy (src);
+    }
+
     // Construct an ideal spec for dst, which is like src but not deep.
     ImageSpec force_spec = src.spec();
     force_spec.deep = false;
     force_spec.channelformats.clear();
 
-    if (! IBAprep (roi, &dst, &src, NULL, &force_spec))
+    if (! IBAprep (roi, &dst, &src, NULL, &force_spec,
+                   IBAprep_SUPPORT_DEEP | IBAprep_DEEP_MIXED))
         return false;
     if (dst.spec().deep) {
         dst.error ("Cannot flatten to a deep image");
@@ -171,22 +180,115 @@ ImageBufAlgo::flatten (ImageBuf &dst, const ImageBuf &src,
     }
 
     const ImageSpec &srcspec (src.spec());
-    int alpha_channel, RA_channel, GA_channel, BA_channel;
+    int alpha_channel, AR_channel, AG_channel, AB_channel;
     int R_channel, G_channel, B_channel, Z_channel, Zback_channel;
     if (! find_deep_channels (srcspec, alpha_channel,
-                              RA_channel, GA_channel, BA_channel,
+                              AR_channel, AG_channel, AB_channel,
                               R_channel, G_channel, B_channel,
                               Z_channel, Zback_channel)) {
         dst.error ("No alpha channel could be identified");
         return false;
     }
 
-    OIIO_DISPATCH_TYPES ("flatten", flatten_, dst.spec().format,
+    bool ok;
+    OIIO_DISPATCH_TYPES (ok, "flatten", flatten_, dst.spec().format,
                          dst, src, roi, nthreads);
-    return false;
+    return ok;
 }
 
 
 
+bool
+ImageBufAlgo::deepen (ImageBuf &dst, const ImageBuf &src, float zvalue,
+                      ROI roi, int nthreads)
+{
+    if (src.deep()) {
+        // For some reason, we were asked to deepen an already-deep image.
+        // So just copy it.
+        return dst.copy (src);
+        // FIXME: once paste works for deep files, this should really be
+        // return paste (dst, roi.xbegin, roi.ybegin, roi.zbegin, roi.chbegin,
+        //               src, roi, nthreads);
+    }
+
+    // Construct an ideal spec for dst, which is like src but deep.
+    const ImageSpec &srcspec (src.spec());
+    int nc = srcspec.nchannels;
+    int zback_channel = -1;
+    ImageSpec force_spec = srcspec;
+    force_spec.deep = true;
+    force_spec.set_format (TypeDesc::FLOAT);
+    force_spec.channelformats.clear();
+    for (int c = 0; c < nc; ++c) {
+        if (force_spec.channelnames[c] == "Z")
+            force_spec.z_channel = c;
+        else if (force_spec.channelnames[c] == "Zback")
+            zback_channel = c;
+    }
+    bool add_z_channel = (force_spec.z_channel < 0);
+    if (add_z_channel) {
+        // No z channel? Make one.
+        force_spec.z_channel = force_spec.nchannels++;
+        force_spec.channelnames.push_back ("Z");
+    }
+
+    if (! IBAprep (roi, &dst, &src, NULL, &force_spec,
+                   IBAprep_SUPPORT_DEEP | IBAprep_DEEP_MIXED))
+        return false;
+    if (! dst.deep()) {
+        dst.error ("Cannot deepen to a flat image");
+        return false;
+    }
+
+    float *pixel = OIIO_ALLOCA (float, nc);
+
+    // First, figure out which pixels get a sample and which do not
+    for (int z = roi.zbegin; z < roi.zend; ++z)
+    for (int y = roi.ybegin; y < roi.yend; ++y)
+    for (int x = roi.xbegin; x < roi.xend; ++x) {
+        bool has_sample = false;
+        src.getpixel (x, y, z, pixel);
+        for (int c = 0; c < nc; ++c)
+            if (c != force_spec.z_channel && c != zback_channel
+                  && pixel[c] != 0.0f) {
+                has_sample = true;
+                break;
+            }
+        if (! has_sample && ! add_z_channel)
+            for (int c = 0; c < nc; ++c)
+                if ((c == force_spec.z_channel || c == zback_channel)
+                    && (pixel[c] != 0.0f && pixel[c] < 1e30)) {
+                    has_sample = true;
+                    break;
+                }
+        if (has_sample)
+            dst.set_deep_samples (x, y, z, 1);
+    }
+
+    dst.deep_alloc ();
+
+    // Now actually set the values
+    for (int z = roi.zbegin; z < roi.zend; ++z)
+    for (int y = roi.ybegin; y < roi.yend; ++y)
+    for (int x = roi.xbegin; x < roi.xend; ++x) {
+        if (dst.deep_samples (x, y, z) == 0)
+            continue;
+        for (int c = 0; c < nc; ++c)
+            dst.set_deep_value (x, y, z, c, 0 /*sample*/,
+                                src.getchannel (x, y, z, c));
+        if (add_z_channel)
+            dst.set_deep_value (x, y, z, nc, 0, zvalue);
+    }
+
+    bool ok = true;
+    // FIXME -- the above doesn't split into threads. Someday, it should
+    // be refactored like this:
+    // OIIO_DISPATCH_COMMON_TYPES2 (ok, "deepen", deepen_,
+    //                              dst.spec().format, srcspec.format,
+    //                              dst, src, add_z_channel, z, roi, nthreads);
+    return ok;
 }
-OIIO_NAMESPACE_EXIT
+
+
+
+OIIO_NAMESPACE_END
