@@ -32,8 +32,6 @@
 /// Implementation of ImageBufAlgo algorithms that do math on 
 /// single pixels at a time.
 
-#include <boost/bind.hpp>
-
 #include <OpenEXR/half.h>
 
 #include <cmath>
@@ -43,12 +41,13 @@
 #include "OpenImageIO/imagebuf.h"
 #include "OpenImageIO/imagebufalgo.h"
 #include "OpenImageIO/imagebufalgo_util.h"
+// #include "OpenImageIO/deepdata.h"
+#include "OpenImageIO/array_view.h"
 #include "OpenImageIO/dassert.h"
 
 
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
 
 
 template<class D, class S>
@@ -60,7 +59,7 @@ clamp_ (ImageBuf &dst, const ImageBuf &src,
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Lots of pixels and request for multi threads? Parallelize.
         ImageBufAlgo::parallel_image (
-            boost::bind(clamp_<D,S>, boost::ref(dst), boost::cref(src),
+            OIIO::bind(clamp_<D,S>, OIIO::ref(dst), OIIO::cref(src),
                         min, max, clampalpha01,
                         _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
@@ -128,8 +127,8 @@ add_impl (ImageBuf &R, const ImageBuf &A, const ImageBuf &B,
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Possible multiple thread case -- recurse via parallel_image
         ImageBufAlgo::parallel_image (
-            boost::bind(add_impl<Rtype,Atype,Btype>,
-                        boost::ref(R), boost::cref(A), boost::cref(B),
+            OIIO::bind(add_impl<Rtype,Atype,Btype>,
+                        OIIO::ref(R), OIIO::cref(A), OIIO::cref(B),
                         _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
         return true;
@@ -155,14 +154,32 @@ add_impl (ImageBuf &R, const ImageBuf &A, const float *b,
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Possible multiple thread case -- recurse via parallel_image
         ImageBufAlgo::parallel_image (
-            boost::bind(add_impl<Rtype,Atype>,
-                        boost::ref(R), boost::cref(A), b,
+            OIIO::bind(add_impl<Rtype,Atype>,
+                        OIIO::ref(R), OIIO::cref(A), b,
                         _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
         return true;
     }
+    // Serial case:
+
+    if (R.deep()) {
+        // Deep case
+        array_view<TypeDesc> channeltypes (R.deepdata()->channeltypes);
+        ImageBuf::Iterator<Rtype> r (R, roi);
+        ImageBuf::ConstIterator<Atype> a (A, roi);
+        for ( ;  !r.done();  ++r, ++a) {
+            for (int samp = 0, samples = r.deep_samples(); samp < samples; ++samp) {
+                for (int c = roi.chbegin;  c < roi.chend;  ++c) {
+                    if (channeltypes[c].basetype == TypeDesc::UINT32)
+                        r.set_deep_value (c, samp, a.deep_value_uint(c, samp));
+                    else
+                        r.set_deep_value (c, samp, a.deep_value(c, samp) + b[c]);
+                }
+            }
+        }
+        return true;
+    }
 
-    // Serial case
     ImageBuf::Iterator<Rtype> r (R, roi);
     ImageBuf::ConstIterator<Atype> a (A, roi);
     for ( ;  !r.done();  ++r, ++a)
@@ -177,7 +194,7 @@ bool
 ImageBufAlgo::add (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
                    ROI roi, int nthreads)
 {
-    if (! IBAprep (roi, &dst, &A, &B))
+    if (! IBAprep (roi, &dst, &A, &B, NULL, IBAprep_CLAMP_MUTUAL_NCHANNELS))
         return false;
     bool ok;
     OIIO_DISPATCH_COMMON_TYPES3 (ok, "add", add_impl, dst.spec().format,
@@ -192,10 +209,18 @@ bool
 ImageBufAlgo::add (ImageBuf &dst, const ImageBuf &A, const float *b,
                    ROI roi, int nthreads)
 {
-    if (! IBAprep (roi, &dst, &A))
+    if (! IBAprep (roi, &dst, &A,
+                   IBAprep_CLAMP_MUTUAL_NCHANNELS | IBAprep_SUPPORT_DEEP))
         return false;
+
+    if (dst.deep()) {
+        // While still serial, set up all the sample counts
+        dst.deepdata()->nsamples = A.deepdata()->nsamples;
+        dst.deep_alloc ();
+    }
+
     bool ok;
-    OIIO_DISPATCH_TYPES2 (ok, "add", add_impl, dst.spec().format,
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "add", add_impl, dst.spec().format,
                           A.spec().format, dst, A, b, roi, nthreads);
     return ok;
 }
@@ -206,14 +231,14 @@ bool
 ImageBufAlgo::add (ImageBuf &dst, const ImageBuf &A, float b,
                    ROI roi, int nthreads)
 {
-    if (! IBAprep (roi, &dst, &A))
+    if (! IBAprep (roi, &dst, &A, IBAprep_CLAMP_MUTUAL_NCHANNELS))
         return false;
     int nc = A.nchannels();
     float *vals = ALLOCA (float, nc);
     for (int c = 0;  c < nc;  ++c)
         vals[c] = b;
     bool ok;
-    OIIO_DISPATCH_TYPES2 (ok, "add", add_impl, dst.spec().format,
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "add", add_impl, dst.spec().format,
                           A.spec().format, dst, A, vals, roi, nthreads);
     return ok;
 }
@@ -229,8 +254,8 @@ sub_impl (ImageBuf &R, const ImageBuf &A, const ImageBuf &B,
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Possible multiple thread case -- recurse via parallel_image
         ImageBufAlgo::parallel_image (
-            boost::bind(sub_impl<Rtype,Atype,Btype>,
-                        boost::ref(R), boost::cref(A), boost::cref(B),
+            OIIO::bind(sub_impl<Rtype,Atype,Btype>,
+                        OIIO::ref(R), OIIO::cref(A), OIIO::cref(B),
                         _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
         return true;
@@ -252,7 +277,7 @@ bool
 ImageBufAlgo::sub (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
                    ROI roi, int nthreads)
 {
-    if (! IBAprep (roi, &dst, &A, &B))
+    if (! IBAprep (roi, &dst, &A, &B, NULL, IBAprep_CLAMP_MUTUAL_NCHANNELS))
         return false;
     bool ok;
     OIIO_DISPATCH_COMMON_TYPES3 (ok, "sub", sub_impl, dst.spec().format,
@@ -267,14 +292,22 @@ bool
 ImageBufAlgo::sub (ImageBuf &dst, const ImageBuf &A, const float *b,
                    ROI roi, int nthreads)
 {
-    if (! IBAprep (roi, &dst, &A))
+    if (! IBAprep (roi, &dst, &A,
+                   IBAprep_CLAMP_MUTUAL_NCHANNELS | IBAprep_SUPPORT_DEEP))
         return false;
+
+    if (dst.deep()) {
+        // While still serial, set up all the sample counts
+        dst.deepdata()->nsamples = A.deepdata()->nsamples;
+        dst.deep_alloc ();
+    }
+
     int nc = A.nchannels();
     float *vals = ALLOCA (float, nc);
     for (int c = 0;  c < nc;  ++c)
         vals[c] = -b[c];
     bool ok;
-    OIIO_DISPATCH_TYPES2 (ok, "sub", add_impl, dst.spec().format,
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "sub", add_impl, dst.spec().format,
                           A.spec().format, dst, A, vals, roi, nthreads);
     return ok;
 }
@@ -285,20 +318,130 @@ bool
 ImageBufAlgo::sub (ImageBuf &dst, const ImageBuf &A, float b,
                    ROI roi, int nthreads)
 {
-    if (! IBAprep (roi, &dst, &A))
+    if (! IBAprep (roi, &dst, &A, IBAprep_CLAMP_MUTUAL_NCHANNELS))
         return false;
     int nc = A.nchannels();
     float *vals = ALLOCA (float, nc);
     for (int c = 0;  c < nc;  ++c)
         vals[c] = -b;
     bool ok;
-    OIIO_DISPATCH_TYPES2 (ok, "sub", add_impl, dst.spec().format,
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "sub", add_impl, dst.spec().format,
                           A.spec().format, dst, A, vals, roi, nthreads);
     return ok;
 }
 
 
 
+template<class Rtype, class Atype, class Btype>
+static bool
+absdiff_impl (ImageBuf &R, const ImageBuf &A, const ImageBuf &B,
+              ROI roi, int nthreads)
+{
+    if (nthreads != 1 && roi.npixels() >= 1000) {
+        // Possible multiple thread case -- recurse via parallel_image
+        ImageBufAlgo::parallel_image (
+            OIIO::bind(absdiff_impl<Rtype,Atype,Btype>,
+                        OIIO::ref(R), OIIO::cref(A), OIIO::cref(B),
+                        _1 /*roi*/, 1 /*nthreads*/),
+            roi, nthreads);
+        return true;
+    }
+
+    // Serial case
+    ImageBuf::Iterator<Rtype> r (R, roi);
+    ImageBuf::ConstIterator<Atype> a (A, roi);
+    ImageBuf::ConstIterator<Btype> b (B, roi);
+    for ( ;  !r.done();  ++r, ++a, ++b)
+        for (int c = roi.chbegin;  c < roi.chend;  ++c)
+            r[c] = std::abs (a[c] - b[c]);
+    return true;
+}
+
+
+template<class Rtype, class Atype>
+static bool
+absdiff_impl (ImageBuf &R, const ImageBuf &A, const float *b,
+              ROI roi, int nthreads)
+{
+    if (nthreads != 1 && roi.npixels() >= 1000) {
+        // Possible multiple thread case -- recurse via parallel_image
+        ImageBufAlgo::parallel_image (
+            OIIO::bind(absdiff_impl<Rtype,Atype>,
+                        OIIO::ref(R), OIIO::cref(A), b,
+                        _1 /*roi*/, 1 /*nthreads*/),
+            roi, nthreads);
+        return true;
+    }
+
+    // Serial case
+    ImageBuf::Iterator<Rtype> r (R, roi);
+    ImageBuf::ConstIterator<Atype> a (A, roi);
+    for ( ;  !r.done();  ++r, ++a)
+        for (int c = roi.chbegin;  c < roi.chend;  ++c)
+            r[c] = std::abs (a[c] - b[c]);
+    return true;
+}
+
+
+
+
+bool
+ImageBufAlgo::absdiff (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
+                       ROI roi, int nthreads)
+{
+    if (! IBAprep (roi, &dst, &A, &B, NULL, IBAprep_CLAMP_MUTUAL_NCHANNELS))
+        return false;
+    bool ok;
+    OIIO_DISPATCH_COMMON_TYPES3 (ok, "absdiff", absdiff_impl, dst.spec().format,
+                                 A.spec().format, B.spec().format,
+                                 dst, A, B, roi, nthreads);
+    return ok;
+}
+
+
+
+bool
+ImageBufAlgo::absdiff (ImageBuf &dst, const ImageBuf &A, const float *b,
+                       ROI roi, int nthreads)
+{
+    if (! IBAprep (roi, &dst, &A, IBAprep_CLAMP_MUTUAL_NCHANNELS))
+        return false;
+    bool ok;
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "absdiff", absdiff_impl, dst.spec().format,
+                          A.spec().format, dst, A, b, roi, nthreads);
+    return ok;
+}
+
+
+
+bool
+ImageBufAlgo::absdiff (ImageBuf &dst, const ImageBuf &A, float b,
+                       ROI roi, int nthreads)
+{
+    if (! IBAprep (roi, &dst, &A, IBAprep_CLAMP_MUTUAL_NCHANNELS))
+        return false;
+    int nc = A.nchannels();
+    float *vals = ALLOCA (float, nc);
+    for (int c = 0;  c < nc;  ++c)
+        vals[c] = b;
+    bool ok;
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "absdiff", absdiff_impl, dst.spec().format,
+                          A.spec().format, dst, A, vals, roi, nthreads);
+    return ok;
+}
+
+
+
+bool
+ImageBufAlgo::abs (ImageBuf &dst, const ImageBuf &A, ROI roi, int nthreads)
+{
+    // Define abs in terms of absdiff(A,0.0)
+    return absdiff (dst, A, 0.0f, roi, nthreads);
+}
+
+
+
+
 template<class Rtype, class Atype, class Btype>
 static bool
 mul_impl (ImageBuf &R, const ImageBuf &A, const ImageBuf &B,
@@ -307,8 +450,8 @@ mul_impl (ImageBuf &R, const ImageBuf &A, const ImageBuf &B,
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Possible multiple thread case -- recurse via parallel_image
         ImageBufAlgo::parallel_image (
-            boost::bind(mul_impl<Rtype,Atype,Btype>,
-                        boost::ref(R), boost::cref(A), boost::cref(B),
+            OIIO::bind(mul_impl<Rtype,Atype,Btype>,
+                        OIIO::ref(R), OIIO::cref(A), OIIO::cref(B),
                         _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
         return true;
@@ -330,7 +473,7 @@ bool
 ImageBufAlgo::mul (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
                    ROI roi, int nthreads)
 {
-    if (! IBAprep (roi, &dst, &A, &B))
+    if (! IBAprep (roi, &dst, &A, &B, NULL, IBAprep_CLAMP_MUTUAL_NCHANNELS))
         return false;
     bool ok;
     OIIO_DISPATCH_COMMON_TYPES3 (ok, "mul", mul_impl, dst.spec().format,
@@ -349,11 +492,30 @@ mul_impl (ImageBuf &R, const ImageBuf &A, const float *b,
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Possible multiple thread case -- recurse via parallel_image
         ImageBufAlgo::parallel_image (
-            boost::bind(mul_impl<Rtype,Atype>, boost::ref(R), boost::cref(A), b,
+            OIIO::bind(mul_impl<Rtype,Atype>, OIIO::ref(R), OIIO::cref(A), b,
                         _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
         return true;
     }
+    // Serial case:
+
+    if (R.deep()) {
+        // Deep case
+        array_view<TypeDesc> channeltypes (R.deepdata()->channeltypes);
+        ImageBuf::Iterator<Rtype> r (R, roi);
+        ImageBuf::ConstIterator<Atype> a (A, roi);
+        for ( ;  !r.done();  ++r, ++a) {
+            for (int samp = 0, samples = r.deep_samples(); samp < samples; ++samp) {
+                for (int c = roi.chbegin;  c < roi.chend;  ++c) {
+                    if (channeltypes[c].basetype == TypeDesc::UINT32)
+                        r.set_deep_value (c, samp, a.deep_value_uint(c, samp));
+                    else
+                        r.set_deep_value (c, samp, a.deep_value(c, samp) * b[c]);
+                }
+            }
+        }
+        return true;
+    }
 
     ImageBuf::ConstIterator<Atype> a (A, roi);
     for (ImageBuf::Iterator<Rtype> r (R, roi);  !r.done();  ++r, ++a)
@@ -367,10 +529,18 @@ bool
 ImageBufAlgo::mul (ImageBuf &dst, const ImageBuf &A, const float *b,
                    ROI roi, int nthreads)
 {
-    if (! IBAprep (roi, &dst, &A))
+    if (! IBAprep (roi, &dst, &A,
+                   IBAprep_CLAMP_MUTUAL_NCHANNELS | IBAprep_SUPPORT_DEEP))
         return false;
+
+    if (dst.deep()) {
+        // While still serial, set up all the sample counts
+        dst.deepdata()->nsamples = A.deepdata()->nsamples;
+        dst.deep_alloc ();
+    }
+
     bool ok;
-    OIIO_DISPATCH_TYPES2 (ok, "mul", mul_impl, dst.spec().format,
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "mul", mul_impl, dst.spec().format,
                           A.spec().format, dst, A, b, roi, nthreads);
     return ok;
 }
@@ -381,14 +551,14 @@ bool
 ImageBufAlgo::mul (ImageBuf &dst, const ImageBuf &A, float b,
                    ROI roi, int nthreads)
 {
-    if (! IBAprep (roi, &dst, &A))
+    if (! IBAprep (roi, &dst, &A, IBAprep_CLAMP_MUTUAL_NCHANNELS))
         return false;
     int nc = A.nchannels();
     float *vals = ALLOCA (float, nc);
     for (int c = 0;  c < nc;  ++c)
         vals[c] = b;
     bool ok;
-    OIIO_DISPATCH_TYPES2 (ok, "mul", mul_impl, dst.spec().format,
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "mul", mul_impl, dst.spec().format,
                           A.spec().format, dst, A, vals, roi, nthreads);
     return ok;
 }
@@ -396,6 +566,239 @@ ImageBufAlgo::mul (ImageBuf &dst, const ImageBuf &A, float b,
 
 
 
+template<class Rtype, class Atype, class Btype>
+static bool
+div_impl (ImageBuf &R, const ImageBuf &A, const ImageBuf &B,
+          ROI roi, int nthreads)
+{
+    if (nthreads != 1 && roi.npixels() >= 1000) {
+        // Possible multiple thread case -- recurse via parallel_image
+        ImageBufAlgo::parallel_image (
+            OIIO::bind(div_impl<Rtype,Atype,Btype>,
+                        OIIO::ref(R), OIIO::cref(A), OIIO::cref(B),
+                        _1 /*roi*/, 1 /*nthreads*/),
+            roi, nthreads);
+        return true;
+    }
+
+    // Serial case
+    ImageBuf::Iterator<Rtype> r (R, roi);
+    ImageBuf::ConstIterator<Atype> a (A, roi);
+    ImageBuf::ConstIterator<Btype> b (B, roi);
+    for ( ;  !r.done();  ++r, ++a, ++b)
+        for (int c = roi.chbegin;  c < roi.chend;  ++c) {
+            float v = b[c];
+            r[c] = (v == 0.0f) ? 0.0f : (a[c] / v);
+        }
+    return true;
+}
+
+
+
+bool
+ImageBufAlgo::div (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
+                   ROI roi, int nthreads)
+{
+    if (! IBAprep (roi, &dst, &A, &B, NULL, IBAprep_CLAMP_MUTUAL_NCHANNELS))
+        return false;
+    bool ok;
+    OIIO_DISPATCH_COMMON_TYPES3 (ok, "div", div_impl, dst.spec().format,
+                                 A.spec().format, B.spec().format,
+                                 dst, A, B, roi, nthreads);
+    return ok;
+}
+
+
+
+bool
+ImageBufAlgo::div (ImageBuf &dst, const ImageBuf &A, const float *b,
+                   ROI roi, int nthreads)
+{
+    if (! IBAprep (roi, &dst, &A,
+                   IBAprep_CLAMP_MUTUAL_NCHANNELS | IBAprep_SUPPORT_DEEP))
+        return false;
+
+    if (dst.deep()) {
+        // While still serial, set up all the sample counts
+        dst.deepdata()->nsamples = A.deepdata()->nsamples;
+        dst.deep_alloc ();
+    }
+
+    int nc = dst.nchannels();
+    float *binv = OIIO_ALLOCA (float, nc);
+    for (int c = 0; c < nc; ++c)
+        binv[c] = (b[c] == 0.0f) ? 0.0f : 1.0f/b[c];
+    bool ok;
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "div", mul_impl, dst.spec().format,
+                          A.spec().format, dst, A, binv, roi, nthreads);
+    return ok;
+}
+
+
+
+bool
+ImageBufAlgo::div (ImageBuf &dst, const ImageBuf &A, float b,
+                   ROI roi, int nthreads)
+{
+    if (! IBAprep (roi, &dst, &A, IBAprep_CLAMP_MUTUAL_NCHANNELS))
+        return false;
+    b = (b == 0.0f) ? 1.0f : 1.0f/b;
+    int nc = dst.nchannels();
+    float *binv = ALLOCA (float, nc);
+    for (int c = 0;  c < nc;  ++c)
+        binv[c] = b;
+    bool ok;
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "div", mul_impl, dst.spec().format,
+                          A.spec().format, dst, A, binv, roi, nthreads);
+    return ok;
+}
+
+
+
+template<class Rtype, class ABCtype>
+static bool
+mad_impl (ImageBuf &R, const ImageBuf &A, const ImageBuf &B, const ImageBuf &C,
+          ROI roi, int nthreads)
+{
+    if (nthreads != 1 && roi.npixels() >= 1000) {
+        // Possible multiple thread case -- recurse via parallel_image
+        ImageBufAlgo::parallel_image (
+            OIIO::bind(mad_impl<Rtype,ABCtype>, OIIO::ref(R),
+                        OIIO::cref(A), OIIO::cref(B), OIIO::cref(C),
+                        _1 /*roi*/, 1 /*nthreads*/),
+            roi, nthreads);
+        return true;
+    }
+
+    // Serial case
+    ImageBuf::Iterator<Rtype> r (R, roi);
+    ImageBuf::ConstIterator<ABCtype> a (A, roi);
+    ImageBuf::ConstIterator<ABCtype> b (B, roi);
+    ImageBuf::ConstIterator<ABCtype> c (C, roi);
+    for ( ;  !r.done();  ++r, ++a, ++b, ++c)
+        for (int ch = roi.chbegin;  ch < roi.chend;  ++ch)
+            r[ch] = a[ch] * b[ch] + c[ch];
+    return true;
+}
+
+
+
+template<class Rtype, class Atype>
+static bool
+mad_implf (ImageBuf &R, const ImageBuf &A, const float *b, const float *c,
+          ROI roi, int nthreads)
+{
+    if (nthreads != 1 && roi.npixels() >= 1000) {
+        // Possible multiple thread case -- recurse via parallel_image
+        ImageBufAlgo::parallel_image (
+            OIIO::bind(mad_implf<Rtype,Atype>, OIIO::ref(R),
+                        OIIO::cref(A), b, c,
+                        _1 /*roi*/, 1 /*nthreads*/),
+            roi, nthreads);
+        return true;
+    }
+
+    // Serial case
+    ImageBuf::Iterator<Rtype> r (R, roi);
+    ImageBuf::ConstIterator<Atype> a (A, roi);
+    for ( ;  !r.done();  ++r, ++a)
+        for (int ch = roi.chbegin;  ch < roi.chend;  ++ch)
+            r[ch] = a[ch] * b[ch] + c[ch];
+    return true;
+}
+
+
+
+bool
+ImageBufAlgo::mad (ImageBuf &dst, const ImageBuf &A_, const ImageBuf &B_,
+                   const ImageBuf &C_, ROI roi, int nthreads)
+{
+    const ImageBuf *A = &A_, *B = &B_, *C = &C_;
+    if (!A->initialized() || !B->initialized() || !C->initialized()) {
+        dst.error ("Uninitialized input image");
+        return false;
+    }
+
+    // To avoid the full cross-product of dst/A/B/C types, force A,B,C to
+    // all be the same data type, copying if we have to.
+    TypeDesc abc_type = type_merge (A->spec().format, B->spec().format,
+                                    C->spec().format);
+    ImageBuf Anew, Bnew, Cnew;
+    if (A->spec().format != abc_type) {
+        Anew.copy (*A, abc_type);
+        A = &Anew;
+    }
+    if (B->spec().format != abc_type) {
+        Bnew.copy (*B, abc_type);
+        B = &Bnew;
+    }
+    if (C->spec().format != abc_type) {
+        Cnew.copy (*C, abc_type);
+        C = &Cnew;
+    }
+    ASSERT (A->spec().format == B->spec().format &&
+            A->spec().format == C->spec().format);
+
+    if (! IBAprep (roi, &dst, A, B, C))
+        return false;
+    bool ok;
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "mad", mad_impl, dst.spec().format,
+                                 abc_type, dst, *A, *B, *C, roi, nthreads);
+    return ok;
+}
+
+
+
+bool
+ImageBufAlgo::mad (ImageBuf &dst, const ImageBuf &A, const float *B,
+                   const float *C, ROI roi, int nthreads)
+{
+    if (!A.initialized()) {
+        dst.error ("Uninitialized input image");
+        return false;
+    }
+    if (! IBAprep (roi, &dst, &A))
+        return false;
+    bool ok;
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "mad", mad_implf, dst.spec().format,
+                                 A.spec().format, dst, A, B, C,
+                                 roi, nthreads);
+    return ok;
+}
+
+
+
+bool
+ImageBufAlgo::mad (ImageBuf &dst, const ImageBuf &A, float b,
+                   float c, ROI roi, int nthreads)
+{
+    if (!A.initialized()) {
+        dst.error ("Uninitialized input image");
+        return false;
+    }
+    if (! IBAprep (roi, &dst, &A))
+        return false;
+    std::vector<float> B (roi.chend, b);
+    std::vector<float> C (roi.chend, c);
+    bool ok;
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "mad", mad_implf, dst.spec().format,
+                                 A.spec().format, dst, A, &B[0], &C[0],
+                                 roi, nthreads);
+    return ok;
+}
+
+
+
+bool
+ImageBufAlgo::invert (ImageBuf &dst, const ImageBuf &A,
+                      ROI roi, int nthreads)
+{
+    // Calculate invert as simply 1-A == A*(-1)+1
+    return mad (dst, A, -1.0, 1.0, roi, nthreads);
+}
+
+
+
 template<class Rtype, class Atype>
 static bool
 pow_impl (ImageBuf &R, const ImageBuf &A, const float *b,
@@ -404,7 +807,7 @@ pow_impl (ImageBuf &R, const ImageBuf &A, const float *b,
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Possible multiple thread case -- recurse via parallel_image
         ImageBufAlgo::parallel_image (
-            boost::bind(pow_impl<Rtype,Atype>, boost::ref(R), boost::cref(A), b,
+            OIIO::bind(pow_impl<Rtype,Atype>, OIIO::ref(R), OIIO::cref(A), b,
                         _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
         return true;
@@ -422,10 +825,10 @@ bool
 ImageBufAlgo::pow (ImageBuf &dst, const ImageBuf &A, const float *b,
                    ROI roi, int nthreads)
 {
-    if (! IBAprep (roi, &dst, &A))
+    if (! IBAprep (roi, &dst, &A, IBAprep_CLAMP_MUTUAL_NCHANNELS))
         return false;
     bool ok;
-    OIIO_DISPATCH_TYPES2 (ok, "pow", pow_impl, dst.spec().format,
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "pow", pow_impl, dst.spec().format,
                           A.spec().format, dst, A, b, roi, nthreads);
     return ok;
 }
@@ -436,14 +839,14 @@ bool
 ImageBufAlgo::pow (ImageBuf &dst, const ImageBuf &A, float b,
                    ROI roi, int nthreads)
 {
-    if (! IBAprep (roi, &dst, &A))
+    if (! IBAprep (roi, &dst, &A, IBAprep_CLAMP_MUTUAL_NCHANNELS))
         return false;
     int nc = A.nchannels();
     float *vals = ALLOCA (float, nc);
     for (int c = 0;  c < nc;  ++c)
         vals[c] = b;
     bool ok;
-    OIIO_DISPATCH_TYPES2 (ok, "pow", pow_impl, dst.spec().format,
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "pow", pow_impl, dst.spec().format,
                           A.spec().format, dst, A, vals, roi, nthreads);
     return ok;
 }
@@ -460,7 +863,7 @@ channel_sum_ (ImageBuf &dst, const ImageBuf &src,
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Possible multiple thread case -- recurse via parallel_image
         ImageBufAlgo::parallel_image (
-            boost::bind(channel_sum_<D,S>, boost::ref(dst), boost::cref(src),
+            OIIO::bind(channel_sum_<D,S>, OIIO::ref(dst), OIIO::cref(src),
                         weights, _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
         return true;
@@ -562,8 +965,8 @@ rangecompress_ (ImageBuf &R, const ImageBuf &A,
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Possible multiple thread case -- recurse via parallel_image
         ImageBufAlgo::parallel_image (
-            boost::bind(rangecompress_<Rtype,Atype>, boost::ref(R),
-                        boost::cref(A), useluma,
+            OIIO::bind(rangecompress_<Rtype,Atype>, OIIO::ref(R),
+                        OIIO::cref(A), useluma,
                         _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
         return true;
@@ -632,8 +1035,8 @@ rangeexpand_ (ImageBuf &R, const ImageBuf &A,
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Possible multiple thread case -- recurse via parallel_image
         ImageBufAlgo::parallel_image (
-            boost::bind(rangeexpand_<Rtype,Atype>, boost::ref(R), 
-                        boost::cref(A), useluma,
+            OIIO::bind(rangeexpand_<Rtype,Atype>, OIIO::ref(R), 
+                        OIIO::cref(A), useluma,
                         _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
         return true;
@@ -698,14 +1101,10 @@ bool
 ImageBufAlgo::rangecompress (ImageBuf &dst, const ImageBuf &src,
                              bool useluma, ROI roi, int nthreads)
 {
-    if (! dst.initialized()) {
-        dst.error ("in-place rangecompress requires the ImageBuf to be initialized");
-        return false;
-    }
-    if (! IBAprep (roi, &dst, &src))
+    if (! IBAprep (roi, &dst, &src, IBAprep_CLAMP_MUTUAL_NCHANNELS))
         return false;
     bool ok;
-    OIIO_DISPATCH_TYPES2 (ok, "rangecompress", rangecompress_,
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "rangecompress", rangecompress_,
                           dst.spec().format, src.spec().format,
                           dst, src, useluma, roi, nthreads);
     return ok;
@@ -717,14 +1116,10 @@ bool
 ImageBufAlgo::rangeexpand (ImageBuf &dst, const ImageBuf &src,
                            bool useluma, ROI roi, int nthreads)
 {
-    if (! dst.initialized()) {
-        dst.error ("in-place rangeexpand requires the ImageBuf to be initialized");
-        return false;
-    }
-    if (! IBAprep (roi, &dst))
+    if (! IBAprep (roi, &dst, &src, IBAprep_CLAMP_MUTUAL_NCHANNELS))
         return false;
     bool ok;
-    OIIO_DISPATCH_TYPES2 (ok, "rangeexpand", rangeexpand_,
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "rangeexpand", rangeexpand_,
                           dst.spec().format, src.spec().format,
                           dst, src, useluma, roi, nthreads);
     return ok;
@@ -739,7 +1134,7 @@ unpremult_ (ImageBuf &R, const ImageBuf &A, ROI roi, int nthreads)
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Possible multiple thread case -- recurse via parallel_image
         ImageBufAlgo::parallel_image (
-            boost::bind(unpremult_<Rtype,Atype>, boost::ref(R), boost::cref(A),
+            OIIO::bind(unpremult_<Rtype,Atype>, OIIO::ref(R), OIIO::cref(A),
                         _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
         return true;
@@ -782,7 +1177,7 @@ bool
 ImageBufAlgo::unpremult (ImageBuf &dst, const ImageBuf &src,
                          ROI roi, int nthreads)
 {
-    if (! IBAprep (roi, &dst, &src))
+    if (! IBAprep (roi, &dst, &src, IBAprep_CLAMP_MUTUAL_NCHANNELS))
         return false;
     if (src.spec().alpha_channel < 0) {
         if (&dst != &src)
@@ -791,7 +1186,7 @@ ImageBufAlgo::unpremult (ImageBuf &dst, const ImageBuf &src,
         return true;
     }
     bool ok;
-    OIIO_DISPATCH_TYPES2 (ok, "unpremult", unpremult_, dst.spec().format,
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "unpremult", unpremult_, dst.spec().format,
                           src.spec().format, dst, src, roi, nthreads);
     return ok;
 }
@@ -805,7 +1200,7 @@ premult_ (ImageBuf &R, const ImageBuf &A, ROI roi, int nthreads)
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Possible multiple thread case -- recurse via parallel_image
         ImageBufAlgo::parallel_image (
-            boost::bind(premult_<Rtype,Atype>, boost::ref(R), boost::cref(A),
+            OIIO::bind(premult_<Rtype,Atype>, OIIO::ref(R), OIIO::cref(A),
                         _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
         return true;
@@ -843,7 +1238,7 @@ bool
 ImageBufAlgo::premult (ImageBuf &dst, const ImageBuf &src,
                        ROI roi, int nthreads)
 {
-    if (! IBAprep (roi, &dst, &src))
+    if (! IBAprep (roi, &dst, &src, IBAprep_CLAMP_MUTUAL_NCHANNELS))
         return false;
     if (src.spec().alpha_channel < 0) {
         if (&dst != &src)
@@ -852,7 +1247,7 @@ ImageBufAlgo::premult (ImageBuf &dst, const ImageBuf &src,
         return true;
     }
     bool ok;
-    OIIO_DISPATCH_TYPES2 (ok, "premult", premult_, dst.spec().format,
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "premult", premult_, dst.spec().format,
                           src.spec().format, dst, src, roi, nthreads);
     return ok;
 }
@@ -874,7 +1269,7 @@ bool fixNonFinite_ (ImageBuf &dst, ImageBufAlgo::NonFiniteFixMode mode,
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Lots of pixels and request for multi threads? Parallelize.
         ImageBufAlgo::parallel_image (
-            boost::bind(fixNonFinite_<T>, boost::ref(dst), mode, pixelsFixed,
+            OIIO::bind(fixNonFinite_<T>, OIIO::ref(dst), mode, pixelsFixed,
                         _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
         return true;
@@ -1037,8 +1432,8 @@ over_impl (ImageBuf &R, const ImageBuf &A, const ImageBuf &B,
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Possible multiple thread case -- recurse via parallel_image
         ImageBufAlgo::parallel_image (
-            boost::bind(over_impl<Rtype,Atype,Btype>,
-                        boost::ref(R), boost::cref(A), boost::cref(B),
+            OIIO::bind(over_impl<Rtype,Atype,Btype>,
+                        OIIO::ref(R), OIIO::cref(A), OIIO::cref(B),
                         zcomp, z_zeroisinf, _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
         return true;
@@ -1123,5 +1518,4 @@ ImageBufAlgo::zover (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
 
 
 
-}
-OIIO_NAMESPACE_EXIT
+OIIO_NAMESPACE_END
