@@ -43,12 +43,24 @@
 #include <boost/scoped_array.hpp>
 
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
     using namespace pvt;
 
 
 
+ImageInput::ImageInput ()
+    : m_threads(0)
+{
+}
+
+
+
+ImageInput::~ImageInput ()
+{
+}
+
+
+
 // Default implementation of valid_file: try to do a full open.  If it
 // succeeds, it's the right kind of file.  We assume that most plugins
 // will override this with something smarter and much less expensive,
@@ -221,7 +233,8 @@ ImageInput::read_scanlines (int ybegin, int yend, int z,
             } else {
                 ok = parallel_convert_image (nchans, m_spec.width, nscanlines, 1, 
                                     &buf[0], m_spec.format, AutoStride, AutoStride, AutoStride,
-                                    data, format, xstride, ystride, zstride);
+                                    data, format, xstride, ystride, zstride,
+                                    -1 /*alpha*/, -1 /*z*/, threads());
             }
         } else {
             // Per-channel formats -- have to convert/copy channels individually
@@ -463,10 +476,11 @@ ImageInput::read_tiles (int xbegin, int xend, int ybegin, int yend,
                     // per-tile data format conversion.
                     ok &= read_tile (x, y, z, format, tilestart,
                                      xstride, ystride, zstride);
+                    if (! ok)
+                        return false;
                 } else {
                     buf.resize (full_tilebytes);
-                    ok &= read_tile (x, y, z, 
-                                     perchanfile ? TypeDesc::UNKNOWN : format,
+                    ok &= read_tile (x, y, z, format,
                                      &buf[0], full_pixelsize,
                                      full_tilewidthbytes, full_tilewhbytes);
                     if (ok)
@@ -594,12 +608,28 @@ ImageInput::read_image (TypeDesc format, void *data,
                         ProgressCallback progress_callback,
                         void *progress_callback_data)
 {
+    return read_image (0, -1, format, data, xstride, ystride, zstride,
+                       progress_callback, progress_callback_data);
+}
+
+
+
+bool
+ImageInput::read_image (int chbegin, int chend, TypeDesc format, void *data,
+                        stride_t xstride, stride_t ystride, stride_t zstride,
+                        ProgressCallback progress_callback,
+                        void *progress_callback_data)
+{
+    if (chend < 0)
+        chend = m_spec.nchannels;
+    chend = clamp (chend, chbegin+1, m_spec.nchannels);
+    int nchans = chend - chbegin;
     bool native = (format == TypeDesc::UNKNOWN);
-    stride_t pixel_bytes = native ? (stride_t) m_spec.pixel_bytes (native)
-                                  : (stride_t) (format.size()*m_spec.nchannels);
+    stride_t pixel_bytes = native ? (stride_t) m_spec.pixel_bytes (chbegin, chend, native)
+                                  : (stride_t) (format.size()*nchans);
     if (native && xstride == AutoStride)
         xstride = pixel_bytes;
-    m_spec.auto_stride (xstride, ystride, zstride, format, m_spec.nchannels,
+    m_spec.auto_stride (xstride, ystride, zstride, format, nchans,
                         m_spec.width, m_spec.height);
     bool ok = true;
     if (progress_callback)
@@ -612,6 +642,7 @@ ImageInput::read_image (TypeDesc format, void *data,
                 ok &= read_tiles (m_spec.x, m_spec.x+m_spec.width,
                                   y+m_spec.y, std::min (y+m_spec.y+m_spec.tile_height, m_spec.y+m_spec.height),
                                   z+m_spec.z, std::min (z+m_spec.z+m_spec.tile_depth, m_spec.z+m_spec.depth),
+                                  chbegin, chend,
                                   format, (char *)data + z*zstride + y*ystride,
                                   xstride, ystride, zstride);
                 if (progress_callback &&
@@ -620,12 +651,16 @@ ImageInput::read_image (TypeDesc format, void *data,
             }
         }
     } else {
-        // Scanline image -- rely on read_scanlines, in chunks of 64
-        const int chunk = 256;
+        // Scanline image -- rely on read_scanlines, in chunks of oiio_read_chunk
+        int read_chunk = oiio_read_chunk;
+        if (!read_chunk) {
+            read_chunk = m_spec.height;
+        }
         for (int z = 0;  z < m_spec.depth;  ++z)
-            for (int y = 0;  y < m_spec.height && ok;  y += chunk) {
-                int yend = std::min (y+m_spec.y+chunk, m_spec.y+m_spec.height);
-                ok &= read_scanlines (y+m_spec.y, yend, z+m_spec.z, format,
+            for (int y = 0;  y < m_spec.height && ok;  y += read_chunk) {
+                int yend = std::min (y+m_spec.y+read_chunk, m_spec.y+m_spec.height);
+                ok &= read_scanlines (y+m_spec.y, yend, z+m_spec.z,
+                                      chbegin, chend, format,
                                       (char *)data + z*zstride + y*ystride,
                                       xstride, ystride);
                 if (progress_callback)
@@ -721,5 +756,4 @@ ImageInput::read_native_tile (int x, int y, int z, void * data)
     return false;
 }
 
-}
-OIIO_NAMESPACE_EXIT
+OIIO_NAMESPACE_END
