@@ -35,38 +35,60 @@
 #include <OpenEXR/ImathFun.h>
 
 #include <boost/scoped_array.hpp>
+#include <boost/thread.hpp>
+#include <boost/thread/tss.hpp>
 
 #include "OpenImageIO/dassert.h"
 #include "OpenImageIO/typedesc.h"
 #include "OpenImageIO/strutil.h"
+#include "OpenImageIO/sysutil.h"
 #include "OpenImageIO/fmath.h"
 #include "OpenImageIO/thread.h"
 #include "OpenImageIO/hash.h"
 #include "OpenImageIO/imageio.h"
 #include "imageio_pvt.h"
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
 
 // Global private data
 namespace pvt {
 recursive_mutex imageio_mutex;
-atomic_int oiio_threads (boost::thread::hardware_concurrency());
-ustring plugin_searchpath;
+atomic_int oiio_threads (Sysutil::physical_concurrency());
+atomic_int oiio_exr_threads (Sysutil::physical_concurrency());
+atomic_int oiio_read_chunk (256);
+ustring plugin_searchpath (OIIO_DEFAULT_PLUGIN_SEARCHPATH);
 std::string format_list;   // comma-separated list of all formats
 std::string extension_list;   // list of all extensions for all formats
 }
 
-
-
 using namespace pvt;
 
+
 namespace {
+// Hidden global OIIO data.
+static spin_mutex attrib_mutex;
+static const int maxthreads = 256;   // reasonable maximum for sanity check
+const char *oiio_debug_env = getenv("OPENIMAGEIO_DEBUG");
+#ifdef NDEBUG
+int print_debug (oiio_debug_env ? atoi(oiio_debug_env) : 0);
+#else
+int print_debug (oiio_debug_env ? atoi(oiio_debug_env) : 1);
+#endif
+};
+
+
+
+int
+openimageio_version ()
+{
+    return OIIO_VERSION;
+}
+
 
 
 // To avoid thread oddities, we have the storage area buffering error
 // messages for seterror()/geterror() be thread-specific.
-static thread_specific_ptr<std::string> thread_error_msg;
+static boost::thread_specific_ptr<std::string> thread_error_msg;
 
 // Return a reference to the string for this thread's error messages,
 // creating it if none exists for this thread thus far.
@@ -81,16 +103,6 @@ error_msg ()
     return *e;
 }
 
-} // end anon namespace
-
-
-
-
-int
-openimageio_version ()
-{
-    return OIIO_VERSION;
-}
 
 
 
@@ -99,7 +111,6 @@ openimageio_version ()
 void
 pvt::seterror (const std::string& message)
 {
-    recursive_lock_guard lock (pvt::imageio_mutex);
     error_msg() = message;
 }
 
@@ -108,7 +119,6 @@ pvt::seterror (const std::string& message)
 std::string
 geterror ()
 {
-    recursive_lock_guard lock (pvt::imageio_mutex);
     std::string e = error_msg();
     error_msg().clear ();
     return e;
@@ -116,14 +126,15 @@ geterror ()
 
 
 
-namespace {
-
-// Private global OIIO data.
-
-static spin_mutex attrib_mutex;
-static const int maxthreads = 64;   // reasonable maximum for sanity check
-
-};
+void
+pvt::debugmsg_ (string_view message)
+{
+    recursive_lock_guard lock (pvt::imageio_mutex);
+    if (print_debug) {
+        std::cerr << "OIIO DEBUG: " << message 
+                  << (message.back() == '\n' ? "" : "\n");
+    }
+}
 
 
 
@@ -133,15 +144,27 @@ attribute (string_view name, TypeDesc type, const void *val)
     if (name == "threads" && type == TypeDesc::TypeInt) {
         int ot = Imath::clamp (*(const int *)val, 0, maxthreads);
         if (ot == 0)
-            ot = boost::thread::hardware_concurrency();
+            ot = Sysutil::physical_concurrency();
         oiio_threads = ot;
         return true;
     }
     spin_lock lock (attrib_mutex);
+    if (name == "read_chunk" && type == TypeDesc::TypeInt) {
+        oiio_read_chunk = *(const int *)val;
+        return true;
+    }
     if (name == "plugin_searchpath" && type == TypeDesc::TypeString) {
         plugin_searchpath = ustring (*(const char **)val);
         return true;
     }
+    if (name == "exr_threads" && type == TypeDesc::TypeInt) {
+        oiio_exr_threads = Imath::clamp (*(const int *)val, 0, maxthreads);
+        return true;
+    }
+    if (name == "debug" && type == TypeDesc::TypeInt) {
+        print_debug = *(const int *)val;
+        return true;
+    }
     return false;
 }
 
@@ -155,6 +178,10 @@ getattribute (string_view name, TypeDesc type, void *val)
         return true;
     }
     spin_lock lock (attrib_mutex);
+    if (name == "read_chunk" && type == TypeDesc::TypeInt) {
+        *(int *)val = oiio_read_chunk;
+        return true;
+    }
     if (name == "plugin_searchpath" && type == TypeDesc::TypeString) {
         *(ustring *)val = plugin_searchpath;
         return true;
@@ -171,15 +198,23 @@ getattribute (string_view name, TypeDesc type, void *val)
         *(ustring *)val = ustring(extension_list);
         return true;
     }
+    if (name == "exr_threads" && type == TypeDesc::TypeInt) {
+        *(int *)val = oiio_exr_threads;
+        return true;
+    }
+    if (name == "debug" && type == TypeDesc::TypeInt) {
+        *(int *)val = print_debug;
+        return true;
+    }
     return false;
 }
 
 
-inline int
-quantize (float value, int quant_min, int quant_max)
+inline long long
+quantize (float value, long long quant_min, long long quant_max)
 {
     value = value * quant_max;
-    return Imath::clamp ((int)(value + 0.5f), quant_min, quant_max);
+    return Imath::clamp ((long long)(value + 0.5f), quant_min, quant_max);
 }
 
 namespace {
@@ -322,7 +357,7 @@ pvt::convert_to_float (const void *src, float *dst, int nvals,
 template<typename T>
 const void *
 _from_float (const float *src, T *dst, size_t nvals,
-             int quant_min, int quant_max)
+             long long quant_min, long long quant_max)
 {
     if (! src) {
         // If no source pixels, assume zeroes
@@ -352,7 +387,7 @@ _from_float (const float *src, T *dst, size_t nvals,
 
 const void *
 pvt::convert_from_float (const float *src, void *dst, size_t nvals,
-                         int quant_min, int quant_max, TypeDesc format)
+                         long long quant_min, long long quant_max, TypeDesc format)
 {
     switch (format.basetype) {
     case TypeDesc::FLOAT :
@@ -397,8 +432,8 @@ pvt::convert_from_float (const float *src, void *dst, size_t nvals,
 // DEPRECATED (1.4)
 const void *
 pvt::convert_from_float (const float *src, void *dst, size_t nvals,
-                         int quant_black, int quant_white,
-                         int quant_min, int quant_max,
+                         long long quant_black, long long quant_white,
+                         long long quant_min, long long quant_max,
                          TypeDesc format)
 {
     return convert_from_float (src, dst, nvals, quant_min, quant_max, format);
@@ -420,7 +455,7 @@ pvt::parallel_convert_from_float (const float *src, void *dst, size_t nvals,
     if (nthreads <= 0)
         nthreads = oiio_threads;
 
-    int quant_min, quant_max;
+    long long quant_min, quant_max;
     get_default_quantize (format, quant_min, quant_max);
 
     if (nthreads <= 1)
@@ -448,8 +483,8 @@ pvt::parallel_convert_from_float (const float *src, void *dst, size_t nvals,
 const void *
 pvt::parallel_convert_from_float (const float *src, void *dst,
                                   size_t nvals,
-                                  int quant_black, int quant_white,
-                                  int quant_min, int quant_max,
+                                  long long quant_black, long long quant_white,
+                                  long long quant_min, long long quant_max,
                                   TypeDesc format, int nthreads)
 {
     return parallel_convert_from_float (src, dst, nvals, format, nthreads);
@@ -542,8 +577,8 @@ convert_image (int nchannels, int width, int height, int depth,
     ImageSpec::auto_stride (dst_xstride, dst_ystride, dst_zstride,
                             dst_type, nchannels, width, height);
     bool result = true;
-    bool contig = (src_xstride == dst_xstride &&
-                   src_xstride == stride_t(nchannels*src_type.size()));
+    bool contig = (src_xstride == stride_t(nchannels * src_type.size()) &&
+                   dst_xstride == stride_t(nchannels * dst_type.size()));
     for (int z = 0;  z < depth;  ++z) {
         for (int y = 0;  y < height;  ++y) {
             const char *f = (const char *)src + (z*src_zstride + y*src_ystride);
@@ -732,41 +767,26 @@ add_dither (int nchannels, int width, int height, int depth,
 
 
 
-void
-DeepData::init (int npix, int nchan,
-                const TypeDesc *chbegin, const TypeDesc *chend)
+template<typename T>
+static void
+premult_impl (int nchannels, int width, int height, int depth,
+              int chbegin, int chend,
+              T *data, stride_t xstride, stride_t ystride, stride_t zstride,
+              int alpha_channel, int z_channel)
 {
-    clear ();
-    npixels = npix;
-    nchannels = nchan;
-    channeltypes.assign (chbegin, chend);
-    nsamples.resize (npixels, 0);
-    pointers.resize (size_t(npixels)*size_t(nchannels), NULL);
-}
-
-
-
-void
-DeepData::alloc ()
-{
-    // Calculate the total size we need, align each channel to 4 byte boundary
-    size_t totalsamples = 0, totalbytes = 0;
-    for (int i = 0;  i < npixels;  ++i) {
-        int s = nsamples[i];
-        totalsamples += s;
-        for (int c = 0;  c < nchannels;  ++c)
-            totalbytes += round_to_multiple (channeltypes[c].size() * s, 4);
-    }
-
-    // Set all the data pointers to the right offsets within the
-    // data block.  Leave the pointes NULL for pixels with no samples.
-    data.resize (totalbytes);
-    char *p = &data[0];
-    for (int i = 0;  i < npixels;  ++i) {
-        if (nsamples[i]) {
-            for (int c = 0;  c < nchannels;  ++c) {
-                pointers[i*nchannels+c] = p;
-                p += round_to_multiple (channeltypes[c].size()*nsamples[i], 4);
+    char *plane = (char *)data;
+    for (int z = 0;  z < depth;  ++z, plane += zstride) {
+        char *scanline = plane;
+        for (int y = 0;  y < height;  ++y, scanline += ystride) {
+            char *pixel = scanline;
+            for (int x = 0;  x < width;  ++x, pixel += xstride) {
+                DataArrayProxy<T,float> val ((T*)pixel);
+                float alpha = val[alpha_channel];
+                for (int c = chbegin;  c < chend;  ++c) {
+                    if (c == alpha_channel || c == z_channel)
+                        continue;
+                    val[c] = alpha * val[c];
+                }
             }
         }
     }
@@ -775,73 +795,73 @@ DeepData::alloc ()
 
 
 void
-DeepData::clear ()
+premult (int nchannels, int width, int height, int depth,
+         int chbegin, int chend,
+         TypeDesc datatype, void *data,
+         stride_t xstride, stride_t ystride, stride_t zstride,
+         int alpha_channel, int z_channel)
 {
-    npixels = 0;
-    nchannels = 0;
-    channeltypes.clear();
-    nsamples.clear();
-    pointers.clear();
-    data.clear();
-}
-
-
-
-void
-DeepData::free ()
-{
-    std::vector<unsigned int>().swap (nsamples);
-    std::vector<void *>().swap (pointers);
-    std::vector<char>().swap (data);
-}
-
-
-
-void *
-DeepData::channel_ptr (int pixel, int channel) const
-{
-    if (pixel < 0 || pixel >= npixels || channel < 0 || channel >= nchannels)
-        return NULL;
-    return pointers[pixel*nchannels + channel];
-}
-
-
-
-float
-DeepData::deep_value (int pixel, int channel, int sample) const
-{
-    if (pixel < 0 || pixel >= npixels || channel < 0 || channel >= nchannels)
-        return 0.0f;
-    int nsamps = nsamples[pixel];
-    if (nsamps == 0 || sample < 0 || sample >= nsamps)
-        return 0.0f;
-    const void *ptr = pointers[pixel*nchannels + channel];
-    if (! ptr)
-        return 0.0f;
-    switch (channeltypes[channel].basetype) {
+    if (alpha_channel < 0 || alpha_channel > nchannels)
+        return;  // nothing to do
+    ImageSpec::auto_stride (xstride, ystride, zstride,
+                            datatype.size(), nchannels, width, height);
+    switch (datatype.basetype) {
     case TypeDesc::FLOAT :
-        return ((const float *)ptr)[sample];
-    case TypeDesc::HALF  :
-        return ((const half *)ptr)[sample];
+        premult_impl (nchannels, width, height, depth, chbegin, chend,
+                      (float*)data, xstride, ystride, zstride,
+                      alpha_channel, z_channel);
+        break;
     case TypeDesc::UINT8 :
-        return ConstDataArrayProxy<unsigned char,float>((const unsigned char *)ptr)[sample];
-    case TypeDesc::INT8  :
-        return ConstDataArrayProxy<char,float>((const char *)ptr)[sample];
-    case TypeDesc::UINT16:
-        return ConstDataArrayProxy<unsigned short,float>((const unsigned short *)ptr)[sample];
+        premult_impl (nchannels, width, height, depth, chbegin, chend,
+                      (unsigned char*)data, xstride, ystride, zstride,
+                      alpha_channel, z_channel);
+        break;
+    case TypeDesc::UINT16 :
+        premult_impl (nchannels, width, height, depth, chbegin, chend,
+                      (unsigned short*)data, xstride, ystride, zstride,
+                      alpha_channel, z_channel);
+        break;
+    case TypeDesc::HALF :
+        premult_impl (nchannels, width, height, depth, chbegin, chend,
+                      (half*)data, xstride, ystride, zstride,
+                      alpha_channel, z_channel);
+        break;
+    case TypeDesc::INT8 :
+        premult_impl (nchannels, width, height, depth, chbegin, chend,
+                      (char*)data, xstride, ystride, zstride,
+                      alpha_channel, z_channel);
+        break;
     case TypeDesc::INT16 :
-        return ConstDataArrayProxy<short,float>((const short *)ptr)[sample];
-    case TypeDesc::UINT  :
-        return ConstDataArrayProxy<unsigned int,float>((const unsigned int *)ptr)[sample];
-    case TypeDesc::INT   :
-        return ConstDataArrayProxy<int,float>((const int *)ptr)[sample];
-    case TypeDesc::UINT64:
-        return ConstDataArrayProxy<unsigned long long,float>((const unsigned long long *)ptr)[sample];
+        premult_impl (nchannels, width, height, depth, chbegin, chend,
+                      (short*)data, xstride, ystride, zstride,
+                      alpha_channel, z_channel);
+        break;
+    case TypeDesc::INT :
+        premult_impl (nchannels, width, height, depth, chbegin, chend,
+                      (int*)data, xstride, ystride, zstride,
+                      alpha_channel, z_channel);
+        break;
+    case TypeDesc::UINT :
+        premult_impl (nchannels, width, height, depth, chbegin, chend,
+                      (unsigned int*)data, xstride, ystride, zstride,
+                      alpha_channel, z_channel);
+        break;
     case TypeDesc::INT64 :
-        return ConstDataArrayProxy<long long,float>((const long long *)ptr)[sample];
-    default:
-        ASSERT (0);
-        return 0.0f;
+        premult_impl (nchannels, width, height, depth, chbegin, chend,
+                      (int64_t*)data, xstride, ystride, zstride,
+                      alpha_channel, z_channel);
+        break;
+    case TypeDesc::UINT64 :
+        premult_impl (nchannels, width, height, depth, chbegin, chend,
+                      (uint64_t*)data, xstride, ystride, zstride,
+                      alpha_channel, z_channel);
+        break;
+    case TypeDesc::DOUBLE :
+        premult_impl (nchannels, width, height, depth, chbegin, chend,
+                      (double*)data, xstride, ystride, zstride,
+                      alpha_channel, z_channel);
+        break;
+    default: break;
     }
 }
 
@@ -905,5 +925,4 @@ wrap_mirror (int &coord, int origin, int width)
 }
 
 
-}
-OIIO_NAMESPACE_EXIT
+OIIO_NAMESPACE_END
