@@ -34,7 +34,6 @@
 #include <string>
 #include <vector>
 
-#include <boost/filesystem.hpp>
 #include <boost/foreach.hpp>
 
 #include "OpenImageIO/dassert.h"
@@ -45,8 +44,7 @@
 #include "imageio_pvt.h"
 
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
     using namespace pvt;
 
 typedef std::map <std::string, ImageInput::Creator> InputPluginMap;
@@ -150,14 +148,10 @@ catalog_plugin (const std::string &format_name,
             // It's ok if they're both the same file; just skip it.
             return;
         }
-        // if (verbosity > 1)
-#ifndef NDEBUG
-        std::cerr << "OpenImageIO WARNING: " << format_name << " had multiple plugins:\n"
-                  << "\t\"" << found_path->second << "\"\n"
-                  << "    as well as\n"
-                  << "\t\"" << plugin_fullpath << "\"\n"
-                  << "    Ignoring all but the first one.\n";
-#endif
+        OIIO::debugmsg ("OpenImageIO WARNING: %s had multiple plugins:\n"
+                        "\t\"%s\"\n    as well as\n\t\"%s\"\n"
+                        "    Ignoring all but the first one.\n",
+                        format_name, found_path->second, plugin_fullpath);
         return;
     }
 
@@ -211,6 +205,7 @@ catalog_plugin (const std::string &format_name,
     PLUGENTRY (cineon);
     PLUGENTRY (dds);
     PLUGENTRY (dpx);
+    PLUGENTRY (ffmpeg);
     PLUGENTRY (field3d);
     PLUGENTRY (fits);
     PLUGENTRY (gif);
@@ -260,6 +255,9 @@ catalog_builtin_plugins ()
     DECLAREPLUG (cineon);
     DECLAREPLUG (dds);
     DECLAREPLUG (dpx);
+#ifdef USE_FFMPEG
+    DECLAREPLUG (ffmpeg);
+#endif
 #ifdef USE_FIELD3D
     DECLAREPLUG (field3d);
 #endif
@@ -278,7 +276,9 @@ catalog_builtin_plugins ()
     DECLAREPLUG (png);
     DECLAREPLUG (pnm);
     DECLAREPLUG (psd);
+#ifdef USE_PTEX
     DECLAREPLUG (ptex);
+#endif
 #ifdef USE_LIBRAW
     DECLAREPLUG (raw);
 #endif
@@ -301,6 +301,25 @@ catalog_builtin_plugins ()
 
 
 
+static void
+append_if_env_exists (std::string &searchpath, const char *env,
+                      bool prepend=false)
+{
+    const char *path = getenv (env);
+    if (path && *path) {
+        std::string newpath = path;
+        if (searchpath.length()) {
+            if (prepend)
+                newpath = newpath + ':' + searchpath;
+            else
+                newpath = searchpath + ':' + newpath;
+        }
+        searchpath = newpath;
+    }
+}
+
+
+
 /// Look at ALL imageio plugins in the searchpath and add them to the
 /// catalog.  This routine is not reentrant and should only be called
 /// by a routine that is holding a lock on imageio_mutex.
@@ -309,22 +328,21 @@ pvt::catalog_all_plugins (std::string searchpath)
 {
     catalog_builtin_plugins ();
 
-    const char *oiio_library_path = getenv ("OIIO_LIBRARY_PATH");
-    if (oiio_library_path && *oiio_library_path) {
-        std::string newpath = oiio_library_path;
-        if (searchpath.length())
-            newpath = newpath + ':' + searchpath;
-        searchpath = newpath;
-    }
+    append_if_env_exists (searchpath, "OIIO_LIBRARY_PATH", true);
+#ifdef __APPLE__
+    append_if_env_exists (searchpath, "DYLD_LIBRARY_PATH");
+#endif
+#if defined(__linux__) || defined(__FreeBSD__)
+    append_if_env_exists (searchpath, "LD_LIBRARY_PATH");
+#endif
 
     size_t patlen = pattern.length();
     std::vector<std::string> dirs;
     Filesystem::searchpath_split (searchpath, dirs, true);
-    BOOST_FOREACH (std::string &dir, dirs) {
-        boost::filesystem::directory_iterator end_itr; // default construction yields past-the-end
-        for (boost::filesystem::directory_iterator itr (dir);
-              itr != end_itr;  ++itr) {
-            std::string full_filename = itr->path().string();
+    BOOST_FOREACH (const std::string &dir, dirs) {
+        std::vector<std::string> dir_entries;
+        Filesystem::get_directory_entries (dir, dir_entries);
+        BOOST_FOREACH (const std::string &full_filename, dir_entries) {
             std::string leaf = Filesystem::filename (full_filename);
             size_t found = leaf.find (pattern);
             if (found != std::string::npos &&
@@ -391,6 +409,14 @@ ImageOutput::create (const std::string &filename,
 
 
 
+void
+ImageOutput::destroy (ImageOutput *x)
+{
+    delete x;
+}
+
+
+
 ImageInput *
 ImageInput::create (const std::string &filename, 
                     const std::string &plugin_searchpath)
@@ -541,5 +567,13 @@ ImageInput::create (const std::string &filename,
     return (ImageInput *) create_function();
 }
 
+
+
+void
+ImageInput::destroy (ImageInput *x)
+{
+    delete x;
 }
-OIIO_NAMESPACE_EXIT
+
+
+OIIO_NAMESPACE_END
