@@ -47,6 +47,7 @@
 #include "OpenImageIO/imageio.h"
 #include "OpenImageIO/texture.h"
 #include "OpenImageIO/imagecache.h"
+#include "OpenImageIO/imagecache.h"
 #include "imagecache_pvt.h"
 #include "texture_pvt.h"
 
@@ -225,9 +226,9 @@ convention is dictated by OpenEXR.
 */
 
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
     using namespace pvt;
+    using namespace simd;
 
 namespace {  // anonymous
 
@@ -246,19 +247,48 @@ TextureSystemImpl::environment (ustring filename, TextureOptions &options,
                                 VaryingRef<Imath::V3f> R,
                                 VaryingRef<Imath::V3f> dRdx,
                                 VaryingRef<Imath::V3f> dRdy,
-                                float *result)
+                                int nchannels, float *result,
+                                float *dresultds, float *dresultdt)
 {
     Perthread *thread_info = get_perthread_info();
     TextureHandle *texture_handle = get_texture_handle (filename, thread_info);
+    return environment (texture_handle, thread_info, options,
+                        runflags, beginactive, endactive,
+                        R, dRdx, dRdy, nchannels, result, dresultds, dresultdt);
+}
+
+
+
+bool
+TextureSystemImpl::environment (TextureHandle *texture_handle,
+                                Perthread *thread_info, TextureOptions &options,
+                                Runflag *runflags,
+                                int beginactive, int endactive,
+                                VaryingRef<Imath::V3f> R,
+                                VaryingRef<Imath::V3f> dRdx,
+                                VaryingRef<Imath::V3f> dRdy,
+                                int nchannels, float *result,
+                                float *dresultds, float *dresultdt)
+{
     if (! texture_handle)
         return false;
     bool ok = true;
+    result += beginactive*nchannels;
+    if (dresultds) {
+        dresultds += beginactive*nchannels;
+        dresultdt += beginactive*nchannels;
+    }
     for (int i = beginactive;  i < endactive;  ++i) {
         if (runflags[i]) {
             TextureOpt opt (options, i);
             ok &= environment (texture_handle, thread_info,
                                opt, R[i], dRdx[i], dRdy[i],
-                               result+i*options.nchannels);
+                               nchannels, result, dresultds, dresultdt);
+        }
+        result += nchannels;
+        if (dresultds) {
+            dresultds += nchannels;
+            dresultdt += nchannels;
         }
     }
     return ok;
@@ -291,12 +321,14 @@ bool
 TextureSystemImpl::environment (ustring filename, TextureOpt &options,
                                 const Imath::V3f &R,
                                 const Imath::V3f &dRdx, const Imath::V3f &dRdy,
-                                float *result)
+                                int nchannels, float *result,
+                                float *dresultds, float *dresultdt)
 {
     PerThreadInfo *thread_info = m_imagecache->get_perthread_info ();
     TextureFile *texturefile = find_texturefile (filename, thread_info);
     return environment ((TextureHandle *)texturefile, (Perthread *)thread_info,
-                        options, R, dRdx, dRdy, result);
+                        options, R, dRdx, dRdy,
+                        nchannels, result, dresultds, dresultdt);
 }
 
 
@@ -306,38 +338,59 @@ TextureSystemImpl::environment (TextureHandle *texture_handle_,
                                 Perthread *thread_info_,
                                 TextureOpt &options, const Imath::V3f &_R,
                                 const Imath::V3f &_dRdx, const Imath::V3f &_dRdy,
-                                float *result)
+                                int nchannels, float *result,
+                                float *dresultds, float *dresultdt)
 {
-    PerThreadInfo *thread_info = (PerThreadInfo *)thread_info_;
-    TextureFile *texturefile = (TextureFile *)texture_handle_;
+    // Handle >4 channel lookups by recursion.
+    if (nchannels > 4) {
+        int save_firstchannel = options.firstchannel;
+        while (nchannels) {
+            int n = std::min (nchannels, 4);
+            bool ok = environment (texture_handle_, thread_info_, options,
+                                   _R, _dRdx, _dRdy,
+                                   n, result, dresultds, dresultdt);
+            if (! ok)
+                return false;
+            result += n;
+            if (dresultds) dresultds += n;
+            if (dresultdt) dresultdt += n;
+            options.firstchannel += n;
+            nchannels -= n;
+        }
+        options.firstchannel = save_firstchannel; // restore what we changed
+        return true;
+    }
+
+    PerThreadInfo *thread_info = m_imagecache->get_perthread_info((PerThreadInfo *)thread_info_);
+    TextureFile *texturefile = verify_texturefile ((TextureFile *)texture_handle_, thread_info);
     ImageCacheStatistics &stats (thread_info->m_stats);
     ++stats.environment_batches;
     ++stats.environment_queries;
 
     if (! texturefile  ||  texturefile->broken())
-        return missing_texture (options, result);
+        return missing_texture (options, nchannels, result,
+                                dresultds, dresultdt);
 
     const ImageSpec &spec (texturefile->spec(options.subimage, 0));
 
-    options.swrap_func = texturefile->m_sample_border ?
-        wrap_periodic_sharedborder : wrap_periodic;
-    options.twrap_func = wrap_clamp;
+    // Environment maps dictate particular wrap modes
+    options.swrap = texturefile->m_sample_border ?
+        TextureOpt::WrapPeriodicSharedBorder : TextureOpt::WrapPeriodic;
+    options.twrap = TextureOpt::WrapClamp;
+
     options.envlayout = LayoutLatLong;
     int actualchannels = Imath::clamp (spec.nchannels - options.firstchannel,
-                                       0, options.nchannels);
-    options.actualchannels = actualchannels;
+                                       0, nchannels);
 
     // Initialize results to 0.  We'll add from here on as we sample.
-    for (int c = 0;  c < options.nchannels;  ++c)
+    for (int c = 0;  c < nchannels;  ++c)
         result[c] = 0;
-    float* dresultds = options.dresultds;
-    float* dresultdt = options.dresultdt;
-    if (dresultds)
-        for (int c = 0;  c < options.nchannels;  ++c)
+    if (dresultds) {
+        for (int c = 0;  c < nchannels;  ++c)
             dresultds[c] = 0;
-    if (dresultdt)
-        for (int c = 0;  c < options.nchannels;  ++c)
+        for (int c = 0;  c < nchannels;  ++c)
             dresultdt[c] = 0;
+    }
     // If the user only provided us with one pointer, clear both to simplify
     // the rest of the code, but only after we zero out the data for them so
     // they know something went wrong.
@@ -375,23 +428,23 @@ TextureSystemImpl::environment (TextureHandle *texture_handle_,
         minorlength = xfilt;
     }
 
-    accum_prototype accumer;
+    sampler_prototype sampler;
     long long *probecount;
     switch (options.interpmode) {
     case TextureOpt::InterpClosest :
-        accumer = &TextureSystemImpl::accum_sample_closest;
+        sampler = &TextureSystemImpl::sample_closest;
         probecount = &stats.closest_interps;
         break;
     case TextureOpt::InterpBilinear :
-        accumer = &TextureSystemImpl::accum_sample_bilinear;
+        sampler = &TextureSystemImpl::sample_bilinear;
         probecount = &stats.bilinear_interps;
         break;
     case TextureOpt::InterpBicubic :
-        accumer = &TextureSystemImpl::accum_sample_bicubic;
+        sampler = &TextureSystemImpl::sample_bicubic;
         probecount = &stats.cubic_interps;
         break;
     default:
-        accumer = NULL;
+        sampler = NULL;
         probecount = NULL;
         break;
     }
@@ -445,7 +498,7 @@ TextureSystemImpl::environment (TextureHandle *texture_handle_,
             if (filtwidth_ras <= 1) {
                 miplevel[0] = m-1;
                 miplevel[1] = m;
-                levelblend = Imath::clamp (2.0f - 1.0f/filtwidth_ras, 0.0f, 1.0f);
+                levelblend = Imath::clamp (2.0f*filtwidth_ras - 1.0f, 0.0f, 1.0f);
                 break;
             }
         }
@@ -484,27 +537,40 @@ TextureSystemImpl::environment (TextureHandle *texture_handle_,
             if (options.interpmode == TextureOpt::InterpSmartBicubic) {
                 if (lev == 0 ||
                     (texturefile->spec(options.subimage,lev).full_height < naturalres/2)) {
-                    accumer = &TextureSystemImpl::accum_sample_bicubic;
+                    sampler = &TextureSystemImpl::sample_bicubic;
                     ++stats.cubic_interps;
                 } else {
-                    accumer = &TextureSystemImpl::accum_sample_bilinear;
+                    sampler = &TextureSystemImpl::sample_bilinear;
                     ++stats.bilinear_interps;
                 }
             } else {
                 *probecount += 1;
             }
 
-            ok &= (this->*accumer) (s, t, miplevel[level], *texturefile,
-                                    thread_info, options,
-                                    levelweight[level]*invsamples, result,
-                                    dresultds, dresultdt);
+            OIIO_SIMD4_ALIGN float sval[4] = { s, 0.0f, 0.0f, 0.0f };
+            OIIO_SIMD4_ALIGN float tval[4] = { t, 0.0f, 0.0f, 0.0f };
+            OIIO_SIMD4_ALIGN float weight[4] = { levelweight[level]*invsamples,
+                                                 0.0f, 0.0f, 0.0f };
+            float4 r, drds, drdt;
+            ok &= (this->*sampler) (1, sval, tval, miplevel[level],
+                                    *texturefile, thread_info, options,
+                                    nchannels, actualchannels, weight,
+                                    &r, dresultds ? &drds : NULL, dresultds ? &drdt : NULL);
+            for (int c = 0; c < nchannels; ++c)
+                result[c] += r[c];
+            if (dresultds) {
+                for (int c = 0; c < nchannels; ++c) {
+                    dresultds[c] += drds[c];
+                    dresultdt[c] += drdt[c];
+                }
+            }
         }
     }
     stats.aniso_probes += nsamples;
     ++stats.aniso_queries;
 
-    if (actualchannels < options.nchannels)
-        fill_gray_channels (spec, options, result);
+    if (actualchannels < nchannels && options.firstchannel == 0 && m_gray_to_rgb)
+        fill_gray_channels (spec, nchannels, result, dresultds, dresultdt);
 
     return ok;
 }
@@ -513,5 +579,4 @@ TextureSystemImpl::environment (TextureHandle *texture_handle_,
 
 }  // end namespace pvt
 
-}
-OIIO_NAMESPACE_EXIT
+OIIO_NAMESPACE_END
