@@ -36,8 +36,17 @@
 #ifndef OPENIMAGEIO_IMAGECACHE_PVT_H
 #define OPENIMAGEIO_IMAGECACHE_PVT_H
 
-#include <boost/unordered_map.hpp>
+#if OIIO_CPLUSPLUS_VERSION >= 11
+#  include <unordered_map>
+#else /* FIXME(C++11): remove this after making C++11 the baseline */
+#  include <boost/unordered_map.hpp>
+#endif
+
+#include <boost/scoped_ptr.hpp>
 #include <boost/scoped_array.hpp>
+#include <boost/thread/tss.hpp>
+
+#include <OpenEXR/half.h>
 
 #include "OpenImageIO/export.h"
 #include "OpenImageIO/texture.h"
@@ -47,8 +56,7 @@
 #include "OpenImageIO/unordered_map_concurrent.h"
 
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
 
 namespace pvt {
 
@@ -63,8 +71,10 @@ namespace pvt {
 #define IMAGECACHE_USE_RW_MUTEX 1
 
 
+using boost::thread_specific_ptr;
+
 class ImageCacheImpl;
-struct ImageCachePerThreadInfo;
+class ImageCachePerThreadInfo;
 
 const char * texture_format_name (TexFormat f);
 const char * texture_type_name (TexFormat f);
@@ -137,7 +147,8 @@ class OIIO_API ImageCacheFile : public RefCnt {
 public:
     ImageCacheFile (ImageCacheImpl &imagecache,
                     ImageCachePerThreadInfo *thread_info, ustring filename,
-                    ImageInput::Creator creator=NULL);
+                    ImageInput::Creator creator=NULL,
+                    const ImageSpec *config=NULL);
     ~ImageCacheFile ();
 
     bool broken () const { return m_broken; }
@@ -169,7 +180,7 @@ public:
     ///
     bool read_tile (ImageCachePerThreadInfo *thread_info,
                     int subimage, int miplevel, int x, int y, int z,
-                    TypeDesc format, void *data);
+                    int chbegin, int chend, TypeDesc format, void *data);
 
     /// Mark the file as recently used.
     ///
@@ -182,8 +193,11 @@ public:
 
     size_t channelsize (int subimage) const { return m_subimages[subimage].channelsize; }
     size_t pixelsize (int subimage) const { return m_subimages[subimage].pixelsize; }
-    bool eightbit (int subimage) const { return m_subimages[subimage].eightbit; }
+    TypeDesc::BASETYPE pixeltype (int subimage) const {
+        return (TypeDesc::BASETYPE) m_subimages[subimage].datatype.basetype;
+    }
     bool mipused (void) const { return m_mipused; }
+    bool sample_border (void) const { return m_sample_border; }
     const std::vector<size_t> &mipreadcount (void) const { return m_mipreadcount; }
 
     void invalidate ();
@@ -198,6 +212,10 @@ public:
     void duplicate (ImageCacheFile *dup) { m_duplicate = dup;}
     ImageCacheFile *duplicate () const { return m_duplicate; }
 
+    // Retrieve the average color, or try to compute it. Return true on
+    // success, false on failure.
+    bool get_average_color (float *avg, int subimage, int chbegin, int chend);
+
     /// Info for each MIP level that isn't in the ImageSpec, or that we
     /// precompute.
     struct LevelInfo {
@@ -221,7 +239,10 @@ public:
         bool unmipped;                  ///< Not really MIP-mapped
         bool volume;                    ///< It's a volume image
         bool full_pixel_range;          ///< pixel data window matches image window
-        bool eightbit;                  ///< Eight bit?  (or float)
+        bool is_constant_image;         ///< Is the image a constant color?
+        bool has_average_color;         ///< We have an average color
+        std::vector<float> average_color; ///< Average color
+        spin_mutex average_color_mutex; ///< protect average_color
 
         // The scale/offset accounts for crops or overscans, converting
         // 0-1 texture space relative to the "display/full window" into 
@@ -232,7 +253,8 @@ public:
         SubimageInfo () : datatype(TypeDesc::UNKNOWN),
                           channelsize(0), pixelsize(0),
                           untiled(false), unmipped(false), volume(false),
-                          full_pixel_range(false), eightbit(false),
+                          full_pixel_range(false),
+                          is_constant_image(false), has_average_color(false),
                           sscale(1.0f), soffset(0.0f),
                           tscale(1.0f), toffset(0.0f) { }
         void init (const ImageSpec &spec, bool forcefloat);
@@ -304,6 +326,7 @@ private:
     ImageCacheFile *m_duplicate;    ///< Is this a duplicate?
     imagesize_t m_total_imagesize;  ///< Total size, uncompressed
     ImageInput::Creator m_inputcreator; ///< Custom ImageInput-creator
+    boost::scoped_ptr<ImageSpec> m_configspec; // Optional configuration hints
 
     /// We will need to read pixels from the file, so be sure it's
     /// currently opened.  Return true if ok, false if error.
@@ -326,14 +349,14 @@ private:
     /// a seek_subimage to the right subimage and MIP level.
     bool read_untiled (ImageCachePerThreadInfo *thread_info,
                        int subimage, int miplevel, int x, int y, int z,
-                       TypeDesc format, void *data);
+                       int chbegin, int chend, TypeDesc format, void *data);
 
     /// Load the requested tile, from a file that's not really MIPmapped.
     /// Preconditions: the ImageInput is already opened, and we already did
     /// a seek_subimage to the right subimage.
     bool read_unmipped (ImageCachePerThreadInfo *thread_info,
                         int subimage, int miplevel, int x, int y, int z,
-                        TypeDesc format, void *data);
+                        int chbegin, int chend, TypeDesc format, void *data);
 
     void lock_input_mutex () {
         m_input_mutex.lock ();
@@ -361,9 +384,12 @@ typedef intrusive_ptr<ImageCacheFile> ImageCacheFileRef;
 
 
 /// Map file names to file references
-///
 typedef unordered_map_concurrent<ustring,ImageCacheFileRef,ustringHash,std::equal_to<ustring>, 8> FilenameMap;
+#if OIIO_CPLUSPLUS_VERSION >= 11
+typedef std::unordered_map<ustring,ImageCacheFileRef,ustringHash> FingerprintMap;
+#else /* FIXME(C++11): remove this after making C++11 the baseline */
 typedef boost::unordered_map<ustring,ImageCacheFileRef,ustringHash> FingerprintMap;
+#endif
 
 
 
@@ -379,10 +405,15 @@ public:
     /// Initialize a TileID based on full elaboration of image file,
     /// subimage, and tile x,y,z indices.
     TileID (ImageCacheFile &file, int subimage, int miplevel,
-            int x, int y, int z=0)
+            int x, int y, int z=0, int chbegin=0, int chend=-1)
         : m_x(x), m_y(y), m_z(z), m_subimage(subimage),
-          m_miplevel(miplevel), m_file(&file)
-    { }
+          m_miplevel(miplevel), m_chbegin(chbegin), m_chend(chend),
+          m_file(&file)
+    {
+        int nc = file.spec(subimage,miplevel).nchannels;
+        if (chend < chbegin || chend > nc)
+            m_chend = nc;
+    }
 
     /// Destructor is trivial, because we don't hold any resources
     /// of our own.  This is by design.
@@ -395,10 +426,15 @@ public:
     int x (void) const { return m_x; }
     int y (void) const { return m_y; }
     int z (void) const { return m_z; }
+    int chbegin () const { return m_chbegin; }
+    int chend () const { return m_chend; }
+    int nchannels () const { return m_chend - m_chbegin; }
 
     void x (int v) { m_x = v; }
     void y (int v) { m_y = v; }
     void z (int v) { m_z = v; }
+    void xy (int x, int y) { m_x = x; m_y = y; }
+    void xyz (int x, int y, int z) { m_x = x; m_y = y; m_z = z; }
 
     /// Is this an uninitialized tileID?
     bool empty () const { return m_file == NULL; }
@@ -410,7 +446,9 @@ public:
         // probable rejection if they really are unequal.
         return (a.m_x == b.m_x && a.m_y == b.m_y && a.m_z == b.m_z &&
                 a.m_subimage == b.m_subimage && 
-                a.m_miplevel == b.m_miplevel && (a.m_file == b.m_file));
+                a.m_miplevel == b.m_miplevel &&
+                (a.m_file == b.m_file) &&
+                a.m_chbegin == b.m_chbegin && a.m_chend == b.m_chend);
     }
 
     /// Do the two ID's refer to the same tile, given that the
@@ -419,12 +457,14 @@ public:
     friend bool equal_same_subimage (const TileID &a, const TileID &b) {
         DASSERT ((a.m_file == b.m_file) &&
                  a.m_subimage == b.m_subimage && a.m_miplevel == b.m_miplevel);
-        return (a.m_x == b.m_x && a.m_y == b.m_y && a.m_z == b.m_z);
+        return (a.m_x == b.m_x && a.m_y == b.m_y && a.m_z == b.m_z &&
+                a.m_chbegin == b.m_chbegin && a.m_chend == b.m_chend);
     }
 
     /// Do the two ID's refer to the same tile?  
     ///
     bool operator== (const TileID &b) const { return equal (*this, b); }
+    bool operator!= (const TileID &b) const { return ! equal (*this, b); }
 
     /// Digest the TileID into a size_t to use as a hash key.
     size_t hash () const {
@@ -436,7 +476,8 @@ public:
 #else
         // Good compromise!
         return bjhash::bjfinal (m_x+1543, m_y + 6151 + m_z*769,
-                                m_miplevel + (m_subimage<<8))
+                                m_miplevel + (m_subimage<<8) +
+                                (chbegin()<<4) + nchannels())
                            + m_file->filename().hash();
 #endif
     }
@@ -448,10 +489,19 @@ public:
         size_t operator() (const TileID &a) const { return a.hash(); }
     };
 
+    friend std::ostream& operator<< (std::ostream& o, const TileID &id) {
+        return (o << "{xyz=" << id.m_x << ',' << id.m_y << ',' << id.m_z
+                  << ", sub=" << id.m_subimage << ", mip=" << id.m_miplevel
+                  << ", chans=[" << id.chbegin() << "," << id.chend() << ")"
+                  << ' ' << (id.m_file ? ustring("nofile") : id.m_file->filename())
+                  << '}');
+    }
+
 private:
     int m_x, m_y, m_z;        ///< x,y,z tile index within the subimage
     int m_subimage;           ///< subimage
     int m_miplevel;           ///< MIP-map level
+    short m_chbegin, m_chend; ///< Channel range
     ImageCacheFile *m_file;   ///< Which ImageCacheFile we refer to
 };
 
@@ -479,20 +529,33 @@ public:
     /// that constructed the tile.
     void read (ImageCachePerThreadInfo *thread_info);
 
-    /// Return pointer to the floating-point pixel data
-    ///
-    const float *data (void) const { return (const float *)&m_pixels[0]; }
+    /// Return pointer to the raw pixel data
+    const void *data (void) const { return &m_pixels[0]; }
 
-    /// Return pointer to the floating-point pixel data for a particular
-    /// pixel.  Be extremely sure the pixel is within this tile!
-    const void *data (int x, int y, int z=0) const;
+    /// Return pointer to the pixel data for a particular pixel.  Be
+    /// extremely sure the pixel is within this tile!
+    const void *data (int x, int y, int z, int c) const;
+
+    /// Return pointer to the floating-point pixel data
+    const float *floatdata (void) const {
+        return (const float *) &m_pixels[0];
+    }
 
     /// Return a pointer to the character data
-    ///
     const unsigned char *bytedata (void) const {
         return (unsigned char *) &m_pixels[0];
     }
 
+    /// Return a pointer to unsigned short data
+    const unsigned short *ushortdata (void) const {
+        return (unsigned short *) &m_pixels[0];
+    }
+
+    /// Return a pointer to half data
+    const half *halfdata (void) const {
+        return (half *) &m_pixels[0];
+    }
+
     /// Return the id for this tile.
     ///
     const TileID& id (void) const { return m_id; }
@@ -507,11 +570,7 @@ public:
 
     /// Return the space that will be needed for this tile's pixels.
     ///
-    size_t memsize_needed () const {
-        const ImageSpec &spec (file().spec(m_id.subimage(),m_id.miplevel()));
-        TypeDesc datatype = file().datatype(id().subimage());
-        return spec.tile_pixels() * spec.nchannels * datatype.size();
-    }
+    size_t memsize_needed () const;
 
     /// Mark the tile as recently used.
     ///
@@ -541,10 +600,15 @@ public:
     ///
     void wait_pixels_ready () const;
 
+    int channelsize () const { return m_channelsize; }
+    int pixelsize () const { return m_pixelsize; }
+
 private:
     TileID m_id;                  ///< ID of this tile
     boost::scoped_array<char> m_pixels;  ///< The pixel data
     size_t m_pixels_size;         ///< How much m_pixels has allocated
+    int m_channelsize;            ///< How big is each channel (bytes)
+    int m_pixelsize;              ///< How big is each pixel (bytes)
     bool m_valid;                 ///< Valid pixels
     volatile bool m_pixels_ready; ///< The pixels have been read from disk
     atomic_int m_used;            ///< Used recently
@@ -568,7 +632,8 @@ typedef unordered_map_concurrent<TileID, ImageCacheTileRef, TileID::Hasher, std:
 /// ImageCache and TextureSystem, so they don't each need a costly
 /// thread_specific_ptr retrieval.  There's no real penalty for this,
 /// even if you are using only ImageCache but not TextureSystem.
-struct ImageCachePerThreadInfo {
+class ImageCachePerThreadInfo {
+public:
     // Store just a few filename/fileptr pairs
     static const int nlastfile = 4;
     ustring last_filename[nlastfile];
@@ -583,11 +648,16 @@ struct ImageCachePerThreadInfo {
     ImageCachePerThreadInfo ()
         : next_last_file(0), shared(false)
     {
+        // std::cout << "Creating PerThreadInfo " << (void*)this << "\n";
         for (int i = 0;  i < nlastfile;  ++i)
             last_file[i] = NULL;
         purge = 0;
     }
 
+    ~ImageCachePerThreadInfo () {
+        // std::cout << "Destroying PerThreadInfo " << (void*)this << "\n";
+    }
+
     // Add a new filename/fileptr pair to our microcache
     void filename (ustring n, ImageCacheFile *f) {
         last_filename[next_last_file] = n;
@@ -633,24 +703,24 @@ public:
         return attribute (name, TypeDesc::STRING, &s);
     }
 
-    virtual bool getattribute (string_view name, TypeDesc type, void *val);
-    virtual bool getattribute (string_view name, int &val) {
+    virtual bool getattribute (string_view name, TypeDesc type, void *val) const;
+    virtual bool getattribute (string_view name, int &val) const {
         return getattribute (name, TypeDesc::INT, &val);
     }
-    virtual bool getattribute (string_view name, float &val) {
+    virtual bool getattribute (string_view name, float &val) const {
         return getattribute (name, TypeDesc::FLOAT, &val);
     }
-    virtual bool getattribute (string_view name, double &val) {
+    virtual bool getattribute (string_view name, double &val) const {
         float f;
         bool ok = getattribute (name, TypeDesc::FLOAT, &f);
         if (ok)
             val = f;
         return ok;
     }
-    virtual bool getattribute (string_view name, char **val) {
+    virtual bool getattribute (string_view name, char **val) const {
         return getattribute (name, TypeDesc::STRING, val);
     }
-    virtual bool getattribute (string_view name, std::string &val) {
+    virtual bool getattribute (string_view name, std::string &val) const {
         ustring s;
         bool ok = getattribute (name, TypeDesc::STRING, &s);
         if (ok)
@@ -686,6 +756,10 @@ public:
     ///
     virtual bool get_image_info (ustring filename, int subimage, int miplevel,
                          ustring dataname, TypeDesc datatype, void *data);
+    virtual bool get_image_info (ImageCacheFile *file,
+                         ImageCachePerThreadInfo *thread_info,
+                         int subimage, int miplevel,
+                         ustring dataname, TypeDesc datatype, void *data);
 
     /// Get the ImageSpec associated with the named image.  If the file
     /// is found and is an image format that can be read, store a copy
@@ -695,31 +769,44 @@ public:
     virtual bool get_imagespec (ustring filename, ImageSpec &spec,
                                 int subimage=0, int miplevel=0,
                                 bool native=false);
+    virtual bool get_imagespec (ImageCacheFile *file,
+                                ImageCachePerThreadInfo *thread_info,
+                                ImageSpec &spec,
+                                int subimage=0, int miplevel=0,
+                                bool native=false);
 
     virtual const ImageSpec *imagespec (ustring filename, int subimage=0,
                                         int miplevel=0, bool native=false);
+    virtual const ImageSpec *imagespec (ImageCacheFile *file,
+                                        ImageCachePerThreadInfo *thread_info=NULL,
+                                        int subimage=0,
+                                        int miplevel=0, bool native=false);
 
     // Retrieve a rectangle of raw unfiltered pixels.
     virtual bool get_pixels (ustring filename, int subimage, int miplevel,
                              int xbegin, int xend,
                              int ybegin, int yend, int zbegin, int zend,
                              TypeDesc format, void *result);
+    virtual bool get_pixels (ImageCacheFile *file,
+                             ImageCachePerThreadInfo *thread_info,
+                             int subimage, int miplevel, int xbegin, int xend,
+                             int ybegin, int yend, int zbegin, int zend,
+                             TypeDesc format, void *result);
     virtual bool get_pixels (ustring filename,
                     int subimage, int miplevel, int xbegin, int xend,
                     int ybegin, int yend, int zbegin, int zend,
                     int chbegin, int chend, TypeDesc format, void *result,
                     stride_t xstride=AutoStride, stride_t ystride=AutoStride,
-                    stride_t zstride=AutoStride);
-
-    /// Retrieve a rectangle of raw unfiltered pixels, from an open valid
-    /// ImageCacheFile.
-    bool get_pixels (ImageCacheFile *file,
+                    stride_t zstride=AutoStride,
+                    int cache_chbegin = 0, int cache_chend = -1);
+    virtual bool get_pixels (ImageCacheFile *file,
                      ImageCachePerThreadInfo *thread_info,
                      int subimage, int miplevel, int xbegin, int xend,
                      int ybegin, int yend, int zbegin, int zend,
                      int chbegin, int chend, TypeDesc format, void *result,
                      stride_t xstride=AutoStride, stride_t ystride=AutoStride,
-                     stride_t zstride=AutoStride);
+                     stride_t zstride=AutoStride,
+                     int cache_chbegin = 0, int cache_chend = -1);
 
     /// Find the ImageCacheFile record for the named image, or NULL if
     /// no such file can be found.  This returns a plain old pointer,
@@ -727,10 +814,34 @@ public:
     /// and those won't be freed until the texture system is destroyed.
     /// If header_only is true, we are finding the file only for the sake
     /// of header information (e.g., called by get_image_info).
+    /// A call to verify_file() is still needed after find_file().
     ImageCacheFile *find_file (ustring filename,
                                ImageCachePerThreadInfo *thread_info,
                                ImageInput::Creator creator=NULL,
-                               bool header_only=false);
+                               bool header_only=false,
+                               const ImageSpec *config=NULL);
+
+    /// Verify & prep the ImageCacheFile record for the named image,
+    /// return the pointer (which may have changed for deduplication),
+    /// or NULL if no such file can be found. This returns a plain old
+    /// pointer, which is ok because the file hash table has ref-counted
+    /// pointers and those won't be freed until the texture system is
+    /// destroyed.  If header_only is true, we are finding the file only
+    /// for the sake of header information (e.g., called by
+    /// get_image_info).
+    ImageCacheFile *verify_file (ImageCacheFile *tf,
+                                 ImageCachePerThreadInfo *thread_info,
+                                 bool header_only=false);
+    
+    virtual ImageCacheFile * get_image_handle (ustring filename,
+                             ImageCachePerThreadInfo *thread_info=NULL) {
+        ImageCacheFile *file = find_file (filename, thread_info);
+        return verify_file (file, thread_info);
+    }
+
+    virtual bool good (ImageCacheFile *handle) {
+        return handle  &&  ! handle->broken();
+    }
 
     /// Is the tile specified by the TileID already in the cache?
     bool tile_in_cache (const TileID &id,
@@ -772,13 +883,21 @@ public:
     }
 
     virtual Tile *get_tile (ustring filename, int subimage, int miplevel,
-                            int x, int y, int z);
+                            int x, int y, int z, int chbegin, int chend);
+    virtual Tile *get_tile (ImageHandle *file, Perthread *thread_info,
+                            int subimage, int miplevel,
+                            int x, int y, int z, int chbegin, int chend);
     virtual void release_tile (Tile *tile) const;
+    virtual TypeDesc tile_format (const Tile *tile) const;
+    virtual ROI tile_roi (const Tile *tile) const;
     virtual const void * tile_pixels (Tile *tile, TypeDesc &format) const;
-    virtual bool add_file (ustring filename, ImageInput::Creator creator);
+    virtual bool add_file (ustring filename, ImageInput::Creator creator,
+                           const ImageSpec *config);
     virtual bool add_tile (ustring filename, int subimage, int miplevel,
-                     int x, int y, int z, TypeDesc format, const void *buffer,
-                     stride_t xstride, stride_t ystride, stride_t zstride);
+                           int x, int y, int z,  int chbegin, int chend,
+                           TypeDesc format, const void *buffer,
+                           stride_t xstride, stride_t ystride,
+                           stride_t zstride);
 
     /// Return the numerical subimage index for the given subimage name,
     /// as stored in the "oiio:subimagename" metadata.  Return -1 if no
@@ -846,9 +965,9 @@ public:
     /// Append a string to the current error message
     void append_error (const std::string& message) const;
 
-    /// Get a pointer to the caller's thread's per-thread info, or create
-    /// one in the first place if there isn't one already.
-    ImageCachePerThreadInfo *get_perthread_info ();
+    virtual Perthread * get_perthread_info (Perthread *thread_info = NULL);
+    virtual Perthread * create_thread_info ();
+    virtual void destroy_thread_info (Perthread *thread_info);
 
     /// Called when the IC is destroyed.  We have a list of all the 
     /// perthread pointers -- go through and delete the ones for which we
@@ -861,7 +980,7 @@ public:
     /// clear its tile microcache), so don't delete the perthread info
     /// (it will be owned thereafter by the IC).  If there is no IC still
     /// depending on it (signalled by m_imagecache == NULL), delete it.
-    static void cleanup_perthread_info (ImageCachePerThreadInfo *p);
+    static void cleanup_perthread_info (Perthread *thread_info);
 
     /// Ensure that the max_memory_bytes is at least newsize bytes.
     /// Override the previous value if necessary, with thread-safety.
@@ -907,7 +1026,7 @@ private:
 
     thread_specific_ptr< ImageCachePerThreadInfo > m_perthread_info;
     std::vector<ImageCachePerThreadInfo *> m_all_perthread_info;
-    static mutex m_perthread_info_mutex; ///< Thread safety for perthread
+    static spin_mutex m_perthread_info_mutex; ///< Thread safety for perthread
     int m_max_open_files;
     atomic_ll m_max_memory_bytes;
     std::string m_searchpath;    ///< Colon-separated image directory list
@@ -987,8 +1106,7 @@ private:
 
 }  // end namespace pvt
 
-}
-OIIO_NAMESPACE_EXIT
+OIIO_NAMESPACE_END
 
 
 #endif // OPENIMAGEIO_IMAGECACHE_PVT_H
