@@ -40,18 +40,21 @@
 #include <boost/foreach.hpp>
 #include <boost/regex.hpp>
 
-#ifdef _WIN32
-#include <windows.h>
-#include <shellapi.h>
-#endif
-
 #include "OpenImageIO/dassert.h"
 #include "OpenImageIO/ustring.h"
 #include "OpenImageIO/filesystem.h"
+#include "OpenImageIO/refcnt.h"
+
+#ifdef _WIN32
+#include <windows.h>
+#include <shellapi.h>
+#include <direct.h>
+#else
+#include <unistd.h>
+#endif
 
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
 
 
 std::string
@@ -199,7 +202,12 @@ Filesystem::get_directory_entries (const std::string &dirname,
     if (dirname.size() && ! is_directory(dirname))
         return false;
     boost::filesystem::path dirpath (dirname.size() ? dirname : std::string("."));
-    boost::regex re (filter_regex);
+    boost::regex re;
+    try {
+        re = boost::regex(filter_regex);
+    } catch (...) {
+        return false;
+    }
 
     if (recursive) {
         for (boost::filesystem::recursive_directory_iterator s (dirpath);
@@ -248,8 +256,17 @@ Filesystem::exists (const std::string &path)
 {
     bool r = false;
     try {
+#if defined(_WIN32)
+        // boost internally doesn't use MultiByteToWideChar (CP_UTF8,...
+        // to convert char* to wchar_t* because they do not know the encoding
+        // See boost::filesystem::path.hpp 
+        // The only correct way to do this is to do the conversion ourselves
+        std::wstring wpath = Strutil::utf8_to_utf16(path);
+        r = boost::filesystem::exists (wpath);
+#else
         r = boost::filesystem::exists (path);
-    } catch (const std::exception &) {
+#endif
+    } catch (...) {
         r = false;
     }
     return r;
@@ -262,8 +279,17 @@ Filesystem::is_directory (const std::string &path)
 {
     bool r = false;
     try {
+#if defined(_WIN32)
+        // boost internally doesn't use MultiByteToWideChar (CP_UTF8,...
+        // to convert char* to wchar_t* because they do not know the encoding
+        // See boost::filesystem::path.hpp 
+        // The only correct way to do this is to do the conversion ourselves
+        std::wstring wpath = Strutil::utf8_to_utf16(path);
+        r = boost::filesystem::is_directory (wpath);
+#else
         r = boost::filesystem::is_directory (path);
-    } catch (const std::exception &) {
+#endif
+    } catch (...) {
         r = false;
     }
     return r;
@@ -276,8 +302,17 @@ Filesystem::is_regular (const std::string &path)
 {
     bool r = false;
     try {
+#if defined(_WIN32)
+        // boost internally doesn't use MultiByteToWideChar (CP_UTF8,...
+        // to convert char* to wchar_t* because they do not know the encoding
+        // See boost::filesystem::path.hpp 
+        // The only correct way to do this is to do the conversion ourselves
+        std::wstring wpath = Strutil::utf8_to_utf16(path);
+        r = boost::filesystem::is_regular_file (wpath);
+#else
         r = boost::filesystem::is_regular_file (path);
-    } catch (const std::exception &) {
+#endif
+    } catch (...) {
         r = false;
     }
     return r;
@@ -288,16 +323,27 @@ Filesystem::is_regular (const std::string &path)
 bool
 Filesystem::create_directory (string_view path, std::string &err)
 {
+
+#if defined(_WIN32)
+	// boost internally doesn't use MultiByteToWideChar (CP_UTF8,...
+	// to convert char* to wchar_t* because they do not know the encoding
+	// See boost::filesystem::path.hpp 
+	// The only correct way to do this is to do the conversion ourselves
+	std::wstring pathStr = Strutil::utf8_to_utf16(path);
+#else
+	std::string pathStr = path.str();
+#endif
+
 #if BOOST_FILESYSTEM_VERSION >= 3
     boost::system::error_code ec;
-    bool ok = boost::filesystem::create_directory (path.str(), ec);
+	bool ok = boost::filesystem::create_directory (pathStr, ec);
     if (ok)
         err.clear();
     else
         err = ec.message();
     return ok;
 #else
-    bool ok = boost::filesystem::create_directory (path.str());
+    bool ok = boost::filesystem::create_directory (pathStr);
     if (ok)
         err.clear();
     else
@@ -310,14 +356,61 @@ Filesystem::create_directory (string_view path, std::string &err)
 bool
 Filesystem::copy (string_view from, string_view to, std::string &err)
 {
+#if defined(_WIN32)
+	// boost internally doesn't use MultiByteToWideChar (CP_UTF8,...
+	// to convert char* to wchar_t* because they do not know the encoding
+	// See boost::filesystem::path.hpp 
+	// The only correct way to do this is to do the conversion ourselves
+	std::wstring fromStr = Strutil::utf8_to_utf16(from);
+	std::wstring toStr = Strutil::utf8_to_utf16(to);
+#else
+	std::string fromStr = from.str();
+	std::string toStr = to.str();
+#endif
+
 #if BOOST_FILESYSTEM_VERSION >= 3
     boost::system::error_code ec;
 # if BOOST_VERSION < 105000
-    boost::filesystem3::copy (from.str(), to.str(), ec);
+    boost::filesystem3::copy (fromStr, toStr, ec);
 # else
-    boost::filesystem::copy (from.str(), to.str(), ec);
+    boost::filesystem::copy (fromStr, toStr, ec);
 # endif
-    if (ec) {
+    if (! ec) {
+        err.clear();
+        return true;
+    } else {
+        err = ec.message();
+        return false;
+    }
+#else
+    return false; // I'm too lazy to figure this out.
+#endif
+}
+
+
+
+bool
+Filesystem::rename (string_view from, string_view to, std::string &err)
+{
+#if defined(_WIN32)
+	// boost internally doesn't use MultiByteToWideChar (CP_UTF8,...
+	// to convert char* to wchar_t* because they do not know the encoding
+	// See boost::filesystem::path.hpp 
+	// The only correct way to do this is to do the conversion ourselves
+	std::wstring fromStr = Strutil::utf8_to_utf16(from);
+	std::wstring toStr = Strutil::utf8_to_utf16(to);
+#else
+	std::string fromStr = from.str();
+	std::string toStr = to.str();
+#endif
+#if BOOST_FILESYSTEM_VERSION >= 3
+    boost::system::error_code ec;
+# if BOOST_VERSION < 105000
+    boost::filesystem3::rename (fromStr, toStr, ec);
+# else
+    boost::filesystem::rename (fromStr, toStr, ec);
+# endif
+    if (! ec) {
         err.clear();
         return true;
     } else {
@@ -334,16 +427,25 @@ Filesystem::copy (string_view from, string_view to, std::string &err)
 bool
 Filesystem::remove (string_view path, std::string &err)
 {
+#if defined(_WIN32)
+	// boost internally doesn't use MultiByteToWideChar (CP_UTF8,...
+	// to convert char* to wchar_t* because they do not know the encoding
+	// See boost::filesystem::path.hpp 
+	// The only correct way to do this is to do the conversion ourselves
+	std::wstring pathStr = Strutil::utf8_to_utf16(path);
+#else
+	std::string pathStr = path.str();
+#endif
 #if BOOST_FILESYSTEM_VERSION >= 3
     boost::system::error_code ec;
-    bool ok = boost::filesystem::remove (path.str(), ec);
+    bool ok = boost::filesystem::remove (pathStr, ec);
     if (ok)
         err.clear();
     else
         err = ec.message();
     return ok;
 #else
-    bool ok = boost::filesystem::remove (path.str());
+    bool ok = boost::filesystem::remove (pathStr);
     if (ok)
         err.clear();
     else
@@ -357,16 +459,25 @@ Filesystem::remove (string_view path, std::string &err)
 unsigned long long
 Filesystem::remove_all (string_view path, std::string &err)
 {
+#if defined(_WIN32)
+	// boost internally doesn't use MultiByteToWideChar (CP_UTF8,...
+	// to convert char* to wchar_t* because they do not know the encoding
+	// See boost::filesystem::path.hpp 
+	// The only correct way to do this is to do the conversion ourselves
+	std::wstring pathStr = Strutil::utf8_to_utf16(path);
+#else
+	std::string pathStr = path.str();
+#endif
 #if BOOST_FILESYSTEM_VERSION >= 3
     boost::system::error_code ec;
-    unsigned long long n = boost::filesystem::remove_all (path.str(), ec);
+    unsigned long long n = boost::filesystem::remove_all (pathStr, ec);
     if (!ec)
         err.clear();
     else
         err = ec.message();
     return n;
 #else
-    unsigned long long n = boost::filesystem::remove_all (path.str());
+    unsigned long long n = boost::filesystem::remove_all (pathStr);
     err.clear();
     return n;
 #endif
@@ -399,9 +510,18 @@ Filesystem::temp_directory_path()
 std::string
 Filesystem::unique_path (string_view model)
 {
+#if defined(_WIN32)
+	// boost internally doesn't use MultiByteToWideChar (CP_UTF8,...
+	// to convert char* to wchar_t* because they do not know the encoding
+	// See boost::filesystem::path.hpp 
+	// The only correct way to do this is to do the conversion ourselves
+	std::wstring modelStr = Strutil::utf8_to_utf16(model);
+#else
+	std::string modelStr = model.str();
+#endif
 #if BOOST_FILESYSTEM_VERSION >= 3
     boost::system::error_code ec;
-    boost::filesystem::path p = boost::filesystem::unique_path (model.str(), ec);
+    boost::filesystem::path p = boost::filesystem::unique_path (modelStr, ec);
     return ec ? std::string() : p.string();
 #elif _MSC_VER
     char buf[TMP_MAX];
@@ -416,6 +536,27 @@ Filesystem::unique_path (string_view model)
 
 
 
+std::string
+Filesystem::current_path()
+{
+#if BOOST_FILESYSTEM_VERSION >= 3
+    boost::system::error_code ec;
+    boost::filesystem::path p = boost::filesystem::current_path (ec);
+    return ec ? std::string() : p.string();
+#else
+    // Fallback if we don't have recent Boost
+    char path[FILENAME_MAX];
+#ifdef _WIN32
+    bool ok = _getcwd (path, sizeof(path));
+#else
+    bool ok = getcwd (path, sizeof(path));
+#endif
+    return ok ? std::string(path) : std::string();
+#endif
+}
+
+
+
 FILE*
 Filesystem::fopen (string_view path, string_view mode)
 {
@@ -434,11 +575,12 @@ Filesystem::fopen (string_view path, string_view mode)
 
 
 void
-Filesystem::open (std::ifstream &stream, string_view path,
+Filesystem::open (OIIO::ifstream &stream, string_view path,
                   std::ios_base::openmode mode)
 {
 #ifdef _WIN32
     // Windows std::ifstream accepts non-standard wchar_t* 
+	// On MingW, we use our own OIIO::ifstream
     std::wstring wpath = Strutil::utf8_to_utf16(path);
     stream.open (wpath.c_str(), mode);
     stream.seekg (0, std::ios_base::beg); // force seek, otherwise broken
@@ -450,11 +592,12 @@ Filesystem::open (std::ifstream &stream, string_view path,
 
 
 void
-Filesystem::open (std::ofstream &stream, string_view path,
+Filesystem::open (OIIO::ofstream &stream, string_view path,
                   std::ios_base::openmode mode)
 {
-#ifdef _WIN32
+#ifdef _WIN32 
     // Windows std::ofstream accepts non-standard wchar_t*
+	// On MingW, we use our own OIIO::ofstream
     std::wstring wpath = Strutil::utf8_to_utf16 (path);
     stream.open (wpath.c_str(), mode);
 #else
@@ -463,6 +606,27 @@ Filesystem::open (std::ofstream &stream, string_view path,
 }
 
 
+/// Read the entire contents of the named file and place it in str,
+/// returning true on success, false on failure.
+bool
+Filesystem::read_text_file (string_view filename, std::string &str)
+{
+    // For info on why this is the fastest method:
+    // http://insanecoding.blogspot.com/2011/11/how-to-read-in-file-in-c.html
+    OIIO::ifstream in;
+    Filesystem::open (in, filename);
+
+    // N.B. for binary read: open(in, filename, std::ios::in|std::ios::binary);
+    if (in) {
+        std::ostringstream contents;
+        contents << in.rdbuf();
+        str = contents.str();
+        return true;
+    }
+    return false;
+}
+
+
 
 std::time_t
 Filesystem::last_write_time (const std::string& path)
@@ -474,7 +638,7 @@ Filesystem::last_write_time (const std::string& path)
 #else
         return boost::filesystem::last_write_time (path);
 #endif
-    } catch (const std::exception &) {
+    } catch (...) {
         // File doesn't exist
         return 0;
     }
@@ -492,7 +656,7 @@ Filesystem::last_write_time (const std::string& path, std::time_t time)
 #else
         boost::filesystem::last_write_time (path, time);
 #endif
-    } catch (const std::exception &) {
+    } catch (...) {
         // File doesn't exist
     }
 }
@@ -641,6 +805,7 @@ Filesystem::enumerate_file_sequence (const std::string &pattern,
                                      const std::vector<int> &numbers,
                                      std::vector<std::string> &filenames)
 {
+    filenames.clear ();
     for (size_t i = 0, e = numbers.size(); i < e; ++i) {
         std::string f = Strutil::format (pattern.c_str(), numbers[i]);
         filenames.push_back (f);
@@ -656,10 +821,10 @@ Filesystem::enumerate_file_sequence (const std::string &pattern,
                                      const std::vector<string_view> &views,
                                      std::vector<std::string> &filenames)
 {
-    DASSERT (views.size() == 0 || views.size() == numbers.size());
-
+    ASSERT (views.size() == 0 || views.size() == numbers.size());
     static boost::regex view_re ("%V"), short_view_re ("%v");
 
+    filenames.clear ();
     for (size_t i = 0, e = numbers.size(); i < e; ++i) {
         std::string f = pattern;
         if (views.size() > 0 && ! views[i].empty()) {
@@ -685,6 +850,9 @@ Filesystem::scan_for_matching_filenames(const std::string &pattern,
     static boost::regex format_re ("%0([0-9]+)d");
     static boost::regex all_views_re ("%[Vv]"), view_re ("%V"), short_view_re ("%v");
 
+    frame_numbers.clear ();
+    frame_views.clear ();
+    filenames.clear ();
     if (boost::regex_search (pattern, all_views_re)) {
         if (boost::regex_search (pattern, format_re)) {
             // case 1: pattern has format and view
@@ -754,8 +922,9 @@ Filesystem::scan_for_matching_filenames(const std::string &pattern_,
                                         std::vector<int> &numbers,
                                         std::vector<std::string> &filenames)
 {
+    numbers.clear ();
+    filenames.clear ();
     std::string pattern = pattern_;
-
     // Isolate the directory name (or '.' if none was specified)
     std::string directory = Filesystem::parent_path (pattern);
     if (directory.size() == 0) {
@@ -780,7 +949,7 @@ Filesystem::scan_for_matching_filenames(const std::string &pattern_,
     std::string prefix (format_match.prefix().first, format_match.prefix().second);
     std::string suffix (format_match.suffix().first, format_match.suffix().second);
 
-    std::string pattern_re_str = prefix + "([0-9]{" + thepadding + "})" + suffix;
+    std::string pattern_re_str = prefix + "([0-9]{" + thepadding + ",})" + suffix;
     std::vector< std::pair< int, std::string > > matches;
 
     // There are some corner cases regex that could be constructed here that
@@ -807,7 +976,7 @@ Filesystem::scan_for_matching_filenames(const std::string &pattern_,
         }
     }
 
-    } catch (std::exception &e) {
+    } catch (...) {
         // Botched regex. Just fail.
         return false;
     }
@@ -823,5 +992,4 @@ Filesystem::scan_for_matching_filenames(const std::string &pattern_,
     return true;
 }
 
-}
-OIIO_NAMESPACE_EXIT
+OIIO_NAMESPACE_END
