@@ -31,9 +31,9 @@
 #include <cstdlib>
 #include <string>
 
-#ifdef _WIN32
-# include "OpenImageIO/osdep.h"
-#else
+#include "OpenImageIO/platform.h"
+
+#ifndef _WIN32
 # include <dlfcn.h>
 #endif
 
@@ -41,8 +41,7 @@
 #include "OpenImageIO/plugin.h"
 
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
 
 using namespace Plugin;
 
@@ -164,5 +163,4 @@ Plugin::geterror (void)
     return e;
 }
 
-}
-OIIO_NAMESPACE_EXIT
+OIIO_NAMESPACE_END
