@@ -36,7 +36,6 @@
 #include <cmath>
 #include <sstream>
 #include <limits>
-#include <boost/tokenizer.hpp>
 #include <boost/foreach.hpp>
 #include <boost/algorithm/string.hpp>
 
@@ -61,8 +60,7 @@
 #include "OpenImageIO/string_view.h"
 
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
 
 
 const char *
@@ -206,11 +204,10 @@ Strutil::get_rest_arguments (const std::string &str, std::string &base,
     }
 
     base = str.substr (0, mark_pos);
-
-    boost::char_separator<char> sep ("&");
     std::string rest = str.substr (mark_pos + 1);
-    boost::tokenizer<boost::char_separator<char> > rest_tokens (rest, sep);
-    BOOST_FOREACH (std::string keyval, rest_tokens) {
+    std::vector<std::string> rest_tokens;
+    Strutil::split (rest, rest_tokens, "&");
+    BOOST_FOREACH (const std::string &keyval, rest_tokens) {
         mark_pos = keyval.find_first_of ("=");
         if (mark_pos == std::string::npos)
             return false;
@@ -493,6 +490,17 @@ Strutil::join (const std::vector<std::string> &seq, string_view str)
 
 
 
+std::string
+Strutil::repeat (string_view str, int n)
+{
+    std::ostringstream out;
+    while (n-- > 0)
+        out << str;
+    return out.str();
+}
+
+
+
 #ifdef _WIN32
 std::wstring
 Strutil::utf8_to_utf16 (string_view str)
@@ -605,13 +613,14 @@ Strutil::parse_int (string_view &str, int &val, bool eat)
     if (! p.size())
         return false;
     const char *end = p.begin();
-    val = strtol (p.begin(), (char**)&end, 10);
+    int v = strtol (p.begin(), (char**)&end, 10);
     if (end == p.begin())
         return false;  // no integer found
     if (eat) {
         p.remove_prefix (end-p.begin());
         str = p;
     }
+    val = v;
     return true;
 }
 
@@ -625,13 +634,14 @@ Strutil::parse_float (string_view &str, float &val, bool eat)
     if (! p.size())
         return false;
     const char *end = p.begin();
-    val = (float) strtod (p.begin(), (char**)&end);
+    float v = (float) strtod (p.begin(), (char**)&end);
     if (end == p.begin())
         return false;  // no integer found
     if (eat) {
         p.remove_prefix (end-p.begin());
         str = p;
     }
+    val = v;
     return true;
 }
 
@@ -639,6 +649,15 @@ Strutil::parse_float (string_view &str, float &val, bool eat)
 
 bool
 Strutil::parse_string (string_view &str, string_view &val, bool eat)
+{
+    return parse_string (str, val, eat, DeleteQuotes);
+}
+
+
+
+bool
+Strutil::parse_string (string_view &str, string_view &val,
+                       bool eat, QuoteBehavior keep_quotes)
 {
     string_view p = str;
     skip_whitespace (p);
@@ -655,7 +674,14 @@ Strutil::parse_string (string_view &str, string_view &val, bool eat)
         ++end;
         escaped = false;
     }
-    val = string_view (begin, size_t(end-begin));
+    if (quoted && keep_quotes == KeepQuotes) {
+        if (*end == '\"')
+            val = string_view (begin-1, size_t(end-begin)+2);
+        else
+            val = string_view (begin-1, size_t(end-begin)+1);
+    } else {
+        val = string_view (begin, size_t(end-begin));
+    }
     p.remove_prefix (size_t(end-begin));
     if (quoted && p.size() && p[0] == '\"')
         p.remove_prefix (1);  // eat closing quote
@@ -744,5 +770,118 @@ Strutil::parse_until (string_view &str, string_view sep, bool eat)
 }
 
 
+string_view
+Strutil::parse_nested (string_view &str, bool eat)
+{
+    // Make sure we have a valid string and ascertain the characters that
+    // nest and unnest.
+    string_view p = str;
+    if (! p.size())
+        return string_view();    // No proper opening
+    char opening = p[0];
+    char closing = 0;
+    if      (opening == '(') closing = ')';
+    else if (opening == '[') closing = ']';
+    else if (opening == '{') closing = '}';
+    else    return string_view();
+
+    // Walk forward in the string until we exactly unnest compared to the
+    // start.
+    size_t len = 1;
+    int nesting = 1;
+    for ( ; nesting && len < p.size(); ++len) {
+        if (p[len] == opening)
+            ++nesting;
+        else if (p[len] == closing)
+            --nesting;
+    }
+
+    if (nesting)
+        return string_view();    // No proper closing
+
+    ASSERT (p[len-1] == closing);
+
+    // The result is the first len characters
+    string_view result = str.substr (0, len);
+    if (eat)
+        str.remove_prefix (len);
+    return result;
 }
-OIIO_NAMESPACE_EXIT
+
+
+
+
+/*
+Copyright for decode function:
+See http://bjoern.hoehrmann.de/utf-8/decoder/dfa/ for details.
+
+Copyright (c) 2008-2009 Bjoern Hoehrmann <bjoern@hoehrmann.de>
+
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to
+deal in the Software without restriction, including without limitation the
+rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+sell copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+IN THE SOFTWARE.
+*/
+
+#define UTF8_ACCEPT 0
+#define UTF8_REJECT 12
+
+static const uint8_t utf8d[] = {
+  // The first part of the table maps bytes to character classes that
+  // to reduce the size of the transition table and create bitmasks.
+   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+   1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
+   7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,  7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
+   8,8,2,2,2,2,2,2,2,2,2,2,2,2,2,2,  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
+  10,3,3,3,3,3,3,3,3,3,3,3,3,4,3,3, 11,6,6,6,5,8,8,8,8,8,8,8,8,8,8,8,
+
+  // The second part is a transition table that maps a combination
+  // of a state of the automaton and a character class to a state.
+   0,12,24,36,60,96,84,12,12,12,48,72, 12,12,12,12,12,12,12,12,12,12,12,12,
+  12, 0,12,12,12,12,12, 0,12, 0,12,12, 12,24,12,12,12,12,12,24,12,24,12,12,
+  12,12,12,12,12,12,12,24,12,12,12,12, 12,24,12,12,12,12,12,12,12,24,12,12,
+  12,12,12,12,12,12,12,36,12,36,12,12, 12,36,12,12,12,12,12,36,12,36,12,12,
+  12,36,12,12,12,12,12,12,12,12,12,12,
+};
+
+inline uint32_t
+decode(uint32_t* state, uint32_t* codep, uint32_t byte)
+{
+  uint32_t type = utf8d[byte];
+  *codep = (*state != UTF8_ACCEPT) ?
+    (byte & 0x3fu) | (*codep << 6) :
+    (0xff >> type) & (byte);
+  *state = utf8d[256 + *state + type];
+  return *state;
+}
+
+void
+Strutil::utf8_to_unicode (string_view &str, std::vector<uint32_t> &uvec)
+{
+    const char* begin = str.begin();
+    const char* end = str.end();
+    uint32_t state = 0;
+    for (; begin != end; ++begin) {
+        uint32_t codepoint;
+        if (!decode(&state, &codepoint, (unsigned char) *begin))
+            uvec.push_back(codepoint);
+    }
+}
+
+OIIO_NAMESPACE_END
