@@ -28,6 +28,8 @@
   (This is the Modified BSD License)
 */
 
+#include <cstdio>
+
 #include "OpenImageIO/strutil.h"
 #include "OpenImageIO/unittest.h"
 
@@ -263,6 +265,10 @@ void test_split ()
     OIIO_CHECK_EQUAL (splits.size(), 2);
     OIIO_CHECK_EQUAL (splits[0], "Now\nis");
     OIIO_CHECK_EQUAL (splits[1], "the  time!");
+
+    Strutil::split ("blah", splits, "!");
+    OIIO_CHECK_EQUAL (splits.size(), 1);
+    OIIO_CHECK_EQUAL (splits[0], "blah");
 }
 
 
@@ -280,6 +286,17 @@ void test_join ()
 
 
 
+void test_repeat ()
+{
+    std::cout << "Testing repeat\n";
+    OIIO_CHECK_EQUAL (Strutil::repeat("foo",3), "foofoofoo");
+    OIIO_CHECK_EQUAL (Strutil::repeat("foo",1), "foo");
+    OIIO_CHECK_EQUAL (Strutil::repeat("foo",0), "");
+    OIIO_CHECK_EQUAL (Strutil::repeat("foo",-1), "");
+}
+
+
+
 void test_conversion ()
 {
     OIIO_CHECK_EQUAL (Strutil::from_string<int>("hi"), 0);
@@ -302,41 +319,55 @@ void test_conversion ()
 void test_extract ()
 {
     std::vector<int> vals;
+    int n;
 
     vals.clear(); vals.resize (3, -1);
-    Strutil::extract_from_list_string (vals, "1");
+    n = Strutil::extract_from_list_string (vals, "1");
     OIIO_CHECK_EQUAL (vals.size(), 3);
     OIIO_CHECK_EQUAL (vals[0], 1);
     OIIO_CHECK_EQUAL (vals[1], 1);
     OIIO_CHECK_EQUAL (vals[2], 1);
+    OIIO_CHECK_EQUAL (n, 1);
 
     vals.clear(); vals.resize (3, -1);
-    Strutil::extract_from_list_string (vals, "1,3,5");
+    n = Strutil::extract_from_list_string (vals, "1,3,5");
     OIIO_CHECK_EQUAL (vals.size(), 3);
     OIIO_CHECK_EQUAL (vals[0], 1);
     OIIO_CHECK_EQUAL (vals[1], 3);
     OIIO_CHECK_EQUAL (vals[2], 5);
+    OIIO_CHECK_EQUAL (n, 3);
 
     vals.clear(); vals.resize (3, -1);
-    Strutil::extract_from_list_string (vals, "1,,5");
+    n = Strutil::extract_from_list_string (vals, "1,,5");
     OIIO_CHECK_EQUAL (vals.size(), 3);
     OIIO_CHECK_EQUAL (vals[0], 1);
     OIIO_CHECK_EQUAL (vals[1], -1);
     OIIO_CHECK_EQUAL (vals[2], 5);
+    OIIO_CHECK_EQUAL (n, 3);
 
     vals.clear(); vals.resize (3, -1);
-    Strutil::extract_from_list_string (vals, "abc");
+    n = Strutil::extract_from_list_string (vals, "abc");
     OIIO_CHECK_EQUAL (vals.size(), 3);
     OIIO_CHECK_EQUAL (vals[0], 0);
     OIIO_CHECK_EQUAL (vals[1], 0);
     OIIO_CHECK_EQUAL (vals[2], 0);
+    OIIO_CHECK_EQUAL (n, 1);
 
     vals.clear(); vals.resize (3, -1);
-    Strutil::extract_from_list_string (vals, "");
+    n = Strutil::extract_from_list_string (vals, "");
     OIIO_CHECK_EQUAL (vals.size(), 3);
     OIIO_CHECK_EQUAL (vals[0], -1);
     OIIO_CHECK_EQUAL (vals[1], -1);
     OIIO_CHECK_EQUAL (vals[2], -1);
+    OIIO_CHECK_EQUAL (n, 0);
+
+    vals.clear();
+    n = Strutil::extract_from_list_string (vals, "1,3,5");
+    OIIO_CHECK_EQUAL (vals.size(), 3);
+    OIIO_CHECK_EQUAL (vals[0], 1);
+    OIIO_CHECK_EQUAL (vals[1], 3);
+    OIIO_CHECK_EQUAL (vals[2], 5);
+    OIIO_CHECK_EQUAL (n, 3);
 }
 
 
@@ -459,6 +490,8 @@ void test_parse ()
     OIIO_CHECK_ASSERT (parse_string (s, ss) && ss == "foo bar" && s == " baz");
     s = "\"foo bar\" baz";
     OIIO_CHECK_ASSERT (parse_string (s, ss, false) && ss == "foo bar" && s == "\"foo bar\" baz");
+    s = "\"foo bar\" baz";
+    OIIO_CHECK_ASSERT (parse_string (s, ss, true, KeepQuotes) && ss == "\"foo bar\"" && s == " baz");
 
     s = " foo bar"; ss = parse_word (s);
     OIIO_CHECK_ASSERT (ss == "foo" && s == " bar");
@@ -486,6 +519,43 @@ void test_parse ()
     OIIO_CHECK_ASSERT (ss == "foo" && s == ";bar blow");
     s = "foo;bar blow"; ss = parse_until (s, "\t ");
     OIIO_CHECK_ASSERT (ss == "foo;bar" && s == " blow");
+    s = "foo;bar blow"; ss = parse_until (s, "/");
+    OIIO_CHECK_ASSERT (ss == "foo;bar blow" && s == "");
+
+    s = "[a([b]c)]x]"; ss = parse_nested (s);
+    OIIO_CHECK_EQUAL (ss, "[a([b]c)]"); OIIO_CHECK_EQUAL (s, "x]");
+    s = "[a([b]c)]x]"; ss = parse_nested (s, false);
+    OIIO_CHECK_EQUAL (ss, "[a([b]c)]"); OIIO_CHECK_EQUAL (s, "[a([b]c)]x]");
+}
+
+
+
+
+void
+test_float_formatting ()
+{
+    // For every possible float value, test that printf("%.9g"), which
+    // we are sure preserves full precision as text, exactly matches
+    // Strutil::format("%.9g") and also matches stream output with
+    // precision(9).  VERY EXPENSIVE!  Takes tens of minutes to run.
+    // Don't do this unless you really need to test it.
+    for (unsigned long long i = 0;  i <= (unsigned long long)0xffffffff;  ++i) {
+        unsigned int i32 = (unsigned int)i;
+        float *f = (float *)&i32;
+        std::ostringstream sstream;
+        sstream.precision (9);
+        sstream << *f;
+        char buffer[64];
+        sprintf (buffer, "%.9g", *f);
+        std::string tiny = Strutil::format ("%.9g", *f);
+        if (sstream.str() != tiny || tiny != buffer)
+            printf ("%x  stream '%s'  printf '%s'  Strutil::format '%s'\n",
+                    i32, sstream.str().c_str(), buffer, tiny.c_str());
+        if ((i32 & 0xfffffff) == 0xfffffff) {
+            printf ("%x\n", i32);
+            fflush (stdout);
+        }
+    }
 }
 
 
@@ -504,11 +574,13 @@ main (int argc, char *argv[])
     test_strip ();
     test_split ();
     test_join ();
+    test_repeat ();
     test_conversion ();
     test_extract ();
     test_safe_strcpy ();
     test_string_view ();
     test_parse ();
+    // test_float_formatting ();
 
     return unit_test_failures;
 }
