@@ -58,8 +58,9 @@
 # include <sys/sysctl.h>
 #endif
 
+#include "OpenImageIO/platform.h"
+
 #ifdef _WIN32
-# include "OpenImageIO/osdep.h"
 # include <Psapi.h>
 #else
 # include <sys/resource.h>
@@ -73,8 +74,11 @@
 #include "OpenImageIO/dassert.h"
 #include "OpenImageIO/sysutil.h"
 
-OIIO_NAMESPACE_ENTER
-{
+#include <boost/version.hpp>
+#include <boost/thread.hpp>
+
+
+OIIO_NAMESPACE_BEGIN
 
 using namespace Sysutil;
 
@@ -339,5 +343,24 @@ Sysutil::put_in_background (int, char* [])
 }
 
 
+
+unsigned int
+Sysutil::hardware_concurrency ()
+{
+    return boost::thread::hardware_concurrency();
 }
-OIIO_NAMESPACE_EXIT
+
+
+
+unsigned int
+Sysutil::physical_concurrency ()
+{
+#if BOOST_VERSION >= 105600
+    return boost::thread::physical_concurrency();
+#else
+    return boost::thread::hardware_concurrency();
+#endif
+}
+
+
+OIIO_NAMESPACE_END
