@@ -43,6 +43,7 @@
 
 #include "OpenImageIO/argparse.h"
 #include "OpenImageIO/strutil.h"
+#include "OpenImageIO/sysutil.h"
 #include "OpenImageIO/imageio.h"
 #include "OpenImageIO/imagebuf.h"
 #include "OpenImageIO/imagebufalgo.h"
@@ -124,9 +125,13 @@ dump_data (ImageInput *input, const print_info_options &opt)
                     for (int s = 0;  s < nsamples;  ++s) {
                         if (s)
                             std::cout << " / ";
-                        for (int c = 0;  c < nc;  ++c)
-                            std::cout << " " << spec.channelnames[c] << "="
-                                      << dd.deep_value (pixel, c, s);
+                        for (int c = 0;  c < nc;  ++c) {
+                            std::cout << " " << spec.channelnames[c] << "=";
+                            if (dd.channeltypes[c] == TypeDesc::UINT)
+                                std::cout << ((uint32_t *)dd.pointers[pixel*dd.nchannels+c])[s];
+                            else
+                                std::cout << dd.deep_value (pixel, c, s);
+                        }
                     }
                     std::cout << "\n";
                 }
@@ -135,7 +140,7 @@ dump_data (ImageInput *input, const print_info_options &opt)
 
     } else {
         std::vector<float> buf(spec.image_pixels() * spec.nchannels);
-        if (! input->read_image (TypeDesc::UNKNOWN /*native*/, &buf[0])) {
+        if (! input->read_image (TypeDesc::FLOAT, &buf[0])) {
             printf ("    dump data: could not read image\n");
             return;
         }
@@ -175,8 +180,6 @@ read_input (const std::string &filename, ImageBuf &img,
         img.read (subimage, miplevel, false, TypeDesc::FLOAT))
         return true;
 
-    std::cerr << "oiiotool ERROR: Could not read " << filename << ":\n\t"
-              << img.geterror() << "\n";
     return false;
 }
 
@@ -246,7 +249,8 @@ print_stats_footer (unsigned int maxval)
 
 
 static void
-print_stats (const std::string &filename,
+print_stats (Oiiotool &ot,
+             const std::string &filename,
              const ImageSpec &originalspec,
              int subimage=0, int miplevel=0, bool indentmip=false)
 {
@@ -254,13 +258,14 @@ print_stats (const std::string &filename,
     ImageBuf input;
     
     if (! read_input (filename, input, subimage, miplevel)) {
-        std::cerr << "Stats: read error: " << input.geterror() << "\n";
+        ot.error ("stats", input.geterror());
         return;
     }
-    
     PixelStats stats;
     if (! computePixelStats (stats, input)) {
-        printf ("%sStats: (unable to compute)\n", indent);
+        std::string err = input.geterror();
+        ot.error ("stats", Strutil::format ("unable to compute: %s",
+                                            err.empty() ? "unspecified error" : err.c_str()));
         return;
     }
     
@@ -337,6 +342,7 @@ print_stats (const std::string &filename,
         int yend = originalspec.y + originalspec.height;
         int zend = originalspec.z + originalspec.depth;
         size_t p = 0;
+        std::vector<size_t> nsamples_histogram;
         for (int z = originalspec.z; z < zend; ++z) {
             for (int y = originalspec.y; y < yend; ++y) {
                 for (int x = originalspec.x; x < xend; ++x, ++p) {
@@ -353,6 +359,9 @@ print_stats (const std::string &filename,
                         minsamples = c;
                     if (c == 0)
                         ++emptypixels;
+                    if (c >= nsamples_histogram.size())
+                        nsamples_histogram.resize (c+1, 0);
+                    nsamples_histogram[c] += 1;
                     if (depthchannel >= 0) {
                         for (unsigned int s = 0;  s < c;  ++s) {
                             float d = input.deep_value (x, y, z, depthchannel, s);
@@ -381,10 +390,32 @@ print_stats (const std::string &filename,
         printf ("%sTotal deep samples in all pixels: %llu\n", indent, (unsigned long long)totalsamples);
         printf ("%sPixels with deep samples   : %llu\n", indent, (unsigned long long)(npixels-emptypixels));
         printf ("%sPixels with no deep samples: %llu\n", indent, (unsigned long long)emptypixels);
-        printf ("%sMinimum depth was %g at (%d, %d)\n", indent, mindepth,
-                mindepth_pixel.x, mindepth_pixel.y);
-        printf ("%sMaximum depth was %g at (%d, %d)\n", indent, maxdepth,
-                maxdepth_pixel.x, maxdepth_pixel.y);
+        printf ("%sSamples/pixel histogram:\n", indent);
+        size_t grandtotal = 0;
+        for (size_t i = 0, e = nsamples_histogram.size();  i < e;  ++i)
+            grandtotal += nsamples_histogram[i];
+        size_t binstart = 0, bintotal = 0;
+        for (size_t i = 0, e = nsamples_histogram.size();  i < e;  ++i) {
+            bintotal += nsamples_histogram[i];
+            if (i < 8 || i == (e-1) || OIIO::ispow2(i+1)) {
+                // batch by powers of 2, unless it's a small number
+                if (i == binstart)
+                    printf ("%s  %3lld    ", indent, (long long)i);
+                else
+                    printf ("%s  %3lld-%3lld", indent,
+                            (long long)binstart, (long long)i);
+                printf (" : %8lld (%4.1f%%)\n", (long long)bintotal,
+                        (100.0*bintotal)/grandtotal);
+                binstart = i+1;
+                bintotal = 0;
+            }
+        }
+        if (depthchannel >= 0) {
+            printf ("%sMinimum depth was %g at (%d, %d)\n", indent, mindepth,
+                    mindepth_pixel.x, mindepth_pixel.y);
+            printf ("%sMaximum depth was %g at (%d, %d)\n", indent, maxdepth,
+                    maxdepth_pixel.x, maxdepth_pixel.y);
+        }
     } else {
         std::vector<float> constantValues(input.spec().nchannels);
         if (isConstantColor(input, &constantValues[0])) {
@@ -538,7 +569,8 @@ extended_format_name (TypeDesc type, int bits)
 // prints basic info (resolution, width, height, depth, channels, data format,
 // and format name) about given subimage.
 static void
-print_info_subimage (int current_subimage, int max_subimages, ImageSpec &spec,
+print_info_subimage (Oiiotool &ot,
+                     int current_subimage, int max_subimages, ImageSpec &spec,
                      ImageInput *input, const std::string &filename,
                      const print_info_options &opt,
                      boost::regex &field_re, boost::regex &field_exclude_re)
@@ -612,7 +644,7 @@ print_info_subimage (int current_subimage, int max_subimages, ImageSpec &spec,
                 printf ("    MIP %d of %d (%d x %d):\n",
                         m, nmip, mipspec.width, mipspec.height);
             }
-            print_stats (filename, spec, current_subimage, m, nmip>1);
+            print_stats (ot, filename, spec, current_subimage, m, nmip>1);
         }
     }
 
@@ -623,7 +655,8 @@ print_info_subimage (int current_subimage, int max_subimages, ImageSpec &spec,
 
 
 bool
-OiioTool::print_info (const std::string &filename, 
+OiioTool::print_info (Oiiotool &ot,
+                      const std::string &filename, 
                       const print_info_options &opt,
                       long long &totalsize,
                       std::string &error)
@@ -721,6 +754,7 @@ OiioTool::print_info (const std::string &filename,
         printf ("\n");
     }
 
+    int movie = spec.get_int_attribute ("oiio:Movie");
     if (opt.verbose && num_of_subimages != 1) {
         // info about num of subimages and their resolutions
         printf ("    %d subimages: ", num_of_subimages);
@@ -730,6 +764,8 @@ OiioTool::print_info (const std::string &filename,
                 printf ("%dx%dx%d ", spec.width, spec.height, spec.depth);
             else
                 printf ("%dx%d ", spec.width, spec.height);
+            if (movie)
+                break;
         }
         printf ("\n");
     }
@@ -739,7 +775,7 @@ OiioTool::print_info (const std::string &filename,
     if ( ! opt.subimages)
         num_of_subimages = 1;
     for (int i = 0; i < num_of_subimages; ++i) {
-        print_info_subimage (i, num_of_subimages, spec, input,
+        print_info_subimage (ot, i, num_of_subimages, spec, input,
                              filename, opt, field_re, field_exclude_re);
     }
 
