@@ -51,7 +51,6 @@
 #include <OpenEXR/ImfFloatAttribute.h>
 #include <OpenEXR/ImfMatrixAttribute.h>
 #include <OpenEXR/ImfVecAttribute.h>
-#include <OpenEXR/ImfVecAttribute.h>
 #include <OpenEXR/ImfBoxAttribute.h>
 #include <OpenEXR/ImfStringAttribute.h>
 #include <OpenEXR/ImfTimeCodeAttribute.h>
@@ -69,6 +68,7 @@
 #include <OpenEXR/ImfDeepScanLineInputPart.h>
 #include <OpenEXR/ImfDeepTiledInputPart.h>
 #include <OpenEXR/ImfDeepFrameBuffer.h>
+#include <OpenEXR/ImfDoubleAttribute.h>
 #endif
 
 #ifdef __GNUC__
@@ -83,10 +83,10 @@
 #include "OpenImageIO/strutil.h"
 #include "OpenImageIO/fmath.h"
 #include "OpenImageIO/filesystem.h"
+#include "OpenImageIO/imagebufalgo_util.h"
 
 #include <boost/scoped_array.hpp>
 
-
 OIIO_PLUGIN_NAMESPACE_BEGIN
 
 
@@ -100,12 +100,13 @@ public:
         // The reason we have this class is for this line, so that we
         // can correctly handle UTF-8 file paths on Windows
         Filesystem::open (ifs, filename, std::ios_base::binary);
-        if (!ifs)
+        if (!ifs) 
             Iex::throwErrnoExc ();
     }
     virtual bool read (char c[], int n) {
-        if (!ifs)
+        if (!ifs) 
             throw Iex::InputExc ("Unexpected end of file.");
+		
         errno = 0;
         ifs.read (c, n);
         return check_error ();
@@ -124,13 +125,14 @@ public:
 private:
     bool check_error () {
         if (!ifs) {
-            if (errno)
+            if (errno) 
                 Iex::throwErrnoExc ();
+			
             return false;
         }
         return true;
     }
-    std::ifstream ifs;
+    OIIO::ifstream ifs;
 };
 
 
@@ -140,6 +142,11 @@ public:
     OpenEXRInput ();
     virtual ~OpenEXRInput () { close(); }
     virtual const char * format_name (void) const { return "openexr"; }
+    virtual int supports (string_view feature) const {
+        return (feature == "arbitrary_metadata"
+             || feature == "exif"   // Because of arbitrary_metadata
+             || feature == "iptc"); // Because of arbitrary_metadata
+    }
     virtual bool valid_file (const std::string &filename) const;
     virtual bool open (const std::string &name, ImageSpec &newspec);
     virtual bool close ();
@@ -206,7 +213,6 @@ private:
     int m_subimage;                       ///< What subimage are we looking at?
     int m_nsubimages;                     ///< How many subimages are there?
     int m_miplevel;                       ///< What MIP level are we looking at?
-    std::vector<unsigned char> m_scratch; ///< Scratch space for us to use
 
     void init () {
         m_input_stream = NULL;
@@ -265,6 +271,7 @@ private:
         m_map["comments"] = "ImageDescription";
         m_map["owner"] = "Copyright";
         m_map["pixelAspectRatio"] = "PixelAspectRatio";
+        m_map["xDensity"] = "XResolution";
         m_map["expTime"] = "ExposureTime";
         // Ones we don't rename -- OpenEXR convention matches ours
         m_map["wrapmodes"] = "wrapmodes";
@@ -292,7 +299,6 @@ private:
         // screenWindowCenter
         // chromaticities whiteLuminance adoptedNeutral
         // renderingTransform, lookModTransform
-        // xDensity
         // utcOffset
         // longitude latitude altitude
         // focus isoSpeed
@@ -300,7 +306,7 @@ private:
     }
 };
 
-static StringMap exr_tag_to_ooio_std;
+static StringMap exr_tag_to_oiio_std;
 
 
 namespace pvt {
@@ -311,12 +317,12 @@ void set_exr_threads ()
     static spin_mutex exr_threads_mutex;  
 
     int oiio_threads = 1;
-    OIIO::getattribute ("threads", oiio_threads);
+    OIIO::getattribute ("exr_threads", oiio_threads);
 
     spin_lock lock (exr_threads_mutex);
     if (exr_threads != oiio_threads) {
         exr_threads = oiio_threads;
-        Imf::setGlobalThreadCount (exr_threads == 1 ? 0 : exr_threads);
+        Imf::setGlobalThreadCount (exr_threads);
     }
 }
 
@@ -359,22 +365,28 @@ OpenEXRInput::open (const std::string &name, ImageSpec &newspec)
     
     try {
         m_input_stream = new OpenEXRInputStream (name.c_str());
-    }
-    catch (const std::exception &e) {
+    } catch (const std::exception &e) {
         m_input_stream = NULL;
         error ("OpenEXR exception: %s", e.what());
         return false;
+    } catch (...) {   // catch-all for edge cases or compiler bugs
+        m_input_stream = NULL;
+        error ("OpenEXR exception: unknown");
+        return false;
     }
 
 #ifdef USE_OPENEXR_VERSION2
     try {
         m_input_multipart = new Imf::MultiPartInputFile (*m_input_stream);
-    }
-    catch (const std::exception &e) {
+    } catch (const std::exception &e) {
         delete m_input_stream;
         m_input_stream = NULL;
         error ("OpenEXR exception: %s", e.what());
         return false;
+    } catch (...) {   // catch-all for edge cases or compiler bugs
+        m_input_stream = NULL;
+        error ("OpenEXR exception: unknown");
+        return false;
     }
 
     m_nsubimages = m_input_multipart->parts();
@@ -386,12 +398,15 @@ OpenEXRInput::open (const std::string &name, ImageSpec &newspec)
         } else {
             m_input_scanline = new Imf::InputFile (*m_input_stream);
         }
-    }
-    catch (const std::exception &e) {
+    } catch (const std::exception &e) {
         delete m_input_stream;
         m_input_stream = NULL;
         error ("OpenEXR exception: %s", e.what());
         return false;
+    } catch (...) {   // catch-all for edge cases or compiler bugs
+        m_input_stream = NULL;
+        error ("OpenEXR exception: unknown");
+        return false;
     }
 
     if (! m_input_scanline && ! m_input_tiled) {
@@ -539,7 +554,8 @@ OpenEXRInput::PartInfo::parse_header (const Imf::Header *header)
         const Imf::IntAttribute *iattr;
         const Imf::FloatAttribute *fattr;
         const Imf::StringAttribute *sattr;
-        const Imf::M44fAttribute *mattr;
+        const Imf::M33fAttribute *m33fattr;
+        const Imf::M44fAttribute *m44fattr;
         const Imf::V3fAttribute *v3fattr;
         const Imf::V3iAttribute *v3iattr;
         const Imf::V2fAttribute *v2fattr;
@@ -550,27 +566,35 @@ OpenEXRInput::PartInfo::parse_header (const Imf::Header *header)
         const Imf::KeyCodeAttribute *kcattr;
 #ifdef USE_OPENEXR_VERSION2
         const Imf::StringVectorAttribute *svattr;
+        const Imf::DoubleAttribute *dattr;
+        const Imf::V2dAttribute *v2dattr;
+        const Imf::V3dAttribute *v3dattr;
+        const Imf::M33dAttribute *m33dattr;
+        const Imf::M44dAttribute *m44dattr;
 #endif
         const char *name = hit.name();
-        std::string oname = exr_tag_to_ooio_std[name];
+        std::string oname = exr_tag_to_oiio_std[name];
         if (oname.empty())   // Empty string means skip this attrib
             continue;
 //        if (oname == name)
 //            oname = std::string(format_name()) + "_" + oname;
         const Imf::Attribute &attrib = hit.attribute();
         std::string type = attrib.typeName();
-        if (type == "string" && 
+        if (type == "string" &&
             (sattr = header->findTypedAttribute<Imf::StringAttribute> (name)))
             spec.attribute (oname, sattr->value().c_str());
-        else if (type == "int" && 
+        else if (type == "int" &&
             (iattr = header->findTypedAttribute<Imf::IntAttribute> (name)))
             spec.attribute (oname, iattr->value());
-        else if (type == "float" && 
+        else if (type == "float" &&
             (fattr = header->findTypedAttribute<Imf::FloatAttribute> (name)))
             spec.attribute (oname, fattr->value());
-        else if (type == "m44f" && 
-            (mattr = header->findTypedAttribute<Imf::M44fAttribute> (name)))
-            spec.attribute (oname, TypeDesc::TypeMatrix, &(mattr->value()));
+        else if (type == "m33f" &&
+            (m33fattr = header->findTypedAttribute<Imf::M33fAttribute> (name)))
+            spec.attribute (oname, TypeDesc::TypeMatrix33, &(m33fattr->value()));
+        else if (type == "m44f" &&
+            (m44fattr = header->findTypedAttribute<Imf::M44fAttribute> (name)))
+            spec.attribute (oname, TypeDesc::TypeMatrix44, &(m44fattr->value()));
         else if (type == "v3f" &&
                  (v3fattr = header->findTypedAttribute<Imf::V3fAttribute> (name)))
             spec.attribute (oname, TypeDesc::TypeVector, &(v3fattr->value()));
@@ -599,6 +623,31 @@ OpenEXRInput::PartInfo::parse_header (const Imf::Header *header)
             TypeDesc sv (TypeDesc::STRING, ustrvec.size());
             spec.attribute(oname, sv, &ustrvec[0]);
         }
+        else if (type == "double" &&
+            (dattr = header->findTypedAttribute<Imf::DoubleAttribute> (name))) {
+            TypeDesc d (TypeDesc::DOUBLE);
+            spec.attribute (oname, d, &(dattr->value()));
+        }
+        else if (type == "v2d" &&
+                 (v2dattr = header->findTypedAttribute<Imf::V2dAttribute> (name))) {
+            TypeDesc v2 (TypeDesc::DOUBLE,TypeDesc::VEC2);
+            spec.attribute (oname, v2, &(v2dattr->value()));
+        }
+        else if (type == "v3d" &&
+                 (v3dattr = header->findTypedAttribute<Imf::V3dAttribute> (name))) {
+            TypeDesc v3 (TypeDesc::DOUBLE,TypeDesc::VEC3, TypeDesc::VECTOR);
+            spec.attribute (oname, v3, &(v3dattr->value()));
+        }
+        else if (type == "m33d" &&
+            (m33dattr = header->findTypedAttribute<Imf::M33dAttribute> (name))) {
+            TypeDesc m33 (TypeDesc::DOUBLE, TypeDesc::MATRIX33);
+            spec.attribute (oname, m33, &(m33dattr->value()));
+        }
+        else if (type == "m44d" &&
+            (m44dattr = header->findTypedAttribute<Imf::M44dAttribute> (name))) {
+            TypeDesc m44 (TypeDesc::DOUBLE, TypeDesc::MATRIX44);
+            spec.attribute (oname, m44, &(m44dattr->value()));
+        }
 #endif
         else if (type == "box2i" &&
                  (b2iattr = header->findTypedAttribute<Imf::Box2iAttribute> (name))) {
@@ -645,6 +694,14 @@ OpenEXRInput::PartInfo::parse_header (const Imf::Header *header)
         }
     }
 
+    float aspect = spec.get_float_attribute ("PixelAspectRatio", 0.0f);
+    float xdensity = spec.get_float_attribute ("XResolution", 0.0f);
+    if (xdensity) {
+        // If XResolution is found, supply the YResolution and unit.
+        spec.attribute ("YResolution", xdensity * (aspect ? aspect : 1.0f));
+        spec.attribute ("ResolutionUnit", "in"); // EXR is always pixels/inch
+    }
+
 #ifdef USE_OPENEXR_VERSION2
     // EXR "name" also gets passed along as "oiio:subimagename".
     if (header->hasName())
@@ -658,16 +715,33 @@ OpenEXRInput::PartInfo::parse_header (const Imf::Header *header)
 
 namespace {
 
+
+static TypeDesc
+TypeDesc_from_ImfPixelType (Imf::PixelType ptype)
+{
+    switch (ptype) {
+    case Imf::UINT  : return TypeDesc::UINT;  break;
+    case Imf::HALF  : return TypeDesc::HALF;  break;
+    case Imf::FLOAT : return TypeDesc::FLOAT; break;
+    default: ASSERT (0);
+    }
+}
+
+
+
 // Used to hold channel information for sorting into canonical order
 struct ChanNameHolder {
     string_view fullname;
-    int channel_number;
+    int exr_channel_number;  // channel index in the exr (sorted by name)
     string_view layer;
     string_view suffix;
     int special_index;
+    Imf::PixelType exr_data_type;
+    TypeDesc datatype;
 
-    ChanNameHolder (string_view fullname, int n)
-        : fullname(fullname), channel_number(n)
+    ChanNameHolder (string_view fullname, int n, Imf::PixelType exrtype)
+        : fullname(fullname), exr_channel_number(n), exr_data_type(exrtype),
+          datatype(TypeDesc_from_ImfPixelType(exrtype))
     {
         size_t dot = fullname.find_last_of ('.');
         if (dot == string_view::npos) {
@@ -680,7 +754,7 @@ struct ChanNameHolder {
             "R", "Red", "G", "Green", "B", "Blue", "real", "imag",
             "A", "Alpha", "RA", "RG", "RB", "Z", "Depth", "Zback", NULL
         };
-        special_index = 1000;
+        special_index = 10000;
         for (int i = 0; special[i]; ++i)
             if (Strutil::iequals (suffix, special[i])) {
                 special_index = i;
@@ -718,14 +792,22 @@ OpenEXRInput::PartInfo::query_channels (const Imf::Header *header)
     int c = 0;
     for (Imf::ChannelList::ConstIterator ci = channels.begin();
          ci != channels.end();  ++c, ++ci) {
-        cnh.push_back (ChanNameHolder (ci.name(), c));
+        cnh.push_back (ChanNameHolder (ci.name(), c, ci.channel().type));
         ++spec.nchannels;
     }
     std::sort (cnh.begin(), cnh.end(), ChanNameHolder::compare_cnh);
-    c = 0;
-    for (Imf::ChannelList::ConstIterator ci = channels.begin();
-         ci != channels.end();  ++c, ++ci) {
+    // Now we should have cnh sorted into the order that we want to present
+    // to the OIIO client.
+    spec.format = TypeDesc::UNKNOWN;
+    bool all_one_format = true;
+    for (int c = 0; c < spec.nchannels; ++c) {
         spec.channelnames.push_back (cnh[c].fullname);
+        spec.channelformats.push_back (cnh[c].datatype);
+        spec.format = TypeDesc(ImageBufAlgo::type_merge (TypeDesc::BASETYPE(spec.format.basetype),
+                                                         TypeDesc::BASETYPE(cnh[c].datatype.basetype)));
+        pixeltype.push_back (cnh[c].exr_data_type);
+        chanbytes.push_back (cnh[c].datatype.size());
+        all_one_format &= (cnh[c].datatype == cnh[0].datatype);
         if (spec.alpha_channel < 0 && (Strutil::iequals (cnh[c].suffix, "A") ||
                                        Strutil::iequals (cnh[c].suffix, "Alpha")))
             spec.alpha_channel = c;
@@ -734,50 +816,9 @@ OpenEXRInput::PartInfo::query_channels (const Imf::Header *header)
             spec.z_channel = c;
     }
     ASSERT ((int)spec.channelnames.size() == spec.nchannels);
-
-    // Figure out data types -- choose the highest range
-    spec.format = TypeDesc::UNKNOWN;
-    std::vector<TypeDesc> chanformat;
-    c = 0;
-    for (Imf::ChannelList::ConstIterator ci = channels.begin();
-         ci != channels.end();  ++c, ++ci) {
-        Imf::PixelType ptype = ci.channel().type;
-        TypeDesc fmt = TypeDesc::HALF;
-        switch (ptype) {
-        case Imf::UINT :
-            fmt = TypeDesc::UINT;
-            if (spec.format == TypeDesc::UNKNOWN)
-                spec.format = TypeDesc::UINT;
-            break;
-        case Imf::HALF :
-            fmt = TypeDesc::HALF;
-            if (spec.format != TypeDesc::FLOAT)
-                spec.format = TypeDesc::HALF;
-            break;
-        case Imf::FLOAT :
-            fmt = TypeDesc::FLOAT;
-            spec.format = TypeDesc::FLOAT;
-            break;
-        default: ASSERT (0);
-        }
-        pixeltype.push_back (ptype);
-        chanbytes.push_back (fmt.size());
-        if (chanformat.size() == 0)
-            chanformat.resize (spec.nchannels, fmt);
-        for (int i = 0;  i < spec.nchannels;  ++i) {
-            ASSERT ((int)spec.channelnames.size() > i);
-            if (spec.channelnames[i] == ci.name()) {
-                chanformat[i] = fmt;
-                break;
-            }
-        }
-    }
     ASSERT (spec.format != TypeDesc::UNKNOWN);
-    bool differing_chanformats = false;
-    for (int c = 1;  c < spec.nchannels;  ++c)
-        differing_chanformats |= (chanformat[c] != chanformat[0]);
-    if (differing_chanformats)
-        spec.channelformats = chanformat;
+    if (all_one_format)
+        spec.channelformats.clear();
 }
 
 
@@ -827,8 +868,7 @@ OpenEXRInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
                 else
                     m_scanline_input_part = new Imf::InputPart (*m_input_multipart, subimage);
             }
-        }
-        catch (const std::exception &e) {
+        } catch (const std::exception &e) {
             error ("OpenEXR exception: %s", e.what());
             m_scanline_input_part = NULL;
             m_tiled_input_part = NULL;
@@ -836,6 +876,14 @@ OpenEXRInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
             m_deep_tiled_input_part = NULL;
             ASSERT(0);
             return false;
+        } catch (...) {   // catch-all for edge cases or compiler bugs
+            error ("OpenEXR exception: unknown");
+            m_scanline_input_part = NULL;
+            m_tiled_input_part = NULL;
+            m_deep_scanline_input_part = NULL;
+            m_deep_tiled_input_part = NULL;
+            ASSERT(0);
+            return false;
         }
     }
 #endif
@@ -982,10 +1030,12 @@ OpenEXRInput::read_native_scanlines (int ybegin, int yend, int z,
         } else {
             ASSERT (0);
         }
-    }
-    catch (const std::exception &e) {
+    } catch (const std::exception &e) {
         error ("Failed OpenEXR read: %s", e.what());
         return false;
+    } catch (...) {   // catch-all for edge cases or compiler bugs
+        error ("Failed OpenEXR read: unknown exception");
+        return false;
     }
     return true;
 }
@@ -1094,10 +1144,12 @@ OpenEXRInput::read_native_tiles (int xbegin, int xend, int ybegin, int yend,
                         (char *)data+(y-ybegin)*scanline_stride,
                         user_scanline_bytes);
         }
-    }
-    catch (const std::exception &e) {
+    } catch (const std::exception &e) {
         error ("Failed OpenEXR read: %s", e.what());
         return false;
+    } catch (...) {   // catch-all for edge cases or compiler bugs
+        error ("Failed OpenEXR read: unknown exception");
+        return false;
     }
 
     return true;
@@ -1154,10 +1206,12 @@ OpenEXRInput::read_native_deep_scanlines (int ybegin, int yend, int z,
 
         // Read the pixels
         m_deep_scanline_input_part->readPixels (ybegin, yend-1);
-    }
-    catch (const std::exception &e) {
+    } catch (const std::exception &e) {
         error ("Failed OpenEXR read: %s", e.what());
         return false;
+    } catch (...) {   // catch-all for edge cases or compiler bugs
+        error ("Failed OpenEXR read: unknown exception");
+        return false;
     }
 
     return true;
@@ -1217,18 +1271,27 @@ OpenEXRInput::read_native_deep_tiles (int xbegin, int xend,
         int xtiles = round_to_multiple (xend-xbegin, m_spec.tile_width) / m_spec.tile_width;
         int ytiles = round_to_multiple (yend-ybegin, m_spec.tile_height) / m_spec.tile_height;
 
+        int firstxtile = (xbegin - m_spec.x) / m_spec.tile_width;
+        int firstytile = (ybegin - m_spec.y) / m_spec.tile_height;
+
         // Get the sample counts for each pixel and compute the total
         // number of samples and resize the data area appropriately.
-        m_deep_tiled_input_part->readPixelSampleCounts (0, xtiles-1, 0, ytiles-1);
+        m_deep_tiled_input_part->readPixelSampleCounts (
+                firstxtile, firstxtile+xtiles-1,
+                firstytile, firstytile+ytiles-1);
         deepdata.alloc ();
 
         // Read the pixels
-        m_deep_tiled_input_part->readTiles (0, xtiles-1, 0, ytiles-1,
-                                            m_miplevel, m_miplevel);
-    }
-    catch (const std::exception &e) {
+        m_deep_tiled_input_part->readTiles (
+                firstxtile, firstxtile+xtiles-1,
+                firstytile, firstytile+ytiles-1,
+                m_miplevel, m_miplevel);
+    } catch (const std::exception &e) {
         error ("Failed OpenEXR read: %s", e.what());
         return false;
+    } catch (...) {   // catch-all for edge cases or compiler bugs
+        error ("Failed OpenEXR read: unknown exception");
+        return false;
     }
 
     return true;
