@@ -53,14 +53,15 @@ private:
       P1, P2, P3, P4, P5, P6, Pf, PF
     };
 
-    std::ifstream m_file;
+    OIIO::ifstream m_file;
+    std::streampos m_header_end_pos; // file position after the header
     std::string m_current_line; ///< Buffer the image pixels
     const char * m_pos;
     PNMType m_pnm_type;
     unsigned int m_max_val;
     float m_scaling_factor;
 
-    bool read_file_scanline (void * data);
+    bool read_file_scanline (void * data, int y);
     bool read_file_header ();
 };
 
@@ -81,7 +82,7 @@ OIIO_PLUGIN_EXPORTS_END
 
 
 inline bool
-nextLine (std::ifstream &file, std::string &current_line, const char * &pos) 
+nextLine (std::istream &file, std::string &current_line, const char * &pos)
 {   
     if (!file.good())
         return false;
@@ -95,7 +96,7 @@ nextLine (std::ifstream &file, std::string &current_line, const char * &pos)
 
 
 inline const char * 
-nextToken (std::ifstream &file, std::string &current_line, const char * &pos)
+nextToken (std::istream &file, std::string &current_line, const char * &pos)
 {		
     while (1) {
         while (isspace (*pos)) 
@@ -111,7 +112,7 @@ nextToken (std::ifstream &file, std::string &current_line, const char * &pos)
 
 
 inline const char *
-skipComments (std::ifstream &file, std::string &current_line, 
+skipComments (std::istream &file, std::string &current_line,
               const char * & pos, char comment = '#')
 {		
     while (1) {
@@ -127,7 +128,7 @@ skipComments (std::ifstream &file, std::string &current_line,
 
 
 inline bool
-nextVal (std::ifstream & file, std::string &current_line,
+nextVal (std::istream & file, std::string &current_line,
          const char * &pos, int &val, char comment = '#')
 {
     skipComments (file, current_line, pos, comment);
@@ -151,7 +152,7 @@ invert (const T *read, T *write, imagesize_t nvals)
 
 template <class T> 
 inline bool 
-ascii_to_raw (std::ifstream &file, std::string &current_line, const char * &pos,
+ascii_to_raw (std::istream &file, std::string &current_line, const char * &pos,
               T *write, imagesize_t nvals, T max)
 {
     if (max)
@@ -241,16 +242,23 @@ read_int (std::istream &in, T &dest, char comment='#')
 
 
 bool 
-PNMInput::read_file_scanline (void * data)
+PNMInput::read_file_scanline (void * data, int y)
 {
     try {
 
     std::vector<unsigned char> buf;
     bool good = true;
-    if (!m_file.is_open())
+    if (!m_file)
         return false;
     int nsamples = m_spec.width * m_spec.nchannels;
 
+    // PFM files are bottom-to-top, so we need to seek to the right spot
+    if (m_pnm_type == PF || m_pnm_type == Pf) {
+        int file_scanline = m_spec.height - 1 - (y-m_spec.y);
+        std::streampos offset = file_scanline * m_spec.scanline_bytes();
+        m_file.seekg (m_header_end_pos + offset, std::ios_base::beg);
+    }
+
     if ((m_pnm_type >= P4 && m_pnm_type <= P6) || m_pnm_type == PF || m_pnm_type == Pf){
         int numbytes;
         if (m_pnm_type == P4)
@@ -268,17 +276,17 @@ PNMInput::read_file_scanline (void * data)
     switch (m_pnm_type) {
         //Ascii 
         case P1:
-            good &= ascii_to_raw (m_file, m_current_line, m_pos, (unsigned char *) data, 
+            good &= ascii_to_raw (m_file, m_current_line, m_pos, (unsigned char *) data,
                                   nsamples, (unsigned char)m_max_val);
             invert ((unsigned char *)data, (unsigned char *)data, nsamples); 
             break;
         case P2:
         case P3:
             if (m_max_val > std::numeric_limits<unsigned char>::max())
-                good &= ascii_to_raw (m_file, m_current_line, m_pos, (unsigned short *) data, 
+                good &= ascii_to_raw (m_file, m_current_line, m_pos, (unsigned short *) data,
                                       nsamples, (unsigned short)m_max_val);
             else 
-                good &= ascii_to_raw (m_file, m_current_line, m_pos, (unsigned char *) data, 
+                good &= ascii_to_raw (m_file, m_current_line, m_pos, (unsigned char *) data,
                                       nsamples, (unsigned char)m_max_val);
             break;
         //Raw
@@ -287,12 +295,15 @@ PNMInput::read_file_scanline (void * data)
             break;
         case P5:
         case P6:
-            if (m_max_val > std::numeric_limits<unsigned char>::max())
+            if (m_max_val > std::numeric_limits<unsigned char>::max()) {
+                if (littleendian())
+                    swap_endian ((unsigned short *)&buf[0], nsamples);
                 raw_to_raw ((unsigned short *)&buf[0], (unsigned short *) data, 
                             nsamples, (unsigned short)m_max_val);
-            else 
+            } else {
                 raw_to_raw ((unsigned char *)&buf[0], (unsigned char *) data, 
                             nsamples, (unsigned char)m_max_val);
+            }
             break;
         //Floating point
         case Pf:
@@ -320,7 +331,7 @@ PNMInput::read_file_header ()
 
     unsigned int width, height;
     char c;
-    if (!m_file.is_open())
+    if (!m_file)
         return false;
   
     //MagicNumber
@@ -375,6 +386,7 @@ PNMInput::read_file_header ()
         //Space before content
         if (!(isspace (m_file.get()) && m_file.good()))
             return false;
+        m_header_end_pos = m_file.tellg ();   // remember file pos
 
         if (m_pnm_type == P3 || m_pnm_type == P6)
             m_spec =  ImageSpec (width, height, 3, 
@@ -404,6 +416,7 @@ PNMInput::read_file_header ()
         //Space before content
         if (!(isspace (m_file.get()) && m_file.good()))
             return false;
+        m_header_end_pos = m_file.tellg ();   // remember file pos
 
         if(m_pnm_type == PF) {
             m_spec = ImageSpec(width, height, 3, TypeDesc::FLOAT);
@@ -434,9 +447,9 @@ bool
 PNMInput::open (const std::string &name, ImageSpec &newspec)
 {
     close(); //close previously opened file
-
+    
     Filesystem::open (m_file, name, std::ios::in|std::ios::binary);
-
+ 
     m_current_line = "";
     m_pos = m_current_line.c_str();
 
@@ -452,8 +465,7 @@ PNMInput::open (const std::string &name, ImageSpec &newspec)
 bool
 PNMInput::close ()
 {
-    if (m_file.is_open())
-        m_file.close();
+    m_file.close();
     return true;
 }
 
@@ -464,7 +476,7 @@ PNMInput::read_native_scanline (int y, int z, void *data)
 {
     if (z)
         return false;
-    if (!read_file_scanline (data))
+    if (!read_file_scanline (data, y))
         return false;
     return true;
 }
