@@ -40,9 +40,8 @@ public:
     PSDOutput ();
     virtual ~PSDOutput ();
     virtual const char * format_name (void) const { return "psd"; }
-    virtual bool supports (const std::string &feature) const {
-        // Support nothing nonstandard
-        return false;
+    virtual int supports (string_view feature) const {
+        return (feature == "alpha");
     }
     virtual bool open (const std::string &name, const ImageSpec &spec,
                        OpenMode mode=Create);
