@@ -39,7 +39,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 #include <errno.h>
 #include <stdio.h>
 
-#include "filesystem.h"
+#include "OpenImageIO/filesystem.h"
 
 #include "Ptexture.h"
 #include "PtexUtils.h"
