@@ -70,7 +70,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 #include <unistd.h>
 #endif
 
-#include "filesystem.h"
+#include "OpenImageIO/filesystem.h"
 
 #include "Ptexture.h"
 #include "PtexUtils.h"
