@@ -33,11 +33,11 @@
 #include <cmath>
 #include <cassert>
 
-#include "dassert.h"
-#include "typedesc.h"
-#include "imageio.h"
-#include "filesystem.h"
-#include "fmath.h"
+#include "OpenImageIO/dassert.h"
+#include "OpenImageIO/typedesc.h"
+#include "OpenImageIO/imageio.h"
+#include "OpenImageIO/filesystem.h"
+#include "OpenImageIO/fmath.h"
 
 #include "rla_pvt.h"
 
@@ -288,12 +288,12 @@ RLAInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
     
     // set window dimensions etc.
     m_spec.x = m_rla.ActiveLeft;
-    m_spec.y = m_spec.height - m_rla.ActiveTop - 1;
+    m_spec.y = m_spec.height-1 - m_rla.ActiveTop;
     m_spec.full_width = m_rla.WindowRight - m_rla.WindowLeft + 1;
     m_spec.full_height = m_rla.WindowTop - m_rla.WindowBottom + 1;
     m_spec.full_depth = 1;
     m_spec.full_x = m_rla.WindowLeft;
-    m_spec.full_y = m_spec.full_height - m_rla.WindowTop - 1;
+    m_spec.full_y = m_spec.full_height-1 - m_rla.WindowTop;
 
     // set channel formats and stride
     int z_channel = -1;
@@ -449,7 +449,7 @@ RLAInput::decode_rle_span (unsigned char *buf, int n, int stride,
 {
     size_t e = 0;
     while (n > 0 && e < elen) {
-        char count = encoded[e++];
+        signed char count = (signed char) encoded[e++];
         if (count >= 0) {
             // run count positive: value repeated count+1 times
             for (int i = 0;  i <= count && n;  ++i, buf += stride, --n)
@@ -600,7 +600,7 @@ bool
 RLAInput::read_native_scanline (int y, int z, void *data)
 {
     // By convention, RLA images store their images bottom-to-top.
-    y = m_spec.height - y - 1;
+    y = m_spec.height - (y - m_spec.y) - 1;
 
     // Seek to scanline start, based on the scanline offset table
     fseek (m_file, m_sot[y], SEEK_SET);
