@@ -141,7 +141,7 @@ class SgiOutput : public ImageOutput {
     SgiOutput () : m_fd(NULL) { }
     virtual ~SgiOutput () { close(); }
     virtual const char *format_name (void) const { return "sgi"; }
-    virtual bool supports (const std::string &feature) const { return false; }
+    virtual int supports (string_view feature) const;
     virtual bool open (const std::string &name, const ImageSpec &spec,
                        OpenMode mode=Create);
     virtual bool close (void);
