@@ -41,8 +41,6 @@
 #include <OpenEXR/ImathVec.h>
 #include <OpenEXR/half.h>
 
-#include <boost/bind.hpp>
-
 #include "OpenImageIO/argparse.h"
 #include "OpenImageIO/imageio.h"
 #include "OpenImageIO/ustring.h"
@@ -59,6 +57,10 @@
 
 OIIO_NAMESPACE_USING
 
+#if OIIO_CPLUSPLUS_VERSION >= 11
+using OIIO::_1;
+#endif
+
 static std::vector<ustring> filenames;
 static std::string output_filename = "out.exr";
 static bool verbose = false;
@@ -318,12 +320,11 @@ adjust_spec (ImageSpec &outspec, const std::string &dataformatname)
 
 
 
-inline Imath::V3f
+inline Imath::V2f
 warp (float x, float y, const Imath::M33f &xform)
 {
-    Imath::V3f coord (x, y, 1.0f);
-    coord *= xform;
-    coord[0] *= 1/(1+2*std::max (-0.5f, coord[1]));
+    Imath::V2f coord (x, y);
+    xform.multVecMatrix (coord, coord);
     return coord;
 }
 
@@ -333,7 +334,6 @@ warp (float x, float y, float z, const Imath::M33f &xform)
 {
     Imath::V3f coord (x, y, z);
     coord *= xform;
-    coord[0] *= 1/(1+2*std::max (-0.5f, coord[1]));
     return coord;
 }
 
@@ -341,11 +341,11 @@ warp (float x, float y, float z, const Imath::M33f &xform)
 inline Imath::V2f
 warp_coord (float x, float y)
 {
-    Imath::V3f coord = warp (x/output_xres, y/output_yres, xform);
+    Imath::V2f coord = warp (x/output_xres, y/output_yres, xform);
     coord.x *= sscale;
     coord.y *= tscale;
-    coord += offset;
-    return Imath::V2f (coord.x, coord.y);
+    coord += Imath::V2f(offset.x, offset.y);
+    return coord;
 }
 
 
@@ -569,7 +569,7 @@ test_plain_texture (Mapping2D mapping)
             std::cout << "iter " << iter << " file " << filename << "\n";
         }
 
-        ImageBufAlgo::parallel_image (boost::bind(plain_tex_region, boost::ref(image), filename, mapping,
+        ImageBufAlgo::parallel_image (OIIO::bind(plain_tex_region, OIIO::ref(image), filename, mapping,
                                                   test_derivs ? &image_ds : NULL,
                                                   test_derivs ? &image_dt : NULL, _1),
                                       get_roi(image.spec()), nthreads);
@@ -651,7 +651,7 @@ test_texture3d (ustring filename, Mapping3D mapping)
         if (iter && filenames.size() > 1)
             filename = filenames[1];
 
-        ImageBufAlgo::parallel_image (boost::bind(tex3d_region, boost::ref(image), filename, mapping, _1),
+        ImageBufAlgo::parallel_image (OIIO::bind(tex3d_region, OIIO::ref(image), filename, mapping, _1),
                                       get_roi(image.spec()), nthreads);
     }
     
@@ -954,9 +954,9 @@ void
 launch_tex_threads (int numthreads, int iterations)
 {
     texsys->invalidate_all (true);
-    boost::thread_group threads;
+    OIIO::thread_group threads;
     for (int i = 0;  i < numthreads;  ++i) {
-        threads.create_thread (boost::bind(do_tex_thread_workout,iterations,i));
+        threads.create_thread (OIIO::bind(do_tex_thread_workout,iterations,i));
     }
     ASSERT ((int)threads.size() == numthreads);
     threads.join_all ();
@@ -1131,10 +1131,13 @@ main (int argc, const char *argv[])
         test_hash ();
     }
 
-    Imath::M33f scale;  scale.scale (Imath::V2f (0.5, 0.5));
-    Imath::M33f rot;    rot.rotate (radians(30.0f));
-    Imath::M33f trans;  trans.translate (Imath::V2f (0.35f, 0.15f));
-    xform = scale * rot * trans;
+    Imath::M33f scale;  scale.scale (Imath::V2f (0.3, 0.3));
+    Imath::M33f rot;    rot.rotate (radians(25.0f));
+    Imath::M33f trans;  trans.translate (Imath::V2f (0.75f, 0.25f));
+    Imath::M33f persp (2, 0, 0,
+                       0, 0.8, -0.55,
+                       0, 0, 1);
+    xform = persp * rot * trans * scale;
     xform.invert();
 
     if (threadtimes) {
@@ -1144,20 +1147,20 @@ main (int argc, const char *argv[])
         const int iterations = iters>1 ? iters : 2000000;
         std::cout << "Workload: " << workload_names[threadtimes] << "\n";
         std::cout << "texture cache size = " << cachesize << " MB\n";
-        std::cout << "hw threads = " << boost::thread::hardware_concurrency() << "\n";
+        std::cout << "hw threads = " << Sysutil::hardware_concurrency() << "\n";
         std::cout << "times are best of " << ntrials << " trials\n\n";
         std::cout << "threads  time (s) efficiency\n";
         std::cout << "-------- -------- ----------\n";
 
         if (nthreads == 0)
-            nthreads = boost::thread::hardware_concurrency();
+            nthreads = Sysutil::hardware_concurrency();
         static int threadcounts[] = { 1, 2, 4, 8, 12, 16, 24, 32, 64, 128, 1024, 1<<30 };
         float single_thread_time = 0.0f;
         for (int i = 0; threadcounts[i] <= nthreads; ++i) {
             int nt = wedge ? threadcounts[i] : nthreads;
             int its = iters>1 ? (std::max (1, iters/nt)) : iterations; // / nt;
             double range;
-            double t = time_trial (boost::bind(launch_tex_threads,nt,its),
+            double t = time_trial (OIIO::bind(launch_tex_threads,nt,its),
                                    ntrials, &range);
             if (nt == 1)
                 single_thread_time = (float)t;
