@@ -1,4 +1,4 @@
-#!/usr/bin/python2
+#!/usr/bin/env python
 
 imagedir = parent + "/oiio-images"
 files = ["gif_animation.gif", "gif_oiio_logo_with_alpha.gif",
@@ -8,3 +8,7 @@ files = ["gif_animation.gif", "gif_oiio_logo_with_alpha.gif",
          "gif_test_loop_count.gif"]
 for f in files:
     command += info_command (imagedir + "/" + f)
+
+# Test write / conversion to GIF
+command += oiiotool ("../oiiotool/tahoe-tiny.tif -o tahoe-tiny.gif")
+command += info_command ("tahoe-tiny.gif")
