@@ -1,9 +1,7 @@
-#!/usr/bin/python 
+#!/usr/bin/env python 
 
-command += (oiio_app ("oiiotool") + 
-                     " bad.exr --fixnan black -o black.exr >> out.txt ;\n")
-command += (oiio_app ("oiiotool") + 
-                     " bad.exr --fixnan box3 -o box3.exr >> out.txt ;\n")
+command += oiiotool ("bad.exr --fixnan black -o black.exr")
+command += oiiotool ("bad.exr --fixnan box3 -o box3.exr")
 command += info_command ("bad.exr", "--stats", safematch=True)
 command += info_command ("black.exr", "--stats", safematch=True)
 command += info_command ("box3.exr", "--stats", safematch=True)
