@@ -6,5 +6,7 @@ imagedir = parent + "oiio-images"
 command += "python test_imagebuf.py > out.txt ;"
 
 # compare the outputs
-outputs = [ "out.tif", "out.txt" ]
+outputs = [ "out.tif", "outtuple.tif",
+            "outarray.tif", "outarrayB.tif", "outarrayH.tif",
+            "out.txt" ]
 
