@@ -3,12 +3,12 @@
 import os.path
 
 imagedir = parent + "oiio-images"
-refdir2 = "../oiiotool/ref/"
-refdir3 = "../oiiotool-composite/ref/"
-refdir4 = "../oiiotool-fixnan/ref/"
-refdir5 = "../oiiotool-deep/ref/"
-failthresh = 0.004
-failpercent = 0.2
+refdir2 = "../../../../testsuite/oiiotool/ref/"
+refdir3 = "../../../../testsuite/oiiotool-composite/ref/"
+refdir4 = "../../../../testsuite/oiiotool-fixnan/ref/"
+refdir5 = "../../../../testsuite/oiiotool-deep/ref/"
+refdir6 = "../../../../testsuite/oiiotool-pattern/ref/"
+refdir7 = "../../../../testsuite/oiiotool-text/ref/"
 
 
 def checkref (name) :
@@ -20,6 +20,10 @@ def checkref (name) :
         return diff_command (name, refdir4+name)
     elif os.path.isfile(refdir5+name) :
         return diff_command (name, refdir5+name)
+    elif os.path.isfile(refdir6+name) :
+        return diff_command (name, refdir6+name)
+    elif os.path.isfile(refdir7+name) :
+        return diff_command (name, refdir7+name)
     else :
         return diff_command (name, refdir+name)
 
@@ -29,8 +33,10 @@ command += "python test_imagebufalgo.py > out.txt ;"
 
 # Checkout outputs -- some of the refs are in the oiiotool test dir
 for f in [ "black.tif", "filled.tif", "checker.tif",
+           "noise-uniform3.tif", "noise-gauss.tif", "noise-salt.tif",
            "chanshuffle.tif", "ch-rgba.exr", "ch-z.exr",
-           "chappend-rgbaz.exr", "flat.exr",
+           "chappend-rgbaz.exr",
+           "flat.exr", "deepen.exr",
            "crop.tif", "cut.tif", "pasted.tif",
            "rotate90.tif", "rotate180.tif", "rotate270.tif",
            "rotated.tif", "rotated-offcenter.tif",
@@ -38,7 +44,12 @@ for f in [ "black.tif", "filled.tif", "checker.tif",
            "flip.tif", "flop.tif", "flipflop.tif", "reorient1.tif",
            "transpose.tif",
            "cshift.tif", "cadd1.exr", "cadd2.exr", "add.exr", "sub.exr",
-           "cmul1.exr", "cmul2.exr", "cpow1.exr", "cpow2.exr",
+           "abs.exr", "absdiff.exr",
+           "mul.exr", "cmul1.exr", "cmul2.exr",
+           "mad.exr",
+           "cpow1.exr", "cpow2.exr",
+           "div.exr", "divc1.exr", "divc2.exr",
+           "invert.tif",
            "chsum.tif", "grid-clamped.tif",
            "rangecompress.tif", "rangeexpand.tif",
            "resize.tif", "resample.tif",
@@ -49,6 +60,7 @@ for f in [ "black.tif", "filled.tif", "checker.tif",
            "box3.exr",
            "a_over_b.exr",
            "tahoe-small.tx",
+           "text.tif",
          ] :
     command += checkref (f)
 
