@@ -1,4 +1,4 @@
-#!/usr/bin/python 
+#!/usr/bin/env python
 
 import os
 import glob
@@ -47,8 +47,11 @@ command = ""
 outputs = [ "out.txt" ]    # default
 failureok = 0
 failthresh = 0.004
+hardfail = 0.012
 failpercent = 0.02
 
+image_extensions = [ ".tif", ".exr", ".jpg", ".png", ".rla" ]
+
 #print ("srcdir = " + srcdir)
 #print ("tmpdir = " + tmpdir)
 #print ("path = " + path)
@@ -83,6 +86,10 @@ def text_diff (fromfile, tofile, diff_file=None):
     if diff_file:
         try:
             open (diff_file, 'w').writelines (diff_lines)
+
+            print ("Diff " + fromfile + " vs " + tofile + " was:\n-------")
+#            print (diff)
+            print ("".join(diff_lines))
         except:
             print ("Unexpected error:", sys.exc_info()[0])
     return 1
@@ -122,10 +129,11 @@ def info_command (file, extraargs="", safematch=0, hash=True) :
 # compilers always match to every last floating point bit.
 def diff_command (fileA, fileB, extraargs="", silent=False, concat=True) :
     command = (oiio_app("idiff") + "-a"
-               + " -fail " + str(failthresh/4)
+               + " -fail " + str(failthresh)
                + " -failpercent " + str(failpercent)
-               + " -hardfail " + str(failthresh)
+               + " -hardfail " + str(hardfail)
                + " -warn " + str(2*failthresh)
+               + " -warnpercent " + str(failpercent)
                + " " + extraargs + " " + oiio_relpath(fileA,tmpdir) 
                + " " + oiio_relpath(fileB,tmpdir))
     if not silent :
@@ -173,6 +181,10 @@ def rw_command (dir, filename, testwrite=1, use_oiiotool=0, extraargs="",
             cmd = (cmd + oiio_app("iconvert") + preargs + " " + fn
                    + " " + extraargs + " " + filename + " >> out.txt ;\n")
         cmd = (cmd + oiio_app("idiff") + " -a " + fn
+               + " -fail " + str(failthresh)
+               + " -failpercent " + str(failpercent)
+               + " -hardfail " + str(hardfail)
+               + " -warn " + str(2*failthresh)
                + " " + idiffextraargs + " " + filename + " >> out.txt ;\n")
     return cmd
 
@@ -200,6 +212,7 @@ def oiiotool (args, silent=False, concat=True) :
 # to pass.  If any outputs do not match their references return 1 to
 # fail.
 def runtest (command, outputs, failureok=0) :
+    err = 0
 #    print ("working dir = " + tmpdir)
     os.chdir (srcdir)
     open ("out.txt", "w").close()    # truncate out.txt
@@ -223,9 +236,8 @@ def runtest (command, outputs, failureok=0) :
         if cmdret != 0 and failureok == 0 :
             print ("#### Error: this command failed: ", sub_command)
             print ("FAIL")
-            return (1)
+            err = 1
 
-    err = 0
     for out in outputs :
         extension = os.path.splitext(out)[1]
         ok = 0
@@ -235,9 +247,10 @@ def runtest (command, outputs, failureok=0) :
         # variants for different platforms, etc.
         for testfile in (["ref/"+out] + glob.glob (os.path.join ("ref", "*"+extension))) :
             # print ("comparing " + out + " to " + testfile)
-            if extension == ".tif" or extension == ".exr" or extension == ".jpg" or extension == ".png":
+            if extension in image_extensions :
+                print ("Testing " + testfile)
                 # images -- use idiff
-                cmpcommand = diff_command (out, testfile, concat=False)
+                cmpcommand = diff_command (out, testfile, concat=False, silent=True)
                 # print ("cmpcommand = " + cmpcommand)
                 cmpresult = os.system (cmpcommand)
             elif extension == ".txt" :
@@ -246,14 +259,29 @@ def runtest (command, outputs, failureok=0) :
                 # anything else
                 cmpresult = 0 if filecmp.cmp (out, testfile) else 1
             if cmpresult == 0 :
-                print ("PASS: " + out + " matches " + testfile)
                 ok = 1
                 break      # we're done
 
-        if ok == 0:
+        if ok :
+            if extension in image_extensions :
+                # If we got a match for an image, save the idiff results
+                os.system (diff_command (out, testfile, silent=False))
+            print ("PASS: " + out + " matches " + testfile)
+        else :
             err = 1
             print ("NO MATCH for " + out)
             print ("FAIL " + out)
+            if extension == ".txt" :
+                # If we failed to get a match for a text file, print the
+                # file and the diff, for easy debugging.
+                print ("-----" + out + "----->")
+                print (open(out,'r').read() + "<----------")
+                print ("Diff was:\n-------")
+                print (open (out+".diff", 'rU').read())
+            if extension in image_extensions :
+                # If we failed to get a match for an image, send the idiff
+                # results to the console
+                os.system (diff_command (out, os.path.join ("ref", out), silent=False))
 
     return (err)
 
@@ -270,6 +298,15 @@ with open("run.py") as f:
     code = compile(f.read(), "run.py", 'exec')
     exec (code)
 
+# Allow a little more slop for slight pixel differences when in DEBUG
+# mode or when running on remote Travis-CI machines.
+if (("TRAVIS" in os.environ and os.environ["TRAVIS"]) or
+    ("DEBUG" in os.environ and os.environ["DEBUG"])) :
+    failthresh *= 2.0
+    hardfail *= 2.0
+    failpercent *= 2.0
+
+
 # Run the test and check the outputs
-ret = runtest (command, outputs)
+ret = runtest (command, outputs, failureok=failureok)
 sys.exit (ret)
